@@ -177,7 +177,7 @@ def is_admin():
     """Check if the script is running with administrator privileges"""
     try:
         return ctypes.windll.shell32.IsUserAnAdmin() != 0
-    except:
+    except Exception:
         return False
 
 def install_service():
