@@ -18,7 +18,7 @@ def is_admin():
     """Check if the script is running with administrator privileges"""
     try:
         return ctypes.windll.shell32.IsUserAnAdmin() != 0
-    except:
+    except Exception:
         return False
 
 def setup_environment():
@@ -110,8 +110,8 @@ def check_requirements():
     # Check Python version
     python_version = sys.version_info
     print(f"Python version: {python_version.major}.{python_version.minor}.{python_version.micro}")
-    if python_version.major < 3 or (python_version.major == 3 and python_version.minor < 9):
-        print("WARNING: Huntarr requires Python 3.9 or higher")
+    if python_version.major < 3 or (python_version.major == 3 and python_version.minor < 12):
+        print("WARNING: Huntarr requires Python 3.12 or higher")
         requirements_met = False
     
     # Check for pywin32
@@ -141,7 +141,7 @@ def check_requirements():
             requirements_met = False
         else:
             print("Port 9705 is available")
-    except:
+    except OSError:
         print("Could not check port availability")
     
     return requirements_met
