@@ -11,7 +11,6 @@ import signal
 import logging # Use standard logging for initial setup
 import atexit
 import time
-import time
 
 # Import path configuration early to set up environment
 try:
@@ -110,7 +109,6 @@ try:
     from primary.background import start_huntarr, stop_event, shutdown_threads
     # Configure logging first
     import logging
-    sys.path.insert(0, os.path.join(os.path.dirname(__file__), "src"))
     from primary.utils.logger import setup_main_logger, get_logger
     from primary.utils.clean_logger import setup_clean_logging
     
@@ -230,14 +228,19 @@ def run_web_server():
     web_logger = get_logger("WebServer") # Use app's logger
     debug_mode = os.environ.get('DEBUG', 'false').lower() == 'true'
     host = os.environ.get('FLASK_HOST', '0.0.0.0')
-    port = int(os.environ.get('HUNTARR_PORT', os.environ.get('PORT', 9705))) # Check HUNTARR_PORT first, then PORT, then default
+    try:
+        port = int(os.environ.get('HUNTARR_PORT', os.environ.get('PORT', 9705)))
+    except (TypeError, ValueError):
+        port = 9705
 
     web_logger.info(f"Starting web server on {host}:{port} (Debug: {debug_mode})...")
+    
+    # TODO: System tray implementation temporarily disabled
+    # Will be re-enabled in a future update after thorough testing
+    # For now, console=False in PyInstaller spec provides silent background operation
 
     # Log the current authentication mode once at startup
     try:
-        import sys
-        sys.path.insert(0, os.path.join(os.path.dirname(__file__), "src"))
         from primary.settings_manager import load_settings
         
         settings = load_settings("general")
@@ -272,8 +275,18 @@ def run_web_server():
             import time
             web_logger.info("Running with Waitress production server.")
             
+            # Read configurable web server thread count from settings
+            web_threads = 32  # default
+            try:
+                from primary.settings_manager import load_settings
+                general_settings = load_settings('general')
+                web_threads = max(8, min(int(general_settings.get('web_server_threads', 32)), 96))
+            except Exception:
+                pass
+            web_logger.info(f"Waitress configured with {web_threads} worker threads")
+            
             # Create the server instance so we can shut it down gracefully
-            waitress_server = create_server(app, host=host, port=port, threads=8)
+            waitress_server = create_server(app, host=host, port=port, threads=web_threads)
             
             web_logger.info("Waitress server starting...")
             
@@ -296,10 +309,10 @@ def run_web_server():
                 waitress_server.close()
                 web_logger.info("Waitress server close() called.")
                 
-                # Wait for server thread to finish
-                server_thread.join(timeout=3.0)
+                # Wait for server thread to finish with longer timeout
+                server_thread.join(timeout=5.0)
                 if server_thread.is_alive():
-                    web_logger.warning("Server thread did not stop within timeout.")
+                    web_logger.debug("Server thread did not stop within timeout (this is normal during container restart).")
                 else:
                     web_logger.info("Server thread stopped successfully.")
                     
@@ -324,11 +337,33 @@ def run_web_server():
             if not stop_event.is_set():
                 stop_event.set()
 
+def sigchld_handler(signum, frame):
+    """Handle SIGCHLD to prevent zombie processes. (Unix only)"""
+    # This handler is only registered on Unix-like systems
+    # Windows doesn't have SIGCHLD or os.waitpid with WNOHANG
+    if not hasattr(os, 'waitpid'):
+        return
+    
+    # Reap all terminated child processes without blocking
+    while True:
+        try:
+            # WNOHANG returns immediately if no child has exited
+            pid, status = os.waitpid(-1, os.WNOHANG)
+            if pid == 0:
+                # No more zombie children
+                break
+        except ChildProcessError:
+            # No child processes
+            break
+        except OSError:
+            # Error occurred, stop trying
+            break
+
 def main_shutdown_handler(signum, frame):
     """Gracefully shut down the application."""
     global _global_shutdown_flag
     _global_shutdown_flag = True  # Set global shutdown flag immediately
-    
+
     signal_name = "SIGINT" if signum == signal.SIGINT else "SIGTERM" if signum == signal.SIGTERM else f"Signal {signum}"
     huntarr_logger.info(f"Received {signal_name}. Initiating graceful shutdown...")
     
@@ -342,21 +377,21 @@ def main_shutdown_handler(signum, frame):
         
         huntarr_logger.info("Performing emergency database checkpoint...")
         
-        # Emergency checkpoint for main database
+        # Emergency checkpoint for main database — use TRUNCATE to fully merge WAL
         try:
             main_db = get_database()
             with main_db.get_connection() as conn:
-                conn.execute("PRAGMA wal_checkpoint(RESTART)")
-                huntarr_logger.info("Main database emergency checkpoint completed")
+                conn.execute("PRAGMA wal_checkpoint(TRUNCATE)")
+                huntarr_logger.info("Main database emergency checkpoint (TRUNCATE) completed")
         except Exception as e:
             huntarr_logger.warning(f"Main database emergency checkpoint failed: {e}")
         
-        # Emergency checkpoint for logs database
+        # Emergency checkpoint for logs database — use TRUNCATE to fully merge WAL
         try:
             logs_db = get_logs_database()
             with logs_db.get_logs_connection() as conn:
-                conn.execute("PRAGMA wal_checkpoint(RESTART)")
-                huntarr_logger.info("Logs database emergency checkpoint completed")
+                conn.execute("PRAGMA wal_checkpoint(TRUNCATE)")
+                huntarr_logger.info("Logs database emergency checkpoint (TRUNCATE) completed")
         except Exception as e:
             huntarr_logger.warning(f"Logs database emergency checkpoint failed: {e}")
             
@@ -438,9 +473,20 @@ def main():
     This function is called by app_launcher.py in the packaged ARM application.
     """
     # Register signal handlers for graceful shutdown in the main process
-    signal.signal(signal.SIGINT, main_shutdown_handler)
-    signal.signal(signal.SIGTERM, main_shutdown_handler)
-    
+    # Signal handlers can only be registered in the main thread
+    try:
+        signal.signal(signal.SIGINT, main_shutdown_handler)
+        signal.signal(signal.SIGTERM, main_shutdown_handler)
+
+        # Register SIGCHLD handler to prevent zombie processes (Docker healthchecks)
+        # SIGCHLD is not available on Windows, only register on Unix-like systems
+        if hasattr(signal, 'SIGCHLD'):
+            signal.signal(signal.SIGCHLD, sigchld_handler)
+    except ValueError as e:
+        # Signal handlers can only be registered in the main thread
+        # This is expected when running in macOS menu bar mode where main() is called from a background thread
+        huntarr_logger.info(f"Signal handlers not registered (running in background thread): {e}")
+
     # Register cleanup handler
     atexit.register(cleanup_handler)
     
