@@ -326,24 +326,6 @@ def get_quality_upgrades(api_url: str, api_key: str, api_timeout: int, monitored
         eros_logger.error(f"Error retrieving quality upgrade items: {str(e)}")
         return None
 
-def refresh_item(api_url: str, api_key: str, api_timeout: int, item_id: int) -> int:
-    """
-    Refresh functionality has been removed as it was a performance bottleneck.
-    This function now returns a placeholder command ID without making any API calls.
-    
-    Args:
-        api_url: The base URL of the Whisparr V3 API
-        api_key: The API key for authentication
-        api_timeout: Timeout for the API request
-        item_id: The ID of the movie to refresh
-        
-    Returns:
-        A placeholder command ID (123) to simulate success
-    """
-    eros_logger.debug(f"Refresh functionality disabled for movie ID: {item_id}")
-    # Return a placeholder command ID to simulate success without actually refreshing
-    return 123
-
 def item_search(api_url: str, api_key: str, api_timeout: int, item_ids: List[int]) -> int:
     """
     Trigger a search for one or more movies in Whisparr V3.
@@ -485,6 +467,39 @@ def check_connection(api_url: str, api_key: str, api_timeout: int) -> bool:
         eros_logger.error(f"Error checking connection to Whisparr V3 API: {str(e)}")
         return False
 
+def get_tag_id_by_label(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
+    """Get tag ID by label (does not create). Used for exempt tags (issue #676)."""
+    try:
+        response = arr_request(api_url, api_key, api_timeout, "tag", count_api=False)
+        if response:
+            for tag in response:
+                if tag.get('label') == tag_label:
+                    return tag.get('id')
+        return None
+    except Exception as e:
+        eros_logger.error(f"Error getting tag '{tag_label}': {e}")
+        return None
+
+
+def get_exempt_tag_ids(api_url: str, api_key: str, api_timeout: int, exempt_tag_labels: list) -> dict:
+    """Resolve exempt tag labels to tag IDs. Returns dict tag_id -> label. Exact match. Issue #676.
+    Only counts tags that are actually entered (non-empty); empty list or all-whitespace = no exclusions.
+    """
+    if exempt_tag_labels is None:
+        return {}
+    if isinstance(exempt_tag_labels, str):
+        exempt_tag_labels = [exempt_tag_labels]
+    labels = [str(l).strip() for l in (exempt_tag_labels or []) if l is not None and str(l).strip()]
+    if not labels:
+        return {}
+    result = {}
+    for label in labels:
+        tid = get_tag_id_by_label(api_url, api_key, api_timeout, label)
+        if tid is not None:
+            result[tid] = label
+    return result
+
+
 def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
     """
     Get existing tag ID or create a new tag in Eros.
