@@ -13,10 +13,14 @@ from typing import List, Dict, Any, Set, Callable
 from src.primary.utils.logger import get_logger
 from src.primary.apps.eros import api as eros_api
 from src.primary.settings_manager import load_settings, get_advanced_setting
-from src.primary.stateful_manager import is_processed, add_processed_id
-from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
+from src.primary.stateful_manager import add_processed_id
+from src.primary.stats_manager import increment_stat
 from src.primary.utils.history_utils import log_processed_media
 from src.primary.state import check_state_reset
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items, filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item
 
 # Get logger for the app
 eros_logger = get_logger("eros")
@@ -41,42 +45,28 @@ def process_missing_items(
     # Reset state files if enough time has passed
     check_state_reset("eros")
     
-    # Load settings to check if tagging is enabled
-    eros_settings = load_settings("eros")
-    tag_processed_items = eros_settings.get("tag_processed_items", True)
+    # Extract common settings using shared utility
+    # Eros uses hunt_missing_items with fallback to hunt_missing_scenes
+    hunt_key_value = app_settings.get("hunt_missing_items", app_settings.get("hunt_missing_scenes", 0))
+    app_settings["hunt_missing_items"] = hunt_key_value  # Normalize for extract
+    s = extract_app_settings(app_settings, "eros", "hunt_missing_items", "Eros Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_missing_items = s['hunt_count']
+    tag_settings = s['tag_settings']
     
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    instance_name = app_settings.get("instance_name", "Eros Default")
-    
-    # Load general settings to get centralized timeout
-    general_settings = load_settings('general')
-    
-    monitored_only = app_settings.get("monitored_only", True)
+    # App-specific settings
     skip_future_releases = app_settings.get("skip_future_releases", True)
-    # skip_item_refresh setting removed as it was a performance bottleneck
-    search_mode = app_settings.get("search_mode", "movie")  # Default to movie mode if not specified
+    search_mode = app_settings.get("search_mode", "movie")
     
     eros_logger.info(f"Using search mode: {search_mode} for missing items")
-    
-    # Use the new hunt_missing_items parameter name, falling back to hunt_missing_scenes for backwards compatibility
-    hunt_missing_items = app_settings.get("hunt_missing_items", app_settings.get("hunt_missing_scenes", 0))
-    
-    # Use advanced settings from database for command operations
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-    
-    # Use the centralized advanced setting for stateful management hours
-    stateful_management_hours = get_advanced_setting("stateful_management_hours", 168)
-    
-    # Log that we're using Eros v3 API
     eros_logger.debug(f"Using Eros API v3 for instance: {instance_name}")
 
-    # Skip if hunt_missing_items is set to a negative value or 0
-    if hunt_missing_items <= 0:
-        eros_logger.info("'hunt_missing_items' setting is 0 or less. Skipping missing item processing.")
+    if not validate_settings(api_url, api_key, hunt_missing_items, "eros", eros_logger):
         return False
 
     # Check for stop signal
@@ -102,7 +92,17 @@ def process_missing_items(
         return False
     
     eros_logger.info(f"Found {len(missing_items)} items with missing files.")
-    
+
+    # Filter out items with exempt tags (issue #676)
+    missing_items = filter_exempt_items(
+        missing_items, s['exempt_tags'], eros_api,
+        api_url, api_key, api_timeout,
+        get_tags_fn=lambda item: item.get("tags", []),
+        get_id_fn=lambda item: item.get("id"),
+        get_title_fn=lambda item: item.get("title", "Unknown"),
+        app_type="eros", logger=eros_logger
+    )
+
     # Filter out future releases if configured
     if skip_future_releases:
         now = datetime.datetime.now().replace(tzinfo=datetime.timezone.utc)
@@ -123,15 +123,11 @@ def process_missing_items(
         eros_logger.info("No missing items left to process after filtering future releases.")
         return False
         
-    # Filter out already processed items using stateful management
-    unprocessed_items = []
-    for item in missing_items:
-        item_id = str(item.get("id"))
-        if not is_processed("eros", instance_name, item_id):
-            unprocessed_items.append(item)
-        else:
-            eros_logger.debug(f"Skipping already processed item ID: {item_id}")
-    
+    # Filter out already processed items using shared utility
+    unprocessed_items = filter_unprocessed(
+        missing_items, "eros", instance_key,
+        get_id_fn=lambda item: item.get("id"), logger=eros_logger
+    )
     eros_logger.info(f"Found {len(unprocessed_items)} unprocessed items out of {len(missing_items)} total items with missing files.")
     
     if not unprocessed_items:
@@ -149,20 +145,9 @@ def process_missing_items(
 
     # Process selected items
     for item in items_to_search:
-        # Check for stop signal before each item
-        if stop_check():
-            eros_logger.info("Stop requested during item processing. Aborting...")
+        if not should_continue_processing("eros", stop_check, eros_logger):
             break
         
-        # Check API limit before processing each item
-        try:
-            if check_hourly_cap_exceeded("eros"):
-                eros_logger.warning(f"🛑 Eros API hourly limit reached - stopping missing items processing after {items_processed} items")
-                break
-        except Exception as e:
-            eros_logger.error(f"Error checking hourly API cap: {e}")
-            # Continue processing if cap check fails - safer than stopping
-        
         # Re-check limit in case it changed
         current_limit = app_settings.get("hunt_missing_items", app_settings.get("hunt_missing_scenes", 1))
         if items_processed >= current_limit:
@@ -188,7 +173,7 @@ def process_missing_items(
         eros_logger.info(f"Processing missing item: \"{item_info}\" (Item ID: {item_id})")
         
         # Mark the item as processed BEFORE triggering any searches
-        add_processed_id("eros", instance_name, str(item_id))
+        add_processed_id("eros", instance_key, str(item_id))
         eros_logger.debug(f"Added item ID {item_id} to processed list for {instance_name}")
         
         # Refresh functionality has been removed as it was identified as a performance bottleneck
@@ -204,25 +189,20 @@ def process_missing_items(
         if search_command_id:
             eros_logger.info(f"Triggered search command {search_command_id}. Assuming success for now.")
             
-            # Tag the movie if enabled
-            if tag_processed_items:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("eros", "missing", "huntarr-missing")
-                try:
-                    eros_api.tag_processed_movie(api_url, api_key, api_timeout, item_id, custom_tag)
-                    eros_logger.debug(f"Tagged movie {item_id} with '{custom_tag}'")
-                except Exception as e:
-                    eros_logger.warning(f"Failed to tag movie {item_id} with '{custom_tag}': {e}")
+            # Tag the movie if enabled (unified tagging)
+            try_tag_item(tag_settings, "missing", eros_api.tag_processed_movie,
+                         api_url, api_key, api_timeout, item_id,
+                         eros_logger, f"movie {item_id}")
             
             # Log to history system
-            log_processed_media("eros", item_info, item_id, instance_name, "missing")
+            log_processed_media("eros", item_info, item_id, instance_key, "missing", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             eros_logger.debug(f"Logged history entry for item: {item_info}")
             
             items_processed += 1
             processing_done = True
             
             # Increment the hunted statistics for Eros
-            increment_stat("eros", "hunted", 1)
+            increment_stat("eros", "hunted", 1, instance_key)
             eros_logger.debug(f"Incremented eros hunted statistics by 1")
 
             # Log progress
