@@ -13,10 +13,14 @@ from typing import List, Dict, Any, Set, Callable
 from src.primary.utils.logger import get_logger
 from src.primary.apps.eros import api as eros_api
 from src.primary.settings_manager import load_settings, get_advanced_setting
-from src.primary.stateful_manager import is_processed, add_processed_id
-from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
+from src.primary.stateful_manager import add_processed_id
+from src.primary.stats_manager import increment_stat
 from src.primary.utils.history_utils import log_processed_media
 from src.primary.state import check_state_reset
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items, filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item, extract_tag_settings
 
 # Get logger for the app
 eros_logger = get_logger("eros")
@@ -41,39 +45,27 @@ def process_cutoff_upgrades(
     # Reset state files if enough time has passed
     check_state_reset("eros")
     
-    # Load settings to check if tagging is enabled
-    eros_settings = load_settings("eros")
-    tag_processed_items = eros_settings.get("tag_processed_items", True)
+    # Extract common settings using shared utility
+    # Eros uses hunt_upgrade_items with fallback to hunt_upgrade_scenes
+    hunt_key_value = app_settings.get("hunt_upgrade_items", app_settings.get("hunt_upgrade_scenes", 0))
+    app_settings["hunt_upgrade_items"] = hunt_key_value  # Normalize for extract
+    s = extract_app_settings(app_settings, "eros", "hunt_upgrade_items", "Eros Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_upgrade_items = s['hunt_count']
+    tag_settings = extract_tag_settings(app_settings)
     
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    instance_name = app_settings.get("instance_name", "Eros Default")
-    
-    # Load general settings to get centralized timeout
-    general_settings = load_settings('general')
-    
-    monitored_only = app_settings.get("monitored_only", True)
-    # skip_item_refresh setting removed as it was a performance bottleneck
-    search_mode = app_settings.get("search_mode", "movie")  # Default to movie mode if not specified
+    # App-specific settings
+    search_mode = app_settings.get("search_mode", "movie")
     
     eros_logger.info(f"Using search mode: {search_mode} for quality upgrades")
-    
-    # Use the new hunt_upgrade_items parameter name, falling back to hunt_upgrade_scenes for backwards compatibility
-    hunt_upgrade_items = app_settings.get("hunt_upgrade_items", app_settings.get("hunt_upgrade_scenes", 0))
-    
-    # Use advanced settings from database for command operations
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-    state_reset_interval_hours = get_advanced_setting("stateful_management_hours", 168)  
-    
-    # Log that we're using Eros API v3
     eros_logger.debug(f"Using Eros API v3 for instance: {instance_name}")
 
-    # Skip if hunt_upgrade_items is set to 0
-    if hunt_upgrade_items <= 0:
-        eros_logger.info("'hunt_upgrade_items' setting is 0 or less. Skipping quality upgrade processing.")
+    if not validate_settings(api_url, api_key, hunt_upgrade_items, "eros", eros_logger):
         return False
 
     # Check for stop signal
@@ -95,16 +87,22 @@ def process_cutoff_upgrades(
         return False
         
     eros_logger.info(f"Found {len(upgrade_eligible_data)} items eligible for quality upgrade.")
-    
-    # Filter out already processed items using stateful management
-    unprocessed_items = []
-    for item in upgrade_eligible_data:
-        item_id = str(item.get("id"))
-        if not is_processed("eros", instance_name, item_id):
-            unprocessed_items.append(item)
-        else:
-            eros_logger.debug(f"Skipping already processed item ID: {item_id}")
-    
+
+    # Filter out items with exempt tags (issue #676)
+    upgrade_eligible_data = filter_exempt_items(
+        upgrade_eligible_data, s['exempt_tags'], eros_api,
+        api_url, api_key, api_timeout,
+        get_tags_fn=lambda item: item.get("tags", []),
+        get_id_fn=lambda item: item.get("id"),
+        get_title_fn=lambda item: item.get("title", "Unknown"),
+        app_type="eros", logger=eros_logger
+    )
+
+    # Filter out already processed items using shared utility
+    unprocessed_items = filter_unprocessed(
+        upgrade_eligible_data, "eros", instance_key,
+        get_id_fn=lambda item: item.get("id"), logger=eros_logger
+    )
     eros_logger.info(f"Found {len(unprocessed_items)} unprocessed items out of {len(upgrade_eligible_data)} total items eligible for quality upgrade.")
     
     if not unprocessed_items:
@@ -122,19 +120,8 @@ def process_cutoff_upgrades(
     
     # Process selected items
     for item in items_to_upgrade:
-        # Check for stop signal before each item
-        if stop_check():
-            eros_logger.info("Stop requested during item processing. Aborting...")
+        if not should_continue_processing("eros", stop_check, eros_logger):
             break
-        
-        # Check API limit before processing each item
-        try:
-            if check_hourly_cap_exceeded("eros"):
-                eros_logger.warning(f"🛑 Eros API hourly limit reached - stopping upgrade processing after {items_processed} items")
-                break
-        except Exception as e:
-            eros_logger.error(f"Error checking hourly API cap: {e}")
-            # Continue processing if cap check fails - safer than stopping
             
         # Re-check limit in case it changed
         current_limit = app_settings.get("hunt_upgrade_items", app_settings.get("hunt_upgrade_scenes", 1))
@@ -184,25 +171,20 @@ def process_cutoff_upgrades(
         if search_command_id:
             eros_logger.info(f"Triggered search command {search_command_id}. Assuming success for now.")
             
-            # Tag the movie if enabled
-            if tag_processed_items:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("eros", "upgrade", "huntarr-upgraded")
-                try:
-                    eros_api.tag_processed_movie(api_url, api_key, api_timeout, item_id, custom_tag)
-                    eros_logger.debug(f"Tagged movie {item_id} with '{custom_tag}'")
-                except Exception as e:
-                    eros_logger.warning(f"Failed to tag movie {item_id} with '{custom_tag}': {e}")
+            # Tag the movie if enabled (unified tagging)
+            try_tag_item(tag_settings, "upgraded", eros_api.tag_processed_movie,
+                         api_url, api_key, api_timeout, item_id,
+                         eros_logger, f"movie {item_id}")
             
             # Log to history so the upgrade appears in the history UI
-            log_processed_media("eros", item_info, item_id, instance_name, "upgrade")
+            log_processed_media("eros", item_info, item_id, instance_key, "upgrade", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             eros_logger.debug(f"Logged quality upgrade to history for item ID {item_id}")
             
             items_processed += 1
             processing_done = True
             
             # Increment the upgraded statistics for Eros
-            increment_stat("eros", "upgraded", 1)
+            increment_stat("eros", "upgraded", 1, instance_key)
             eros_logger.debug(f"Incremented eros upgraded statistics by 1")
             
             # Log progress
