@@ -172,33 +172,8 @@ def get_system_status(api_url: str, api_key: str, api_timeout: int, verify_ssl:
 
 def check_connection(api_url: str, api_key: str, api_timeout: int) -> bool:
     """Checks connection by fetching system status."""
-    if not api_url:
-        lidarr_logger.error("API URL is empty or not set")
-        return False
-    if not api_key:
-        lidarr_logger.error("API Key is empty or not set")
-        return False
-
-    try:
-        # Use a shorter timeout for a quick connection check
-        quick_timeout = min(api_timeout, 15) 
-        
-        # Get SSL verification setting
-        verify_ssl = get_ssl_verify_setting()
-        
-        status = get_system_status(api_url, api_key, quick_timeout, verify_ssl)
-        if status and isinstance(status, dict) and 'version' in status:
-             # Log success only if debug is enabled to avoid clutter
-             lidarr_logger.debug(f"Connection check successful for {api_url}. Version: {status.get('version')}")
-             return True
-        else:
-             # Log details if the status response was unexpected
-             lidarr_logger.warning(f"Connection check for {api_url} returned unexpected status: {str(status)[:200]}")
-             return False
-    except Exception as e:
-        # Error should have been logged by arr_request, just indicate failure
-        lidarr_logger.error(f"Connection check failed for {api_url}: {str(e)}")
-        return False
+    from src.primary.apps._common.arr_api import check_connection as _check
+    return _check(api_url, api_key, api_timeout, "lidarr", lidarr_logger, api_version="v1")
 
 def get_artists(api_url: str, api_key: str, api_timeout: int, artist_id: Optional[int] = None) -> Union[List, Dict, None]:
     """Get artist information from Lidarr."""
@@ -228,7 +203,7 @@ def get_tracks(api_url: str, api_key: str, api_timeout: int, album_id: Optional[
 
 def get_queue(api_url: str, api_key: str, api_timeout: int) -> List:
     """Get the current queue from Lidarr (handles pagination)."""
-    # Lidarr v1 queue endpoint supports pagination, unlike Sonarr v3's simple list
+    # Lidarr v1 queue endpoint supports pagination
     all_records = []
     page = 1
     page_size = 1000 # Request large page size
@@ -434,20 +409,13 @@ def search_albums(api_url: str, api_key: str, api_timeout: int, album_ids: List[
         "name": "AlbumSearch",
         "albumIds": album_ids
     }
-    response = arr_request(api_url, api_key, api_timeout, "command", method="POST", data=payload)
+    # count_api=False: caller uses increment_stat so API bar = searches + upgrades (one count per action)
+    response = arr_request(api_url, api_key, api_timeout, "command", method="POST", data=payload, count_api=False)
     
     if response and isinstance(response, dict) and 'id' in response:
         command_id = response.get('id')
         lidarr_logger.info(f"Triggered Lidarr AlbumSearch for album IDs: {album_ids}. Command ID: {command_id}")
         
-        # Increment API counter after successful request
-        try:
-            from src.primary.stats_manager import increment_hourly_cap
-            increment_hourly_cap("lidarr", 1)
-            lidarr_logger.debug(f"Incremented Lidarr hourly API cap for album search ({len(album_ids)} albums)")
-        except Exception as cap_error:
-            lidarr_logger.error(f"Failed to increment hourly API cap for album search: {cap_error}")
-        
         return response # Return the full command object including ID
     else:
         lidarr_logger.error(f"Failed to trigger Lidarr AlbumSearch for album IDs {album_ids}. Response: {response}")
@@ -470,38 +438,17 @@ def search_artist(api_url: str, api_key: str, api_timeout: int, artist_id: int)
         "name": "ArtistSearch",
         "artistIds": [artist_id]
     }
-    response = arr_request(api_url, api_key, api_timeout, "command", method="POST", data=payload)
+    # count_api=False: caller uses increment_stat so API bar = searches + upgrades (one count per action)
+    response = arr_request(api_url, api_key, api_timeout, "command", method="POST", data=payload, count_api=False)
 
     if response and isinstance(response, dict) and 'id' in response:
-        command_id = response.get('id')
+        command_id = response.get("id")
         lidarr_logger.info(f"Triggered Lidarr ArtistSearch for artist ID: {artist_id}. Command ID: {command_id}")
-        
-        # Increment API counter after successful request
-        try:
-            from src.primary.stats_manager import increment_hourly_cap
-            increment_hourly_cap("lidarr", 1)
-            lidarr_logger.debug(f"Incremented Lidarr hourly API cap for artist search (artist {artist_id})")
-        except Exception as cap_error:
-            lidarr_logger.error(f"Failed to increment hourly API cap for artist search: {cap_error}")
-        
         return response # Return the full command object
     else:
         lidarr_logger.error(f"Failed to trigger Lidarr ArtistSearch for artist ID {artist_id}. Response: {response}")
         return None
 
-def refresh_artist(api_url: str, api_key: str, api_timeout: int, artist_id: int) -> Optional[Dict]:
-    """Refresh functionality has been removed as it was a performance bottleneck.
-    This function now returns a placeholder success value without making any API calls."""
-    lidarr_logger.debug(f"Refresh functionality disabled for artist ID: {artist_id}")
-    # Return a placeholder command object to simulate success
-    return {
-        'id': 123,
-        'name': 'RefreshArtist',
-        'status': 'completed',
-        'artistId': artist_id,
-        'message': 'Refresh functionality disabled for performance reasons'
-    }
-
 def get_command_status(api_url: str, api_key: str, api_timeout: int, command_id: int) -> Optional[Dict[str, Any]]:
     """Get the status of a Lidarr command."""
     response = arr_request(api_url, api_key, api_timeout, f"command/{command_id}")
@@ -516,6 +463,65 @@ def get_artist_by_id(api_url: str, api_key: str, api_timeout: int, artist_id: in
     """Get artist details by ID from Lidarr."""
     return arr_request(api_url, api_key, api_timeout, f"artist/{artist_id}")
 
+def get_tag_id_by_label(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
+    """Get tag ID by label (does not create). Used for tag-based upgrade selection."""
+    try:
+        response = arr_request(api_url, api_key, api_timeout, "tag", count_api=False)
+        if response:
+            for tag in response:
+                if tag.get('label') == tag_label:
+                    return tag.get('id')
+        return None
+    except Exception as e:
+        lidarr_logger.error(f"Error getting tag '{tag_label}': {e}")
+        return None
+
+
+def get_exempt_tag_ids(api_url: str, api_key: str, api_timeout: int, exempt_tag_labels: list) -> dict:
+    """Resolve exempt tag labels to tag IDs. Returns dict tag_id -> label. Exact match. Issue #676.
+    Only counts tags that are actually entered (non-empty); empty list or all-whitespace = no exclusions.
+    """
+    if exempt_tag_labels is None:
+        return {}
+    if isinstance(exempt_tag_labels, str):
+        exempt_tag_labels = [exempt_tag_labels]
+    labels = [str(l).strip() for l in (exempt_tag_labels or []) if l is not None and str(l).strip()]
+    if not labels:
+        return {}
+    result = {}
+    for label in labels:
+        tid = get_tag_id_by_label(api_url, api_key, api_timeout, label)
+        if tid is not None:
+            result[tid] = label
+    return result
+
+
+def get_albums_without_artist_tag(
+    api_url: str, api_key: str, api_timeout: int, tag_label: str, monitored_only: bool
+) -> Optional[List[Dict[str, Any]]]:
+    """
+    Get albums whose artist DON'T have the given tag (Upgradinatorr-style: tag tracks processed).
+    After searching, the tag will be added to the artist to mark them as processed.
+    """
+    tag_id = get_or_create_tag(api_url, api_key, api_timeout, tag_label)
+    if tag_id is None:
+        lidarr_logger.error(f"Failed to get or create tag '{tag_label}' in Lidarr.")
+        return None
+    albums = get_albums(api_url, api_key, api_timeout)
+    if albums is None:
+        lidarr_logger.error("Failed to retrieve albums from Lidarr API.")
+        return None
+    if not isinstance(albums, list):
+        albums = [albums]
+    out = [
+        a for a in albums
+        if tag_id not in (a.get('artist') or {}).get('tags', [])
+        and (not monitored_only or (a.get('monitored', False) and (a.get('artist') or {}).get('monitored', False)))
+    ]
+    lidarr_logger.debug(f"Found {len(out)} albums whose artists DON'T have tag '{tag_label}' (monitored_only={monitored_only}).")
+    return out
+
+
 def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
     """
     Get existing tag ID or create a new tag in Lidarr.
@@ -857,7 +863,7 @@ def get_quality_profiles(api_url: str, api_key: str, api_timeout: int) -> Option
         lidarr_logger.debug("Fetching quality profiles from Lidarr...")
         
         # Use the qualityProfile endpoint - this doesn't count toward API limits since it's configuration data
-        # Note: Lidarr uses API v1, not v3 like Sonarr/Radarr
+        # Note: Lidarr uses API v1, not v3
         profiles = arr_request(api_url, api_key, api_timeout, "qualityProfile", count_api=False)
         
         if profiles is None:
