@@ -17,6 +17,10 @@ from src.primary.stateful_manager import is_processed, add_processed_id
 from src.primary.utils.history_utils import log_processed_media
 from src.primary.settings_manager import load_settings, get_advanced_setting
 from src.primary.state import check_state_reset
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item
 
 # Get the logger for the Lidarr module
 lidarr_logger = get_logger(__name__) # Use __name__ for correct logger hierarchy
@@ -37,25 +41,23 @@ def process_missing_albums(
         bool: True if any items were processed, False otherwise.
     """
     
-    # Copy instance-specific information
-    instance_name = app_settings.get("instance_name", "Default")
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    monitored_only = app_settings.get("monitored_only", True)
-    skip_future_releases = app_settings.get("skip_future_releases", False)
-    hunt_missing_items = app_settings.get("hunt_missing_items", 0)
+    # Extract common settings using shared utility
+    s = extract_app_settings(app_settings, "lidarr", "hunt_missing_items", "Lidarr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_missing_items = s['hunt_count']
+    command_wait_delay = s['command_wait_delay']
+    command_wait_attempts = s['command_wait_attempts']
+    
+    # App-specific settings
     hunt_missing_mode = app_settings.get("hunt_missing_mode", "album")
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
     
     # Early exit for disabled features
-    if not api_url or not api_key:
-        lidarr_logger.warning(f"Missing API URL or API key, skipping missing processing for {instance_name}")
-        return False
-    
-    if hunt_missing_items <= 0:
-        lidarr_logger.debug(f"Hunting for missing items is disabled (hunt_missing_items={hunt_missing_items}) for {instance_name}")
+    if not validate_settings(api_url, api_key, hunt_missing_items, "lidarr", lidarr_logger):
         return False
     
     # Make sure any requested stop function is executable
@@ -73,9 +75,8 @@ def process_missing_albums(
     processed_artists_or_albums = set()
     total_items_to_process = hunt_missing_items
     
-    # Load settings to check if tagging is enabled
-    lidarr_settings = load_settings("lidarr")
-    tag_processed_items = lidarr_settings.get("tag_processed_items", True)
+    # Per-instance tagging (from instance settings)
+    tag_settings = s['tag_settings']
 
     try:
         # Get missing albums or artists data based on the hunt_missing_mode
@@ -95,12 +96,22 @@ def process_missing_albums(
                 return False
             
             lidarr_logger.info(f"Retrieved {len(missing_albums_data)} missing albums from random page selection.")
+
+            # Filter out albums whose artist has an exempt tag (issue #676)
+            missing_albums_data = filter_exempt_items(
+                missing_albums_data, s['exempt_tags'], lidarr_api,
+                api_url, api_key, api_timeout,
+                get_tags_fn=lambda a: (a.get("artist") or {}).get("tags", []),
+                get_id_fn=lambda a: a.get("id"),
+                get_title_fn=lambda a: a.get("title", "Unknown"),
+                app_type="lidarr", logger=lidarr_logger
+            )
             
             # Convert to the expected format for album processing - keep IDs as integers
             unprocessed_entities = []
             for album in missing_albums_data:
                 album_id = album.get("id")  # Keep as integer, don't convert to string
-                if album_id and not is_processed("lidarr", instance_name, str(album_id)):  # Convert to string only for processed check
+                if album_id and not is_processed("lidarr", instance_key, str(album_id)):  # Convert to string only for processed check
                     unprocessed_entities.append(album_id)
             
             search_entity_type = "album"
@@ -120,6 +131,16 @@ def process_missing_albums(
             
             lidarr_logger.info(f"Retrieved {len(missing_albums_data)} missing albums.")
 
+            # Filter out albums whose artist has an exempt tag (issue #676)
+            missing_albums_data = filter_exempt_items(
+                missing_albums_data, s['exempt_tags'], lidarr_api,
+                api_url, api_key, api_timeout,
+                get_tags_fn=lambda a: (a.get("artist") or {}).get("tags", []),
+                get_id_fn=lambda a: a.get("id"),
+                get_title_fn=lambda a: a.get("title", "Unknown"),
+                app_type="lidarr", logger=lidarr_logger
+            )
+
             # Group by artist ID
             items_by_artist = {}
             for item in missing_albums_data: # Use the potentially filtered missing_items list
@@ -137,7 +158,7 @@ def process_missing_albums(
             # Filter out already processed artists
             lidarr_logger.info(f"Found {len(target_entities)} artists with missing albums before filtering")
             unprocessed_entities = [eid for eid in target_entities 
-                                   if not is_processed("lidarr", instance_name, str(eid))]
+                                   if not is_processed("lidarr", instance_key, str(eid))]
             
             lidarr_logger.info(f"Found {len(unprocessed_entities)} unprocessed artists out of {len(target_entities)} total")
             search_entity_type = "artist"
@@ -199,18 +220,8 @@ def process_missing_albums(
                 
             lidarr_logger.info(f"Triggering Artist Search for {len(entities_to_search_ids)} artists on {instance_name}...")
             for i, artist_id in enumerate(entities_to_search_ids):
-                if stop_check(): # Use the new stop_check function
-                    lidarr_logger.warning("Shutdown requested during artist search trigger.")
+                if not should_continue_processing("lidarr", stop_check, lidarr_logger):
                     break
-                
-                # Check API limit before processing each artist
-                try:
-                    if check_hourly_cap_exceeded("lidarr"):
-                        lidarr_logger.warning(f"🛑 Lidarr API hourly limit reached - stopping artist processing after {processed_count} artists")
-                        break
-                except Exception as e:
-                    lidarr_logger.error(f"Error checking hourly API cap: {e}")
-                    # Continue processing if cap check fails - safer than stopping
 
                 # Get artist name from cached details or first album
                 artist_name = f"Artist ID {artist_id}" # Default if name not found
@@ -225,7 +236,7 @@ def process_missing_albums(
                          artist_name = artist_info.get('artistName', artist_name)
                 
                 # Mark the artist as processed right away - BEFORE triggering the search
-                success = add_processed_id("lidarr", instance_name, str(artist_id))
+                success = add_processed_id("lidarr", instance_key, str(artist_id))
                 lidarr_logger.debug(f"Added artist ID {artist_id} to processed list for {instance_name}, success: {success}")
                 
                 # Trigger the search AFTER marking as processed
@@ -235,30 +246,25 @@ def process_missing_albums(
                 
                 # Increment stats for UI tracking
                 if command_result:
-                    increment_stat("lidarr", "hunted")
+                    increment_stat("lidarr", "hunted", 1, instance_key)
                     processed_count += 1  # Count successful searches
                     processed_artists_or_albums.add(artist_id)
                 
-                    # Tag the artist if enabled
-                    if tag_processed_items:
-                        from src.primary.settings_manager import get_custom_tag
-                        custom_tag = get_custom_tag("lidarr", "missing", "huntarr-missing")
-                        try:
-                            lidarr_api.tag_processed_artist(api_url, api_key, api_timeout, artist_id, custom_tag)
-                            lidarr_logger.debug(f"Tagged artist {artist_id} with '{custom_tag}'")
-                        except Exception as e:
-                            lidarr_logger.warning(f"Failed to tag artist {artist_id} with '{custom_tag}': {e}")
+                    # Tag the artist if enabled (unified tagging)
+                    try_tag_item(tag_settings, "missing", lidarr_api.tag_processed_artist,
+                                 api_url, api_key, api_timeout, artist_id,
+                                 lidarr_logger, f"artist {artist_id}")
                 
                 # Also mark all albums from this artist as processed
                 if artist_id in items_by_artist:
                     for album in items_by_artist[artist_id]:
                         album_id = album.get('id')
                         if album_id:
-                            album_success = add_processed_id("lidarr", instance_name, str(album_id))
+                            album_success = add_processed_id("lidarr", instance_key, str(album_id))
                             lidarr_logger.debug(f"Added album ID {album_id} to processed list for {instance_name}, success: {album_success}")
                 
                 # Log to history system
-                log_processed_media("lidarr", f"{artist_name}", artist_id, instance_name, "missing")
+                log_processed_media("lidarr", f"{artist_name}", artist_id, instance_key, "missing", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
                 lidarr_logger.debug(f"Logged history entry for artist: {artist_name}")
                 
                 time.sleep(0.1) # Small delay between triggers
@@ -313,7 +319,7 @@ def process_missing_albums(
 
             # Mark the albums as processed BEFORE triggering the search
             for album_id in album_ids_to_search:
-                success = add_processed_id("lidarr", instance_name, str(album_id))
+                success = add_processed_id("lidarr", instance_key, str(album_id))
                 lidarr_logger.debug(f"Added album ID {album_id} to processed list for {instance_name}, success: {success}")
             
             # Now trigger the search
@@ -321,26 +327,21 @@ def process_missing_albums(
             if command_id:
                 # Log after successful search
                 lidarr_logger.debug(f"Album search command triggered with ID: {command_id} for albums: [{', '.join(album_details_log)}]")
-                increment_stat("lidarr", "hunted") # Changed from "missing" to "hunted"
+                increment_stat("lidarr", "hunted", len(album_ids_to_search), instance_key) # Changed from "missing" to "hunted"
                 processed_count += len(album_ids_to_search) # Count albums searched
                 processed_artists_or_albums.update(album_ids_to_search)
                 
-                # Tag artists if enabled (from albums)
-                if tag_processed_items:
-                    from src.primary.settings_manager import get_custom_tag
-                    custom_tag = get_custom_tag("lidarr", "missing", "huntarr-missing")
-                    tagged_artists = set()  # Track which artists we've already tagged
-                    for album_id in album_ids_to_search:
-                        album_info = missing_items_dict.get(album_id)
-                        if album_info:
-                            artist_id = album_info.get('artistId')
-                            if artist_id and artist_id not in tagged_artists:
-                                try:
-                                    lidarr_api.tag_processed_artist(api_url, api_key, api_timeout, artist_id, custom_tag)
-                                    lidarr_logger.debug(f"Tagged artist {artist_id} with '{custom_tag}'")
-                                    tagged_artists.add(artist_id)
-                                except Exception as e:
-                                    lidarr_logger.warning(f"Failed to tag artist {artist_id} with '{custom_tag}': {e}")
+                # Tag artists if enabled (unified tagging)
+                tagged_artists = set()
+                for album_id in album_ids_to_search:
+                    album_info = missing_items_dict.get(album_id)
+                    if album_info:
+                        artist_id = album_info.get('artistId')
+                        if artist_id and artist_id not in tagged_artists:
+                            try_tag_item(tag_settings, "missing", lidarr_api.tag_processed_artist,
+                                         api_url, api_key, api_timeout, artist_id,
+                                         lidarr_logger, f"artist {artist_id}")
+                            tagged_artists.add(artist_id)
                 
                 # Log to history system
                 for album_id in album_ids_to_search:
@@ -350,7 +351,9 @@ def process_missing_albums(
                         title = album_info.get('title', f'Album ID {album_id}')
                         artist_name = album_info.get('artist', {}).get('artistName', 'Unknown Artist')
                         media_name = f"{artist_name} - {title}"
-                        log_processed_media("lidarr", media_name, album_id, instance_name, "missing")
+                        # Use foreignAlbumId for Lidarr URLs (falls back to internal ID if not available)
+                        foreign_album_id = album_info.get('foreignAlbumId', album_id)
+                        log_processed_media("lidarr", media_name, foreign_album_id, instance_key, "missing", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
                         lidarr_logger.debug(f"Logged history entry for album: {media_name}")
                 
                 time.sleep(command_wait_delay) # Basic delay after the single command
