@@ -13,7 +13,11 @@ from src.primary.utils.history_utils import log_processed_media
 from src.primary.stateful_manager import is_processed, add_processed_id
 from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
 from src.primary.settings_manager import load_settings, get_advanced_setting
-from src.primary.state import check_state_reset  # Add the missing import
+from src.primary.state import check_state_reset
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items, filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item, extract_tag_settings
 
 # Get logger for the app
 lidarr_logger = get_logger(__name__) # Use __name__ for correct logger hierarchy
@@ -35,37 +39,28 @@ def process_cutoff_upgrades(
     lidarr_logger.info("Starting quality cutoff upgrades processing cycle for Lidarr.")
     processed_any = False
 
-    # --- Extract Settings --- #
-    # Instance details are now part of app_settings passed from background loop
-    instance_name = app_settings.get("instance_name", "Lidarr Default")
+    # Extract common settings using shared utility
+    s = extract_app_settings(app_settings, "lidarr", "hunt_upgrade_items", "Lidarr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_upgrade_items = s['hunt_count']
+    command_wait_delay = s['command_wait_delay']
+    command_wait_attempts = s['command_wait_attempts']
     
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    
-    # Get command wait settings from database
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-
-    # General Lidarr settings (also from app_settings)
-    hunt_upgrade_items = app_settings.get("hunt_upgrade_items", 0)
-    monitored_only = app_settings.get("monitored_only", True)
+    # App-specific settings
+    upgrade_selection_method = (app_settings.get("upgrade_selection_method") or "cutoff").strip().lower()
+    if upgrade_selection_method not in ("cutoff", "tags"):
+        upgrade_selection_method = "cutoff"
+    upgrade_tag = (app_settings.get("upgrade_tag") or "").strip()
 
     lidarr_logger.info(f"Using API timeout of {api_timeout} seconds for Lidarr upgrades")
-
     lidarr_logger.debug(f"Processing upgrades for instance: {instance_name}")
-    # lidarr_logger.debug(f"Instance Config (extracted): {{ 'api_url': '{api_url}', 'api_key': '***' }}")
-    # lidarr_logger.debug(f"General Settings (from app_settings): {app_settings}") # Avoid logging full settings potentially containing sensitive info
 
-    # Check if API URL or Key are missing
-    if not api_url or not api_key:
-        lidarr_logger.error(f"Missing API URL or Key for instance '{instance_name}'. Cannot process upgrades.")
-        return False
-
-    # Check if upgrade hunting is enabled
-    if hunt_upgrade_items <= 0:
-        lidarr_logger.info(f"'hunt_upgrade_items' is {hunt_upgrade_items} or less. Skipping upgrade processing for {instance_name}.")
+    if not validate_settings(api_url, api_key, hunt_upgrade_items, "lidarr", lidarr_logger):
         return False
 
     lidarr_logger.info(f"Looking for quality upgrades for {instance_name}")
@@ -77,32 +72,53 @@ def process_cutoff_upgrades(
     processed_count = 0
     processed_any = False
 
-    # Load settings to check if tagging is enabled
-    lidarr_settings = load_settings("lidarr")
-    tag_processed_items = lidarr_settings.get("tag_processed_items", True)
+    # Extract tag settings using shared utility
+    tag_settings = extract_tag_settings(app_settings)
 
     try:
-        lidarr_logger.info(f"Retrieving cutoff unmet albums...")
-        # Use efficient random page selection instead of fetching all albums
-        cutoff_unmet_data = lidarr_api.get_cutoff_unmet_albums_random_page(
-            api_url, api_key, api_timeout, monitored_only, hunt_upgrade_items * 2
+        if upgrade_selection_method == "tags":
+            if not upgrade_tag:
+                lidarr_logger.warning("Upgrade selection method is 'Tags' but no upgrade tag is configured. Skipping.")
+                return False
+            lidarr_logger.info(f"Retrieving albums whose artists DON'T have tag \"{upgrade_tag}\" (Upgradinatorr-style: tag tracks processed)...")
+            cutoff_unmet_data = lidarr_api.get_albums_without_artist_tag(
+                api_url, api_key, api_timeout, upgrade_tag, monitored_only
+            )
+            if cutoff_unmet_data is None:
+                return False
+            if not cutoff_unmet_data:
+                lidarr_logger.info(f"No albums found whose artists lack the tag \"{upgrade_tag}\" (all have been processed).")
+                return False
+            lidarr_logger.info(f"Found {len(cutoff_unmet_data)} albums whose artists DON'T have tag \"{upgrade_tag}\".")
+        else:
+            lidarr_logger.info(f"Retrieving cutoff unmet albums...")
+            # Use efficient random page selection instead of fetching all albums
+            cutoff_unmet_data = lidarr_api.get_cutoff_unmet_albums_random_page(
+                api_url, api_key, api_timeout, monitored_only, hunt_upgrade_items * 2
+            )
+            if cutoff_unmet_data is None:  # API call failed
+                lidarr_logger.error("Failed to retrieve cutoff unmet albums from Lidarr API.")
+                return False
+            if not cutoff_unmet_data:
+                lidarr_logger.info("No cutoff unmet albums found.")
+                return False
+            lidarr_logger.info(f"Retrieved {len(cutoff_unmet_data)} cutoff unmet albums from random page selection.")
+
+        # Filter out albums whose artist has an exempt tag (issue #676)
+        cutoff_unmet_data = filter_exempt_items(
+            cutoff_unmet_data, s['exempt_tags'], lidarr_api,
+            api_url, api_key, api_timeout,
+            get_tags_fn=lambda a: (a.get("artist") or {}).get("tags", []),
+            get_id_fn=lambda a: a.get("id"),
+            get_title_fn=lambda a: a.get("title", "Unknown"),
+            app_type="lidarr", logger=lidarr_logger
         )
-        
-        if cutoff_unmet_data is None: # API call failed
-            lidarr_logger.error("Failed to retrieve cutoff unmet albums from Lidarr API.")
-            return False
-        
-        if not cutoff_unmet_data:
-            lidarr_logger.info("No cutoff unmet albums found.")
-            return False
-        
-        lidarr_logger.info(f"Retrieved {len(cutoff_unmet_data)} cutoff unmet albums from random page selection.")
 
         # Filter out already processed items
         unprocessed_albums = []
         for album in cutoff_unmet_data:
             album_id = album.get('id')  # Keep as integer
-            if album_id and not is_processed("lidarr", instance_name, str(album_id)):  # Convert to string only for processed check
+            if album_id and not is_processed("lidarr", instance_key, str(album_id)):  # Convert to string only for processed check
                 unprocessed_albums.append(album)
             else:
                 lidarr_logger.debug(f"Skipping already processed album ID: {album_id}")
@@ -154,7 +170,7 @@ def process_cutoff_upgrades(
 
         # Mark the albums as processed BEFORE triggering the search
         for album_id in album_ids_to_search:
-            success = add_processed_id("lidarr", instance_name, str(album_id))
+            success = add_processed_id("lidarr", instance_key, str(album_id))
             lidarr_logger.debug(f"Added album ID {album_id} to processed list for {instance_name}, success: {success}")
 
         lidarr_logger.info(f"Triggering Album Search for {len(album_ids_to_search)} albums for upgrade on instance {instance_name}: {album_ids_to_search}")
@@ -167,22 +183,32 @@ def process_cutoff_upgrades(
         )
         if command_id:
             lidarr_logger.debug(f"Upgrade album search command triggered with ID: {command_id} for albums: {album_ids_to_search}")
-            increment_stat("lidarr", "upgraded") # Use appropriate stat key
+            increment_stat("lidarr", "upgraded", 1, instance_key)  # Use appropriate stat key
             
-            # Tag artists if enabled (from albums)
-            if tag_processed_items:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("lidarr", "upgrade", "huntarr-upgraded")
-                tagged_artists = set()  # Track which artists we've already tagged
+            # For tag-based method: add the upgrade tag to artists to mark as processed (Upgradinatorr-style)
+            if upgrade_selection_method == "tags" and upgrade_tag:
+                tagged_artists = set()
                 for album in albums_to_search:
                     artist_id = album.get('artistId')
                     if artist_id and artist_id not in tagged_artists:
                         try:
-                            lidarr_api.tag_processed_artist(api_url, api_key, api_timeout, artist_id, custom_tag)
-                            lidarr_logger.debug(f"Tagged artist {artist_id} with '{custom_tag}'")
-                            tagged_artists.add(artist_id)
+                            tag_id = lidarr_api.get_or_create_tag(api_url, api_key, api_timeout, upgrade_tag)
+                            if tag_id:
+                                lidarr_api.add_tag_to_artist(api_url, api_key, api_timeout, artist_id, tag_id)
+                                lidarr_logger.debug(f"Added upgrade tag '{upgrade_tag}' to artist {artist_id} to mark as processed")
+                                tagged_artists.add(artist_id)
                         except Exception as e:
-                            lidarr_logger.warning(f"Failed to tag artist {artist_id} with '{custom_tag}': {e}")
+                            lidarr_logger.warning(f"Failed to add upgrade tag '{upgrade_tag}' to artist {artist_id}: {e}")
+            
+            # Tag artists with huntarr-upgraded if enabled (unified tagging)
+            tagged_artists_upgraded = set()
+            for album in albums_to_search:
+                artist_id = album.get('artistId')
+                if artist_id and artist_id not in tagged_artists_upgraded:
+                    try_tag_item(tag_settings, "upgraded", lidarr_api.tag_processed_artist,
+                                 api_url, api_key, api_timeout, artist_id,
+                                 lidarr_logger, f"artist {artist_id}")
+                    tagged_artists_upgraded.add(artist_id)
             
             # Log to history
             for album_id in album_ids_to_search:
@@ -192,7 +218,9 @@ def process_cutoff_upgrades(
                         album_title = album.get('title', f'Album ID {album_id}')
                         artist_name = album.get('artist', {}).get('artistName', 'Unknown Artist')
                         media_name = f"{artist_name} - {album_title}"
-                        log_processed_media("lidarr", media_name, album_id, instance_name, "upgrade")
+                        # Use foreignAlbumId for Lidarr URLs (falls back to internal ID if not available)
+                        foreign_album_id = album.get('foreignAlbumId', album_id)
+                        log_processed_media("lidarr", media_name, foreign_album_id, instance_key, "upgrade", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
                         lidarr_logger.debug(f"Logged quality upgrade to history for album ID {album_id}")
                         break
                 
