@@ -42,9 +42,9 @@ def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, met
             radarr_logger.error("No URL or API key provided")
             return None
         
-        # Check API limit before making request
+        # Check API limit before making request (skip for read-only / count_api=False, e.g. Activity queue)
         from src.primary.stats_manager import check_hourly_cap_exceeded, increment_hourly_cap
-        if check_hourly_cap_exceeded("radarr"):
+        if count_api and check_hourly_cap_exceeded("radarr"):
             radarr_logger.warning("\U0001F6D1 Radarr API hourly limit reached - skipping request")
             return None
         
@@ -127,6 +127,76 @@ def get_download_queue_size(api_url: str, api_key: str, api_timeout: int) -> int
         radarr_logger.error(f"An unexpected error occurred while getting Radarr queue size: {e}")
         return -1
 
+
+def get_queue(api_url: str, api_key: str, api_timeout: int, page: int = 1, page_size: int = 100) -> Dict[str, Any]:
+    """
+    Get the download queue from Radarr (for Movie Hunt Activity).
+
+    Args:
+        api_url: Base URL of the Radarr API
+        api_key: API key for authentication
+        api_timeout: Request timeout
+        page: Page number (1-based)
+        page_size: Number of records per page
+
+    Returns:
+        Dict with 'records' (list of queue items) and 'totalRecords' (int), or {'records': [], 'totalRecords': 0} on error.
+    """
+    if not api_url or not api_key:
+        return {'records': [], 'totalRecords': 0}
+    try:
+        params = {
+            'page': page,
+            'pageSize': min(page_size, 1000),
+            'includeUnknownMovieItems': True,  # include items just sent to download client (e.g. SABnzbd)
+            'includeMovie': True,
+        }
+        queue_data = arr_request(api_url, api_key, api_timeout, 'queue', params=params, count_api=False)
+        if not queue_data or not isinstance(queue_data, dict):
+            return {'records': [], 'totalRecords': 0}
+        records = queue_data.get('records') if isinstance(queue_data.get('records'), list) else []
+        total = queue_data.get('totalRecords', len(records))
+        return {'records': records, 'totalRecords': total}
+    except Exception as e:
+        radarr_logger.debug(f"Error getting Radarr queue: {e}")
+        return {'records': [], 'totalRecords': 0}
+
+
+def delete_queue_bulk(api_url: str, api_key: str, api_timeout: int, ids: List[int],
+                     remove_from_client: bool = True, blocklist: bool = False) -> bool:
+    """
+    Remove multiple items from Radarr queue (and optionally from download client e.g. SABnzbd).
+
+    Args:
+        api_url: Base URL of the Radarr API
+        api_key: API key for authentication
+        api_timeout: Request timeout
+        ids: List of queue record ids to remove
+        remove_from_client: If True, also remove from download client (default True)
+        blocklist: If True, add release to blocklist (default False)
+
+    Returns:
+        True if request succeeded, False otherwise.
+    """
+    if not api_url or not api_key or not ids:
+        return False
+    try:
+        full_url = f"{api_url.rstrip('/')}/api/v3/queue/bulk"
+        params = {'removeFromClient': str(remove_from_client).lower(), 'blocklist': str(blocklist).lower()}
+        headers = {
+            "X-Api-Key": api_key,
+            "Content-Type": "application/json",
+        }
+        verify_ssl = get_ssl_verify_setting()
+        response = session.delete(full_url, headers=headers, params=params, json={"ids": ids},
+                                  timeout=api_timeout, verify=verify_ssl)
+        response.raise_for_status()
+        return True
+    except Exception as e:
+        radarr_logger.debug(f"Error deleting Radarr queue bulk: {e}")
+        return False
+
+
 def get_quality_profiles(api_url: str, api_key: str, api_timeout: int) -> Optional[List[Dict]]:
     """
     Get all quality profiles configured in Radarr.
@@ -345,27 +415,6 @@ def get_cutoff_unmet_movies_random_page(api_url: str, api_key: str, api_timeout:
     radarr_logger.info(f"✅ Returning {len(records)} cutoff unmet movies from random sampling")
     return records
 
-def refresh_movie(api_url: str, api_key: str, api_timeout: int, movie_id: int, 
-                 command_wait_delay: int = 1, command_wait_attempts: int = 600) -> Optional[int]:
-    """
-    Refresh functionality has been removed as it was a performance bottleneck.
-    This function now returns a placeholder success value without making any API calls.
-    
-    Args:
-        api_url: The base URL of the Radarr API
-        api_key: The API key for authentication
-        api_timeout: Timeout for the API request
-        movie_id: The ID of the movie to refresh
-        command_wait_delay: Seconds to wait between command status checks
-        command_wait_attempts: Maximum number of status check attempts
-        
-    Returns:
-        A placeholder command ID (123) to simulate success
-    """
-    radarr_logger.debug(f"Refresh functionality disabled for movie ID: {movie_id}")
-    # Return a placeholder command ID (123) to simulate success without actually refreshing
-    return 123
-
 def movie_search(api_url: str, api_key: str, api_timeout: int, movie_ids: List[int]) -> Optional[int]:
     """
     Trigger a search for one or more movies.
@@ -389,8 +438,8 @@ def movie_search(api_url: str, api_key: str, api_timeout: int, movie_ids: List[i
         "movieIds": movie_ids
     }
     
-    # Use the updated arr_request
-    response = arr_request(api_url, api_key, api_timeout, endpoint, method="POST", data=data)
+    # count_api=False: caller (missing/upgrade) uses increment_stat_only so API bar = searches + upgrades (one count per action)
+    response = arr_request(api_url, api_key, api_timeout, endpoint, method="POST", data=data, count_api=False)
     if response and 'id' in response:
         command_id = response['id']
         radarr_logger.debug(f"Triggered search for movie IDs: {movie_ids}. Command ID: {command_id}")
@@ -401,31 +450,8 @@ def movie_search(api_url: str, api_key: str, api_timeout: int, movie_ids: List[i
 
 def check_connection(api_url: str, api_key: str, api_timeout: int) -> bool:
     """Check the connection to Radarr API."""
-    try:
-        # Ensure api_url is properly formatted
-        if not api_url:
-            radarr_logger.error("API URL is empty or not set")
-            return False
-            
-        # Make sure api_url has a scheme
-        if not (api_url.startswith('http://') or api_url.startswith('https://')):
-            radarr_logger.error(f"Invalid URL format: {api_url} - URL must start with http:// or https://")
-            return False
-            
-        # Ensure URL doesn't end with a slash before adding the endpoint
-        base_url = api_url.rstrip('/')
-        full_url = f"{base_url}/api/v3/system/status"
-        
-        response = requests.get(full_url, headers={"X-Api-Key": api_key}, timeout=api_timeout)
-        response.raise_for_status() # Raise HTTPError for bad responses (4xx or 5xx)
-        radarr_logger.debug("Successfully connected to Radarr.")
-        return True
-    except requests.exceptions.RequestException as e:
-        radarr_logger.error(f"Error connecting to Radarr: {e}")
-        return False
-    except Exception as e:
-        radarr_logger.error(f"An unexpected error occurred during Radarr connection check: {e}")
-        return False
+    from src.primary.apps._common.arr_api import check_connection as _check
+    return _check(api_url, api_key, api_timeout, "radarr", radarr_logger)
 
 def wait_for_command(api_url: str, api_key: str, api_timeout: int, command_id: int, 
                     delay_seconds: int = 1, max_attempts: int = 600) -> bool:
@@ -458,6 +484,68 @@ def wait_for_command(api_url: str, api_key: str, api_timeout: int, command_id: i
     radarr_logger.warning(f"Timed out waiting for command {command_id} to complete")
     return False
 
+def get_tag_id_by_label(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
+    """
+    Get tag ID by label (does not create the tag).
+    Used for tag-based upgrade selection (e.g. Upgradinatorr-style).
+    """
+    try:
+        response = arr_request(api_url, api_key, api_timeout, "tag", count_api=False)
+        if response:
+            for tag in response:
+                if tag.get('label') == tag_label:
+                    return tag.get('id')
+        return None
+    except Exception as e:
+        radarr_logger.error(f"Error getting tag '{tag_label}': {e}")
+        return None
+
+
+def get_exempt_tag_ids(api_url: str, api_key: str, api_timeout: int, exempt_tag_labels: list) -> dict:
+    """
+    Resolve exempt tag labels to tag IDs. Returns dict tag_id -> label for tags that exist.
+    Exact match on label. Used to filter out movies with exempt tags (issue #676).
+    Only counts tags that are actually entered (non-empty); empty list or all-whitespace = no exclusions.
+    """
+    if exempt_tag_labels is None:
+        return {}
+    if isinstance(exempt_tag_labels, str):
+        exempt_tag_labels = [exempt_tag_labels]
+    labels = [str(l).strip() for l in (exempt_tag_labels or []) if l is not None and str(l).strip()]
+    if not labels:
+        return {}
+    result = {}
+    for label in labels:
+        tid = get_tag_id_by_label(api_url, api_key, api_timeout, label)
+        if tid is not None:
+            result[tid] = label
+    return result
+
+
+def get_movies_without_tag(
+    api_url: str, api_key: str, api_timeout: int, tag_label: str, monitored_only: bool
+) -> Optional[List[Dict]]:
+    """
+    Get movies that DON'T have the given tag (Upgradinatorr-style: tag tracks processed items).
+    After searching, the tag will be added to mark them as processed.
+    """
+    tag_id = get_or_create_tag(api_url, api_key, api_timeout, tag_label)
+    if tag_id is None:
+        radarr_logger.error(f"Failed to get or create tag '{tag_label}' in Radarr.")
+        return None
+    movies = arr_request(api_url, api_key, api_timeout, "movie", count_api=False)
+    if movies is None:
+        radarr_logger.error("Failed to retrieve movies from Radarr API.")
+        return None
+    out = [
+        m for m in movies
+        if tag_id not in m.get('tags', [])
+        and (not monitored_only or m.get('monitored', False))
+    ]
+    radarr_logger.debug(f"Found {len(out)} movies WITHOUT tag '{tag_label}' (monitored_only={monitored_only}).")
+    return out
+
+
 def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
     """
     Get existing tag ID or create a new tag in Radarr.
