@@ -11,10 +11,14 @@ import datetime
 from typing import List, Dict, Any, Set, Callable
 from src.primary.utils.logger import get_logger
 from src.primary.apps.radarr import api as radarr_api
-from src.primary.stats_manager import increment_stat_only, check_hourly_cap_exceeded
-from src.primary.stateful_manager import is_processed, add_processed_id
+from src.primary.stats_manager import increment_stat_only
+from src.primary.stateful_manager import add_processed_id
 from src.primary.utils.history_utils import log_processed_media
-from src.primary.settings_manager import load_settings, get_advanced_setting
+from src.primary.settings_manager import load_settings
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items, filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item
 
 # Get logger for the app
 radarr_logger = get_logger("radarr")
@@ -93,41 +97,25 @@ def process_missing_movies(
     """
     processed_any = False
     
-    # Get instance name - check for instance_name first, fall back to legacy "name" key if needed
-    instance_name = app_settings.get("instance_name", app_settings.get("name", "Radarr Default"))
+    # Extract common settings using shared utility
+    s = extract_app_settings(app_settings, "radarr", "hunt_missing_movies", "Radarr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_missing_movies = s['hunt_count']
+    tag_settings = s['tag_settings']
     
-    # Load settings to check if tagging is enabled
-    radarr_settings = load_settings("radarr")
-    tag_processed_items = radarr_settings.get("tag_processed_items", True)
+    # App-specific settings
+    skip_future_releases = app_settings.get("skip_future_releases", True)
     
-    # Log important settings
     radarr_logger.info("=== Radarr Missing Movies Settings ===")
     radarr_logger.debug(f"Instance Name: {instance_name}")
-    
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    monitored_only = app_settings.get("monitored_only", True)
-    skip_future_releases = app_settings.get("skip_future_releases", True)
-    # skip_movie_refresh setting removed as it was a performance bottleneck
-    hunt_missing_movies = app_settings.get("hunt_missing_movies", 0)
-    
-    # Use advanced settings from database for command operations
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-    
-    # Configuration logging removed to reduce log spam
-    
     radarr_logger.info("Starting missing movies processing cycle for Radarr.")
     
-    if not api_url or not api_key:
-        radarr_logger.error("API URL or Key not configured in settings. Cannot process missing movies.")
-        return False
-
-    # Skip if hunt_missing_movies is set to 0
-    if hunt_missing_movies <= 0:
-        radarr_logger.info("'hunt_missing_movies' setting is 0 or less. Skipping missing movie processing.")
+    if not validate_settings(api_url, api_key, hunt_missing_movies, "radarr", radarr_logger):
         return False
 
     # Check for stop signal
@@ -227,19 +215,25 @@ def process_missing_movies(
     if not missing_movies:
         radarr_logger.info("No missing movies left to process after filtering future releases.")
         return False
-        
+
+    # Filter out movies with exempt tags (issue #676)
+    missing_movies = filter_exempt_items(
+        missing_movies, s['exempt_tags'], radarr_api,
+        api_url, api_key, api_timeout,
+        get_tags_fn=lambda m: m.get("tags", []),
+        get_id_fn=lambda m: m.get("id"),
+        get_title_fn=lambda m: m.get("title", "Unknown"),
+        app_type="radarr", logger=radarr_logger
+    )
+
     movies_processed = 0
     processing_done = False
     
-    # Filter out already processed movies using stateful management
-    unprocessed_movies = []
-    for movie in missing_movies:
-        movie_id = str(movie.get("id"))
-        if not is_processed("radarr", instance_name, movie_id):
-            unprocessed_movies.append(movie)
-        else:
-            radarr_logger.debug(f"Skipping already processed movie ID: {movie_id}")
-    
+    # Filter out already processed movies using shared utility
+    unprocessed_movies = filter_unprocessed(
+        missing_movies, "radarr", instance_key,
+        get_id_fn=lambda m: m.get("id"), logger=radarr_logger
+    )
     radarr_logger.info(f"Found {len(unprocessed_movies)} unprocessed missing movies out of {len(missing_movies)} total.")
     
     if not unprocessed_movies:
@@ -266,18 +260,8 @@ def process_missing_movies(
     
     # Process each movie
     for movie in movies_to_process:
-        if stop_check():
-            radarr_logger.info("Stop requested during processing. Aborting...")
+        if not should_continue_processing("radarr", stop_check, radarr_logger):
             break
-        
-        # Check API limit before processing each movie
-        try:
-            if check_hourly_cap_exceeded("radarr"):
-                radarr_logger.warning(f"🛑 Radarr API hourly limit reached - stopping missing movies processing after {movies_processed} movies")
-                break
-        except Exception as e:
-            radarr_logger.error(f"Error checking hourly API cap: {e}")
-            # Continue processing if cap check fails - safer than stopping
             
         movie_id = movie.get("id")
         movie_title = movie.get("title", "Unknown Title")
@@ -291,27 +275,24 @@ def process_missing_movies(
         if search_success:
             radarr_logger.info(f"Successfully triggered search for movie '{movie_title}'")
             
-            # Tag the movie if enabled
-            if tag_processed_items:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("radarr", "missing", "huntarr-missing")
-                try:
-                    radarr_api.tag_processed_movie(api_url, api_key, api_timeout, movie_id, custom_tag)
-                    radarr_logger.debug(f"Tagged movie {movie_id} with '{custom_tag}'")
-                except Exception as e:
-                    radarr_logger.warning(f"Failed to tag movie {movie_id} with '{custom_tag}': {e}")
+            # Tag the movie if enabled (unified tagging)
+            try_tag_item(tag_settings, "missing", radarr_api.tag_processed_movie,
+                         api_url, api_key, api_timeout, movie_id,
+                         radarr_logger, f"movie {movie_id}")
             
             # Immediately add to processed IDs to prevent duplicate processing
-            success = add_processed_id("radarr", instance_name, str(movie_id))
+            success = add_processed_id("radarr", instance_key, str(movie_id))
             radarr_logger.debug(f"Added processed ID: {movie_id}, success: {success}")
             
             # Log to history system
             year = movie.get("year", "Unknown Year")
             media_name = f"{movie_title} ({year})"
-            log_processed_media("radarr", media_name, movie_id, instance_name, "missing")
+            # Use TMDb ID for Radarr URLs (falls back to internal ID if TMDb ID not available)
+            tmdb_id = movie.get("tmdbId", movie_id)
+            log_processed_media("radarr", media_name, tmdb_id, instance_key, "missing", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             radarr_logger.debug(f"Logged history entry for movie: {media_name}")
             
-            increment_stat_only("radarr", "hunted")
+            increment_stat_only("radarr", "hunted", 1, instance_key)
             movies_processed += 1
             processed_any = True
         else:
