@@ -10,11 +10,15 @@ import datetime
 from typing import List, Dict, Any, Set, Callable
 from src.primary.utils.logger import get_logger
 from src.primary.apps.radarr import api as radarr_api
-from src.primary.stats_manager import increment_stat, increment_stat_only, check_hourly_cap_exceeded
-from src.primary.stateful_manager import is_processed, add_processed_id
+from src.primary.stats_manager import increment_stat_only
+from src.primary.stateful_manager import add_processed_id
 from src.primary.utils.history_utils import log_processed_media
-from src.primary.settings_manager import get_advanced_setting, load_settings
+from src.primary.settings_manager import load_settings
 from src.primary.utils.date_utils import parse_date
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items, filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item
 
 # Get logger for the app
 radarr_logger = get_logger("radarr")
@@ -71,37 +75,110 @@ def process_cutoff_upgrades(
     radarr_logger.info("Starting quality cutoff upgrades processing cycle for Radarr.")
     processed_any = False
     
-    # Load settings to check if tagging is enabled
-    radarr_settings = load_settings("radarr")
-    tag_processed_items = radarr_settings.get("tag_processed_items", True)
+    # Extract common settings using shared utility
+    s = extract_app_settings(app_settings, "radarr", "hunt_upgrade_movies", "Radarr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_upgrade_movies = s['hunt_count']
+    tag_settings = s['tag_settings']
     
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    monitored_only = app_settings.get("monitored_only", True)
-    # skip_movie_refresh setting removed as it was a performance bottleneck
-    hunt_upgrade_movies = app_settings.get("hunt_upgrade_movies", 0)
+    # App-specific settings
     skip_future_releases = app_settings.get("skip_future_releases", True)
-    
-    # Use advanced settings from database for command operations
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-    
-    # Get instance name - check for instance_name first, fall back to legacy "name" key if needed
-    instance_name = app_settings.get("instance_name", app_settings.get("name", "Radarr Default"))
-    
-    # Get movies eligible for upgrade
-    radarr_logger.info("Retrieving movies eligible for cutoff upgrade...")
-    upgrade_eligible_data = radarr_api.get_cutoff_unmet_movies_random_page(
-        api_url, api_key, api_timeout, monitored_only, count=50
-    )
-    
-    if not upgrade_eligible_data:
-        radarr_logger.info("No movies found eligible for upgrade or error retrieving them.")
+    upgrade_selection_method = (app_settings.get("upgrade_selection_method") or "cutoff").strip().lower()
+    if upgrade_selection_method not in ("cutoff", "tags"):
+        upgrade_selection_method = "cutoff"
+    upgrade_tag = (app_settings.get("upgrade_tag") or "").strip()
+ 
+    quality_profiles = radarr_api.get_quality_profiles(api_url, api_key, api_timeout)
+ 
+    if not quality_profiles:
+        radarr_logger.warning("Failed to retrieve quality profiles.")
         return False
-        
-    radarr_logger.info(f"Found {len(upgrade_eligible_data)} movies eligible for upgrade.")
+ 
+    profile_upgrade_map = {
+        profile.get('id'): profile.get('upgradeAllowed', False)
+        for profile in quality_profiles
+    }
+ 
+    profile_name_map = {
+        profile.get('id'): profile.get('name', 'Unknown')
+        for profile in quality_profiles
+    }
+ 
+    no_upgrade_profiles = [
+        f"{profile.get('name')} (ID: {profile.get('id')})"
+        for profile in quality_profiles
+        if not profile.get('upgradeAllowed', False)
+    ]
+    if no_upgrade_profiles:
+        radarr_logger.info(f"Quality profiles without upgrade allowed: {', '.join(no_upgrade_profiles)}")
+ 
+    if upgrade_selection_method == "tags":
+        if not upgrade_tag:
+            radarr_logger.warning("Upgrade selection method is 'Tags' but no upgrade tag is configured. Skipping.")
+            return False
+        radarr_logger.info(f"Retrieving movies WITHOUT tag \"{upgrade_tag}\" (Upgradinatorr-style: tag tracks processed)...")
+        upgrade_eligible_data = radarr_api.get_movies_without_tag(
+            api_url, api_key, api_timeout, upgrade_tag, monitored_only
+        )
+        if upgrade_eligible_data is None:
+            return False
+        if not upgrade_eligible_data:
+            radarr_logger.info(f"No movies found without the tag \"{upgrade_tag}\" (all have been processed).")
+            return False
+        radarr_logger.info(f"Found {len(upgrade_eligible_data)} movies without tag \"{upgrade_tag}\".")
+    else:
+        radarr_logger.info("Retrieving movies eligible for cutoff upgrade...")
+        upgrade_eligible_data = radarr_api.get_cutoff_unmet_movies_random_page(
+            api_url, api_key, api_timeout, monitored_only, count=50
+        )
+        if not upgrade_eligible_data:
+            radarr_logger.info("No movies found eligible for upgrade or error retrieving them.")
+            return False
+        radarr_logger.info(f"Found {len(upgrade_eligible_data)} movies below quality cutoff.")
+ 
+    upgrade_allowed_movies = []
+    filtered_count = 0
+    for movie in upgrade_eligible_data:
+        movie_id = movie.get('id')
+        movie_title = movie.get('title', 'Unknown Title')
+        quality_profile_id = movie.get('qualityProfileId')
+ 
+        if quality_profile_id in profile_upgrade_map:
+            if profile_upgrade_map[quality_profile_id]:
+                upgrade_allowed_movies.append(movie)
+            else:
+                profile_name = profile_name_map.get(quality_profile_id, 'Unknown')
+                radarr_logger.debug(f"Skipping movie ID {movie_id} ('{movie_title}') - quality profile '{profile_name}' does not have upgrade allowed")
+                filtered_count += 1
+        else:
+            radarr_logger.warning(f"Movie ID {movie_id} ('{movie_title}') has unknown quality profile ID {quality_profile_id}, including it")
+            upgrade_allowed_movies.append(movie)
+ 
+    if filtered_count > 0:
+        radarr_logger.info(f"Filtered out {filtered_count} movies without upgrade allowed quality profiles")
+ 
+    upgrade_eligible_data = upgrade_allowed_movies
+ 
+    if not upgrade_eligible_data:
+        radarr_logger.info("No movies with upgrade allowed quality profiles.")
+        return False
+ 
+    radarr_logger.info(f"Found {len(upgrade_eligible_data)} movies with upgrade allowed quality profiles.")
+
+    # Filter out movies with exempt tags (issue #676)
+    upgrade_eligible_data = filter_exempt_items(
+        upgrade_eligible_data, s['exempt_tags'], radarr_api,
+        api_url, api_key, api_timeout,
+        get_tags_fn=lambda m: m.get("tags", []),
+        get_id_fn=lambda m: m.get("id"),
+        get_title_fn=lambda m: m.get("title", "Unknown"),
+        app_type="radarr", logger=radarr_logger
+    )
 
     # Skip future releases if enabled (matching missing movies logic)
     if skip_future_releases:
@@ -179,15 +256,11 @@ def process_cutoff_upgrades(
         radarr_logger.info("No movies eligible for upgrade left to process after filtering future releases.")
         return False
 
-    # Filter out already processed movies using stateful management
-    unprocessed_movies = []
-    for movie in upgrade_eligible_data:
-        movie_id = str(movie.get("id"))
-        if not is_processed("radarr", instance_name, movie_id):
-            unprocessed_movies.append(movie)
-        else:
-            radarr_logger.debug(f"Skipping already processed movie ID: {movie_id}")
-    
+    # Filter out already processed movies using shared utility
+    unprocessed_movies = filter_unprocessed(
+        upgrade_eligible_data, "radarr", instance_key,
+        get_id_fn=lambda m: m.get("id"), logger=radarr_logger
+    )
     radarr_logger.info(f"Found {len(unprocessed_movies)} unprocessed movies for upgrade out of {len(upgrade_eligible_data)} total.")
     
     if not unprocessed_movies:
@@ -202,18 +275,8 @@ def process_cutoff_upgrades(
     processed_something = False
     
     for movie in movies_to_process:
-        if stop_check():
-            radarr_logger.info("Stop signal received, aborting Radarr upgrade cycle.")
+        if not should_continue_processing("radarr", stop_check, radarr_logger):
             break
-        
-        # Check API limit before processing each movie
-        try:
-            if check_hourly_cap_exceeded("radarr"):
-                radarr_logger.warning(f"🛑 Radarr API hourly limit reached - stopping upgrade processing after {processed_count} movies")
-                break
-        except Exception as e:
-            radarr_logger.error(f"Error checking hourly API cap: {e}")
-            # Continue processing if cap check fails - safer than stopping
             
         movie_id = movie.get("id")
         movie_title = movie.get("title")
@@ -229,22 +292,29 @@ def process_cutoff_upgrades(
         
         if search_result:
             radarr_logger.info(f"  - Successfully triggered search for quality upgrade.")
-            add_processed_id("radarr", instance_name, str(movie_id))
-            increment_stat_only("radarr", "upgraded")
+            add_processed_id("radarr", instance_key, str(movie_id))
+            increment_stat_only("radarr", "upgraded", 1, instance_key)
             
-            # Tag the movie if enabled
-            if tag_processed_items:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("radarr", "upgrade", "huntarr-upgraded")
+            # For tag-based method: add the upgrade tag to mark as processed (Upgradinatorr-style)
+            if upgrade_selection_method == "tags" and upgrade_tag:
                 try:
-                    radarr_api.tag_processed_movie(api_url, api_key, api_timeout, movie_id, custom_tag)
-                    radarr_logger.debug(f"Tagged movie {movie_id} with '{custom_tag}'")
+                    tag_id = radarr_api.get_or_create_tag(api_url, api_key, api_timeout, upgrade_tag)
+                    if tag_id:
+                        radarr_api.add_tag_to_movie(api_url, api_key, api_timeout, movie_id, tag_id)
+                        radarr_logger.debug(f"Added upgrade tag '{upgrade_tag}' to movie {movie_id} to mark as processed")
                 except Exception as e:
-                    radarr_logger.warning(f"Failed to tag movie {movie_id} with '{custom_tag}': {e}")
+                    radarr_logger.warning(f"Failed to add upgrade tag '{upgrade_tag}' to movie {movie_id}: {e}")
+            
+            # Tag with huntarr-upgrade if enabled (unified tagging)
+            try_tag_item(tag_settings, "upgrade", radarr_api.tag_processed_movie,
+                         api_url, api_key, api_timeout, movie_id,
+                         radarr_logger, f"movie {movie_id}")
             
             # Log to history so the upgrade appears in the history UI
             media_name = f"{movie_title} ({movie_year})"
-            log_processed_media("radarr", media_name, movie_id, instance_name, "upgrade")
+            # Use TMDb ID for Radarr URLs (falls back to internal ID if TMDb ID not available)
+            tmdb_id = movie.get("tmdbId", movie_id)
+            log_processed_media("radarr", media_name, tmdb_id, instance_key, "upgrade", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             radarr_logger.debug(f"Logged quality upgrade to history for movie ID {movie_id}")
             
             processed_count += 1
