@@ -27,37 +27,8 @@ API_TIMEOUT = 30
 
 def check_connection(api_url: str, api_key: str, api_timeout: int) -> bool:
     """Check the connection to Readarr API."""
-    try:
-        # Ensure api_url is properly formatted
-        if not api_url:
-            logger.error("API URL is empty or not set")
-            return False
-            
-        # Make sure api_url has a scheme
-        if not (api_url.startswith('http://') or api_url.startswith('https://')):
-            logger.error(f"Invalid URL format: {api_url} - URL must start with http:// or https://")
-            return False
-            
-        # Ensure URL doesn't end with a slash before adding the endpoint
-        base_url = api_url.rstrip('/')
-        full_url = f"{base_url}/api/v1/system/status"
-        
-        # Add User-Agent header to identify Huntarr
-        headers = {
-            "X-Api-Key": api_key,
-            "User-Agent": "Huntarr/1.0 (https://github.com/plexguide/Huntarr.io)"
-        }
-        
-        response = requests.get(full_url, headers=headers, timeout=api_timeout)
-        response.raise_for_status() # Raise HTTPError for bad responses (4xx or 5xx)
-        logger.debug("Successfully connected to Readarr.")
-        return True
-    except requests.exceptions.RequestException as e:
-        logger.error(f"Error connecting to Readarr: {e}")
-        return False
-    except Exception as e:
-        logger.error(f"An unexpected error occurred during Readarr connection check: {e}")
-        return False
+    from src.primary.apps._common.arr_api import check_connection as _check
+    return _check(api_url, api_key, api_timeout, "readarr", logger, api_version="v1")
 
 def get_download_queue_size(api_url: str = None, api_key: str = None, timeout: int = 30) -> int:
     """
@@ -136,7 +107,7 @@ def arr_request(endpoint: str, method: str = "GET", data: Dict = None, app_type:
             module_name = f'src.primary.apps.{app_type}'
             module = importlib.import_module(module_name)
             if hasattr(module, 'get_configured_instances'):
-                instances = module.get_configured_instances()
+                instances = module.get_configured_instances(quiet=True)
                 if instances:
                     # Use the first instance by default
                     instance_data = instances[0]
@@ -316,8 +287,13 @@ def get_wanted_missing_books(api_url: str, api_key: str, api_timeout: int, monit
             # 'sortDirection': 'ascending',
             # 'monitored': monitored_only # Note: Check if Readarr API supports this directly for wanted/missing
         }
+        
+        # Get SSL verification setting
+        from src.primary.settings_manager import get_ssl_verify_setting
+        verify_ssl = get_ssl_verify_setting()
+        
         try:
-            response = requests.get(url, headers=headers, params=params, timeout=api_timeout)
+            response = requests.get(url, headers=headers, params=params, timeout=api_timeout, verify=verify_ssl)
             response.raise_for_status()
             data = response.json()
 
@@ -385,11 +361,15 @@ def get_wanted_missing_books_random_page(api_url: str, api_key: str, api_timeout
         'pageSize': 1
     }
     
+    # Get SSL verification setting
+    from src.primary.settings_manager import get_ssl_verify_setting
+    verify_ssl = get_ssl_verify_setting()
+    
     for attempt in range(retries + 1):
         try:
             # Get total record count from a minimal query
             logger.debug(f"Getting missing books count (attempt {attempt+1}/{retries+1})")
-            response = requests.get(url, headers=headers, params=params, timeout=api_timeout)
+            response = requests.get(url, headers=headers, params=params, timeout=api_timeout, verify=verify_ssl)
             response.raise_for_status()
             
             if not response.content:
@@ -424,7 +404,7 @@ def get_wanted_missing_books_random_page(api_url: str, api_key: str, api_timeout
                     'pageSize': page_size
                 }
                 
-                response = requests.get(url, headers=headers, params=params, timeout=api_timeout)
+                response = requests.get(url, headers=headers, params=params, timeout=api_timeout, verify=verify_ssl)
                 response.raise_for_status()
                 
                 if not response.content:
@@ -487,24 +467,6 @@ def get_wanted_missing_books_random_page(api_url: str, api_key: str, api_timeout
     logger.error("All attempts to get missing books failed")
     return []
 
-def refresh_author(author_id: int, api_url: Optional[str] = None, api_key: Optional[str] = None, api_timeout: Optional[int] = None) -> bool:
-    """
-    Refresh functionality has been removed as it was a performance bottleneck.
-    This function now returns a success value without making any API calls.
-    
-    Args:
-        author_id: The ID of the author to refresh
-        api_url: Optional API URL
-        api_key: Optional API key
-        api_timeout: Optional API timeout
-        
-    Returns:
-        Always returns True to simulate success
-    """
-    logger.debug(f"Refresh functionality disabled for author ID: {author_id}")
-    # Always return success without making any API calls
-    return True
-
 def book_search(book_ids: List[int], api_url: Optional[str] = None, api_key: Optional[str] = None, api_timeout: Optional[int] = None) -> bool:
     """
     Trigger a search for one or more books.
@@ -535,8 +497,13 @@ def get_author_details(api_url: str, api_key: str, author_id: int, api_timeout:
     """Fetches details for a specific author from the Readarr API."""
     endpoint = f"{api_url}/api/v1/author/{author_id}"
     headers = {'X-Api-Key': api_key}
+    
+    # Get SSL verification setting
+    from src.primary.settings_manager import get_ssl_verify_setting
+    verify_ssl = get_ssl_verify_setting()
+    
     try:
-        response = requests.get(endpoint, headers=headers, timeout=api_timeout)
+        response = requests.get(endpoint, headers=headers, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status() # Raise HTTPError for bad responses (4xx or 5xx)
         author_data = response.json()
         logger.debug(f"Successfully fetched details for author ID {author_id}.")
@@ -567,22 +534,20 @@ def search_books(api_url: str, api_key: str, book_ids: List[int], api_timeout: i
         'name': 'BookSearch',
         'bookIds': book_ids
     }
+    
+    # Get SSL verification setting
+    from src.primary.settings_manager import get_ssl_verify_setting
+    verify_ssl = get_ssl_verify_setting()
+    
     try:
         # This uses requests.post directly, not arr_request. It's already correct.
-        response = requests.post(endpoint, headers=headers, json=payload, timeout=api_timeout)
+        response = requests.post(endpoint, headers=headers, json=payload, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         command_data = response.json()
         command_id = command_data.get('id')
         logger.info(f"Successfully triggered BookSearch command for book IDs: {book_ids}. Command ID: {command_id}")
         
-        # Increment API counter after successful request
-        try:
-            from src.primary.stats_manager import increment_hourly_cap
-            increment_hourly_cap("readarr", 1)
-            logger.debug(f"Incremented Readarr hourly API cap for book search ({len(book_ids)} books)")
-        except Exception as cap_error:
-            logger.error(f"Failed to increment hourly API cap for book search: {cap_error}")
-        
+        # API bar incremented by caller via increment_stat (one count per hunt/upgrade)
         return command_data # Return the full command object which includes the ID
     except requests.exceptions.RequestException as e:
         logger.error(f"Error triggering BookSearch command for book IDs {book_ids} via {endpoint}: {e}")
@@ -591,6 +556,65 @@ def search_books(api_url: str, api_key: str, book_ids: List[int], api_timeout: i
         logger.error(f"An unexpected error occurred triggering BookSearch for book IDs {book_ids}: {e}")
         return None
 
+def get_tag_id_by_label(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
+    """Get tag ID by label (does not create). Used for tag-based upgrade selection."""
+    try:
+        response = arr_request("tag", api_url=api_url, api_key=api_key, api_timeout=api_timeout, count_api=False)
+        if response:
+            for tag in response:
+                if tag.get('label') == tag_label:
+                    return tag.get('id')
+        return None
+    except Exception as e:
+        logger.error(f"Error getting tag '{tag_label}': {e}")
+        return None
+
+
+def get_exempt_tag_ids(api_url: str, api_key: str, api_timeout: int, exempt_tag_labels: list) -> dict:
+    """Resolve exempt tag labels to tag IDs. Returns dict tag_id -> label. Exact match. Issue #676.
+    Only counts tags that are actually entered (non-empty); empty list or all-whitespace = no exclusions.
+    """
+    if exempt_tag_labels is None:
+        return {}
+    if isinstance(exempt_tag_labels, str):
+        exempt_tag_labels = [exempt_tag_labels]
+    labels = [str(l).strip() for l in (exempt_tag_labels or []) if l is not None and str(l).strip()]
+    if not labels:
+        return {}
+    result = {}
+    for label in labels:
+        tid = get_tag_id_by_label(api_url, api_key, api_timeout, label)
+        if tid is not None:
+            result[tid] = label
+    return result
+
+
+def get_books_without_author_tag(
+    api_url: str, api_key: str, api_timeout: int, tag_label: str, monitored_only: bool
+) -> Optional[List[Dict]]:
+    """
+    Get books whose author DON'T have the given tag (Upgradinatorr-style: tag tracks processed).
+    After searching, the tag will be added to the author to mark them as processed.
+    """
+    tag_id = get_or_create_tag(api_url, api_key, api_timeout, tag_label)
+    if tag_id is None:
+        logger.error(f"Failed to get or create tag '{tag_label}' in Readarr.")
+        return None
+    books = arr_request("book", api_url=api_url, api_key=api_key, api_timeout=api_timeout, count_api=False)
+    if not books:
+        logger.error("Failed to retrieve books from Readarr API.")
+        return None
+    if not isinstance(books, list):
+        books = [books]
+    out = [
+        b for b in books
+        if tag_id not in (b.get('author') or {}).get('tags', [])
+        and (not monitored_only or b.get('monitored', False))
+    ]
+    logger.debug(f"Found {len(out)} books whose authors DON'T have tag '{tag_label}' (monitored_only={monitored_only}).")
+    return out
+
+
 def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
     """
     Get existing tag ID or create a new tag in Readarr.
@@ -724,7 +748,7 @@ def get_quality_profiles(api_url: str, api_key: str, api_timeout: int) -> Option
         logger.debug("Fetching quality profiles from Readarr...")
         
         # Use the qualityProfile endpoint - this doesn't count toward API limits since it's configuration data
-        # Note: Readarr uses API v1, not v3 like Sonarr/Radarr
+        # Note: Readarr uses API v1, not v3
         profiles = arr_request("qualityProfile", api_url=api_url, api_key=api_key, api_timeout=api_timeout, count_api=False)
         
         if profiles is None:
