@@ -9,11 +9,15 @@ import random
 from typing import List, Dict, Any, Set, Callable
 from src.primary.utils.logger import get_logger
 from src.primary.apps.readarr import api as readarr_api
-from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
-from src.primary.stateful_manager import is_processed, add_processed_id
+from src.primary.stats_manager import increment_stat
+from src.primary.stateful_manager import add_processed_id
 from src.primary.utils.history_utils import log_processed_media
-from src.primary.settings_manager import load_settings, get_advanced_setting
+from src.primary.settings_manager import load_settings
 from src.primary.state import check_state_reset
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items, filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item
 
 # Get logger for the app
 readarr_logger = get_logger("readarr")
@@ -38,38 +42,20 @@ def process_missing_books(
     # Reset state files if enough time has passed
     check_state_reset("readarr")
     
-    # Load settings to check if tagging is enabled
-    readarr_settings = load_settings("readarr")
-    tag_processed_items = readarr_settings.get("tag_processed_items", True)
-    
-    # Get the settings for the instance
-    general_settings = readarr_api.load_settings('general')
-    
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    instance_name = app_settings.get("instance_name", "Readarr Default")
+    # Extract common settings using shared utility
+    s = extract_app_settings(app_settings, "readarr", "hunt_missing_books", "Readarr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_missing_books = s['hunt_count']
+    tag_settings = s['tag_settings']
     
     readarr_logger.info(f"Using API timeout of {api_timeout} seconds for Readarr")
     
-    monitored_only = app_settings.get("monitored_only", True)
-    skip_future_releases = app_settings.get("skip_future_releases", True)
-    hunt_missing_books = app_settings.get("hunt_missing_books", 0)
-    
-    # Use advanced settings from database for command operations
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-
-    # Configuration logging removed to reduce log spam
-
-    if not api_url or not api_key:
-        readarr_logger.error("API URL or Key not configured in settings. Cannot process missing books.")
-        return False
-
-    # Skip if hunt_missing_books is set to 0
-    if hunt_missing_books <= 0:
-        readarr_logger.info("'hunt_missing_books' setting is 0 or less. Skipping missing book processing.")
+    if not validate_settings(api_url, api_key, hunt_missing_books, "readarr", readarr_logger):
         return False
 
     # Check for stop signal
@@ -93,20 +79,26 @@ def process_missing_books(
     
     readarr_logger.info(f"Retrieved {len(missing_books_data)} missing books from random page selection.")
 
+    # Filter out books whose author has an exempt tag (issue #676)
+    missing_books_data = filter_exempt_items(
+        missing_books_data, s['exempt_tags'], readarr_api,
+        api_url, api_key, api_timeout,
+        get_tags_fn=lambda b: (b.get("author") or {}).get("tags", []),
+        get_id_fn=lambda b: b.get("id"),
+        get_title_fn=lambda b: b.get("title", "Unknown"),
+        app_type="readarr", logger=readarr_logger
+    )
+
     # Check for stop signal after retrieving books
     if stop_check():
         readarr_logger.info("Stop requested after retrieving missing books. Aborting...")
         return False
 
-    # Filter out already processed books using stateful management (now book-based instead of author-based)
-    unprocessed_books = []
-    for book in missing_books_data:
-        book_id = str(book.get("id"))
-        if not is_processed("readarr", instance_name, book_id):
-            unprocessed_books.append(book)
-        else:
-            readarr_logger.debug(f"Skipping already processed book ID: {book_id}")
-
+    # Filter out already processed books using shared utility
+    unprocessed_books = filter_unprocessed(
+        missing_books_data, "readarr", instance_key,
+        get_id_fn=lambda b: b.get("id"), logger=readarr_logger
+    )
     readarr_logger.info(f"Found {len(unprocessed_books)} unprocessed missing books out of {len(missing_books_data)} total.")
     
     if not unprocessed_books:
@@ -133,18 +125,8 @@ def process_missing_books(
 
     # Process each individual book
     for book in books_to_process:
-        if stop_check():
-            readarr_logger.info("Stop signal received, aborting Readarr missing cycle.")
+        if not should_continue_processing("readarr", stop_check, readarr_logger):
             break
-        
-        # Check API limit before processing each book
-        try:
-            if check_hourly_cap_exceeded("readarr"):
-                readarr_logger.warning(f"🛑 Readarr API hourly limit reached - stopping missing books processing after {processed_count} books")
-                break
-        except Exception as e:
-            readarr_logger.error(f"Error checking hourly API cap: {e}")
-            # Continue processing if cap check fails - safer than stopping
 
         book_id = book.get("id")
         book_title = book.get("title", f"Unknown Book ID {book_id}")
@@ -160,7 +142,7 @@ def process_missing_books(
         readarr_logger.info(f"  - Searching for individual book: '{book_title}'...")
         
         # Mark book as processed BEFORE triggering search to prevent duplicates
-        add_processed_id("readarr", instance_name, str(book_id))
+        add_processed_id("readarr", instance_key, str(book_id))
         readarr_logger.debug(f"Added book ID {book_id} to processed list for {instance_name}")
         
         # Search for the specific book (using book search instead of author search)
@@ -170,21 +152,16 @@ def process_missing_books(
             # Extract command ID if the result is a dictionary, otherwise use the result directly
             command_id = search_command_result.get('id') if isinstance(search_command_result, dict) else search_command_result
             readarr_logger.info(f"Triggered book search command {command_id} for '{book_title}' by {author_name}.")
-            increment_stat("readarr", "hunted")
+            increment_stat("readarr", "hunted", 1, instance_key)
             
-            # Tag the book's author if enabled (keep author tagging as it's still useful)
-            if tag_processed_items and author_id:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("readarr", "missing", "huntarr-missing")
-                try:
-                    readarr_api.tag_processed_author(api_url, api_key, api_timeout, author_id, custom_tag)
-                    readarr_logger.debug(f"Tagged author {author_id} with '{custom_tag}'")
-                except Exception as e:
-                    readarr_logger.warning(f"Failed to tag author {author_id} with '{custom_tag}': {e}")
+            if author_id:
+                try_tag_item(tag_settings, "missing", readarr_api.tag_processed_author,
+                             api_url, api_key, api_timeout, author_id,
+                             readarr_logger, f"author {author_id}")
             
             # Log history entry for this specific book
             media_name = f"{author_name} - {book_title}"
-            log_processed_media("readarr", media_name, book_id, instance_name, "missing")
+            log_processed_media("readarr", media_name, book_id, instance_key, "missing", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             readarr_logger.debug(f"Logged missing book history entry: {media_name} (ID: {book_id})")
             
             processed_count += 1
