@@ -11,10 +11,14 @@ from typing import List, Dict, Any, Set, Callable, Union, Optional
 from src.primary.utils.logger import get_logger
 from src.primary.apps.readarr import api as readarr_api
 from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
-from src.primary.stateful_manager import is_processed, add_processed_id
+from src.primary.stateful_manager import add_processed_id
 from src.primary.utils.history_utils import log_processed_media
 from src.primary.state import check_state_reset
-from src.primary.settings_manager import load_settings # Import load_settings function
+from src.primary.settings_manager import load_settings
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_exempt_items, filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item
 
 # Get logger for the app
 readarr_logger = get_logger("readarr")
@@ -40,43 +44,63 @@ def process_cutoff_upgrades(
     
     processed_any = False
     
-    # Load general settings to get centralized timeout
-    general_settings = load_settings('general')
-    
-    # Load settings to check if tagging is enabled
-    readarr_settings = load_settings("readarr")
-    tag_processed_items = readarr_settings.get("tag_processed_items", True)
-    
-    # Get the API credentials for this instance
-    api_url = app_settings.get('api_url', '')
-    api_key = app_settings.get('api_key', '')
-    
-    # Use the centralized timeout from general settings with app-specific as fallback
-    api_timeout = general_settings.get("api_timeout", app_settings.get("api_timeout", 90))  # Use centralized timeout
+    # Extract common settings using shared utility
+    s = extract_app_settings(app_settings, "readarr", "hunt_upgrade_books", "Readarr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_upgrade_books = s['hunt_count']
+    tag_settings = s['tag_settings']
     
     readarr_logger.info(f"Using API timeout of {api_timeout} seconds for Readarr")
     
-    # Extract necessary settings
-    instance_name = app_settings.get("instance_name", "Readarr Default")
-    monitored_only = app_settings.get("monitored_only", True)
-    # skip_author_refresh setting removed as it was a performance bottleneck
-    hunt_upgrade_books = app_settings.get("hunt_upgrade_books", 0)
+    # App-specific settings
+    upgrade_selection_method = (app_settings.get("upgrade_selection_method") or "cutoff").strip().lower()
+    if upgrade_selection_method not in ("cutoff", "tags"):
+        upgrade_selection_method = "cutoff"
+    upgrade_tag = (app_settings.get("upgrade_tag") or "").strip()
     command_wait_delay = app_settings.get("command_wait_delay", 5)
     command_wait_attempts = app_settings.get("command_wait_attempts", 12)
     
     # Get books eligible for upgrade
-    readarr_logger.info("Retrieving books eligible for quality upgrade...")
-    # Pass API credentials explicitly
-    upgrade_eligible_data = readarr_api.get_cutoff_unmet_books(api_url=api_url, api_key=api_key, api_timeout=api_timeout)
-    
-    if upgrade_eligible_data is None: # Check if the API call failed (assuming it returns None on error)
-        readarr_logger.error("Error retrieving books eligible for upgrade from Readarr API.")
-        return False
-    elif not upgrade_eligible_data: # Check if the list is empty
-        readarr_logger.info("No books found eligible for upgrade.")
-        return False
-        
-    readarr_logger.info(f"Found {len(upgrade_eligible_data)} books eligible for quality upgrade.")
+    if upgrade_selection_method == "tags":
+        if not upgrade_tag:
+            readarr_logger.warning("Upgrade selection method is 'Tags' but no upgrade tag is configured. Skipping.")
+            return False
+        readarr_logger.info(f"Retrieving books whose authors DON'T have tag \"{upgrade_tag}\" (Upgradinatorr-style: tag tracks processed)...")
+        upgrade_eligible_data = readarr_api.get_books_without_author_tag(
+            api_url, api_key, api_timeout, upgrade_tag, monitored_only
+        )
+        if upgrade_eligible_data is None:
+            return False
+        if not upgrade_eligible_data:
+            readarr_logger.info(f"No books found whose authors lack the tag \"{upgrade_tag}\" (all have been processed).")
+            return False
+        readarr_logger.info(f"Found {len(upgrade_eligible_data)} books whose authors DON'T have tag \"{upgrade_tag}\".")
+    else:
+        readarr_logger.info("Retrieving books eligible for quality upgrade...")
+        # Pass API credentials explicitly
+        upgrade_eligible_data = readarr_api.get_cutoff_unmet_books(api_url=api_url, api_key=api_key, api_timeout=api_timeout)
+        if upgrade_eligible_data is None:  # Check if the API call failed (assuming it returns None on error)
+            readarr_logger.error("Error retrieving books eligible for upgrade from Readarr API.")
+            return False
+        elif not upgrade_eligible_data:  # Check if the list is empty
+            readarr_logger.info("No books found eligible for upgrade.")
+            return False
+        readarr_logger.info(f"Found {len(upgrade_eligible_data)} books eligible for quality upgrade.")
+
+    # Filter out books whose author has an exempt tag (issue #676)
+    upgrade_eligible_data = filter_exempt_items(
+        upgrade_eligible_data, s['exempt_tags'], readarr_api,
+        api_url, api_key, api_timeout,
+        get_tags_fn=lambda b: (b.get("author") or {}).get("tags", []),
+        get_id_fn=lambda b: b.get("id"),
+        get_title_fn=lambda b: b.get("title", "Unknown"),
+        app_type="readarr", logger=readarr_logger
+    )
 
     # Filter out future releases if configured
     skip_future_releases = app_settings.get("skip_future_releases", True)
@@ -119,15 +143,11 @@ def process_cutoff_upgrades(
         readarr_logger.info("No upgradeable books found to process (after potential filtering). Skipping.")
         return False
         
-    # Filter out already processed books using stateful management
-    unprocessed_books = []
-    for book in upgrade_eligible_data:
-        book_id = str(book.get("id"))
-        if not is_processed("readarr", instance_name, book_id):
-            unprocessed_books.append(book)
-        else:
-            readarr_logger.debug(f"Skipping already processed book ID: {book_id}")
-    
+    # Filter out already processed books using shared utility
+    unprocessed_books = filter_unprocessed(
+        upgrade_eligible_data, "readarr", instance_key,
+        get_id_fn=lambda b: b.get("id"), logger=readarr_logger
+    )
     readarr_logger.info(f"Found {len(unprocessed_books)} unprocessed books out of {len(upgrade_eligible_data)} total books eligible for upgrade.")
     
     if not unprocessed_books:
@@ -155,7 +175,7 @@ def process_cutoff_upgrades(
 
     # Mark books as processed BEFORE triggering any searches
     for book_id in book_ids_to_search:
-        add_processed_id("readarr", instance_name, str(book_id))
+        add_processed_id("readarr", instance_key, str(book_id))
         readarr_logger.debug(f"Added book ID {book_id} to processed list for {instance_name}")
         
     # Now trigger the search
@@ -164,23 +184,33 @@ def process_cutoff_upgrades(
     if search_command_result:
         command_id = search_command_result
         readarr_logger.info(f"Triggered upgrade search command {command_id} for {len(book_ids_to_search)} books.")
-        increment_stat("readarr", "upgraded")
+        increment_stat("readarr", "upgraded", 1, instance_key)
         
-        # Tag authors if enabled (from books)
-        if tag_processed_items:
-            from src.primary.settings_manager import get_custom_tag
-            custom_tag = get_custom_tag("readarr", "upgrade", "huntarr-upgraded")
-            tagged_authors = set()  # Track which authors we've already tagged
+        # For tag-based method: add the upgrade tag to authors to mark as processed (Upgradinatorr-style)
+        if upgrade_selection_method == "tags" and upgrade_tag:
+            tagged_authors = set()
             for book in books_to_process:
                 author_id = book.get('authorId')
                 if author_id and author_id not in tagged_authors:
                     try:
-                        readarr_api.tag_processed_author(api_url, api_key, api_timeout, author_id, custom_tag)
-                        readarr_logger.debug(f"Tagged author {author_id} with '{custom_tag}'")
-                        tagged_authors.add(author_id)
+                        tag_id = readarr_api.get_or_create_tag(api_url, api_key, api_timeout, upgrade_tag)
+                        if tag_id:
+                            readarr_api.add_tag_to_author(api_url, api_key, api_timeout, author_id, tag_id)
+                            readarr_logger.debug(f"Added upgrade tag '{upgrade_tag}' to author {author_id} to mark as processed")
+                            tagged_authors.add(author_id)
                     except Exception as e:
-                        readarr_logger.warning(f"Failed to tag author {author_id} with '{custom_tag}': {e}")
-            
+                        readarr_logger.warning(f"Failed to add upgrade tag '{upgrade_tag}' to author {author_id}: {e}")
+        
+        # Tag authors with huntarr-upgraded if enabled (unified tagging)
+        tagged_authors_upgraded = set()
+        for book in books_to_process:
+            author_id = book.get('authorId')
+            if author_id and author_id not in tagged_authors_upgraded:
+                try_tag_item(tag_settings, "upgraded", readarr_api.tag_processed_author,
+                             api_url, api_key, api_timeout, author_id,
+                             readarr_logger, f"author {author_id}")
+                tagged_authors_upgraded.add(author_id)
+
         # Log to history system for each book
         for book in books_to_process:
             # Ensure we have a valid author name - if missing, fetch it
@@ -204,7 +234,7 @@ def process_cutoff_upgrades(
             media_name = f"{author_name} - {book_title}"
             
             # Include full details in history entry
-            log_processed_media("readarr", media_name, book.get("id"), instance_name, "upgrade")
+            log_processed_media("readarr", media_name, book.get("id"), instance_key, "upgrade", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             readarr_logger.debug(f"Logged quality upgrade to history for '{media_name}' (Book ID: {book.get('id')})")
 
             
