@@ -78,11 +78,13 @@ def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, met
             # Check for successful response
             response.raise_for_status()
             
-            # Increment API counter only if count_api is True and request was successful
+            # Increment API counter only if count_api is True and request was successful (per-instance when in instance context)
             if count_api:
                 try:
                     from src.primary.stats_manager import increment_hourly_cap
-                    increment_hourly_cap("sonarr")
+                    from src.primary.utils.clean_logger import get_instance_name_for_cap
+                    instance_name = get_instance_name_for_cap()
+                    increment_hourly_cap("sonarr", 1, instance_name=instance_name)
                 except Exception as e:
                     sonarr_logger.warning(f"Failed to increment API counter for sonarr: {e}")
             
@@ -121,29 +123,8 @@ def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, met
 
 def check_connection(api_url: str, api_key: str, api_timeout: int) -> bool:
     """Checks connection by fetching system status."""
-    if not api_url:
-        sonarr_logger.error("API URL is empty or not set")
-        return False
-    if not api_key:
-        sonarr_logger.error("API Key is empty or not set")
-        return False
-
-    try:
-        # Use a shorter timeout for a quick connection check
-        quick_timeout = min(api_timeout, 15) 
-        status = get_system_status(api_url, api_key, quick_timeout)
-        if status and isinstance(status, dict) and 'version' in status:
-             # Log success only if debug is enabled to avoid clutter
-             sonarr_logger.debug(f"Connection check successful for {api_url}. Version: {status.get('version')}")
-             return True
-        else:
-             # Log details if the status response was unexpected
-             sonarr_logger.warning(f"Connection check for {api_url} returned unexpected status: {str(status)[:200]}")
-             return False
-    except Exception as e:
-        # Error should have been logged by arr_request, just indicate failure
-        sonarr_logger.error(f"Connection check failed for {api_url}")
-        return False
+    from src.primary.apps._common.arr_api import check_connection as _check
+    return _check(api_url, api_key, api_timeout, "sonarr", sonarr_logger)
 
 def get_system_status(api_url: str, api_key: str, api_timeout: int) -> Dict:
     """
@@ -279,6 +260,7 @@ def get_missing_episodes(api_url: str, api_key: str, api_timeout: int, monitored
     while True:
         retry_count = 0
         success = False
+        records = []  # Ensure defined for "if not records" after inner loop (e.g. on request exception)
         
         while retry_count <= retries_per_page and not success:
             # Parameters for the request
@@ -286,20 +268,23 @@ def get_missing_episodes(api_url: str, api_key: str, api_timeout: int, monitored
                 "page": page,
                 "pageSize": page_size,
                 "includeSeries": "true",
-                "monitored": monitored_only
+                "monitored": str(monitored_only).lower()  # Sonarr API expects "true"/"false"
             }
             
             # Add series ID filter if provided
             if series_id is not None:
                 params["seriesId"] = series_id
             
+            # Get SSL verification setting
+            verify_ssl = get_ssl_verify_setting()
+            
             # Ensure proper URL construction with scheme
             base_url = api_url.rstrip('/')
             url = f"{base_url}/api/v3/{endpoint.lstrip('/')}"
             sonarr_logger.debug(f"Requesting missing episodes page {page} (attempt {retry_count+1}/{retries_per_page+1})")
             
             try:
-                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
+                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout, verify=verify_ssl)
                 response.raise_for_status() # Check for HTTP errors (4xx or 5xx)
                 
                 if not response.content:
@@ -412,11 +397,15 @@ def get_cutoff_unmet_episodes(api_url: str, api_key: str, api_timeout: int, moni
                 "sortDir": "asc",
                 "monitored": monitored_only
             }
+            
+            # Get SSL verification setting
+            verify_ssl = get_ssl_verify_setting()
+            
             url = f"{api_url}/api/v3/{endpoint}"
             sonarr_logger.debug(f"Requesting cutoff unmet page {page} (attempt {retry_count+1}/{retries_per_page+1})")
 
             try:
-                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
+                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout, verify=verify_ssl)
                 sonarr_logger.debug(f"Sonarr API response status code for cutoff unmet page {page}: {response.status_code}")
                 response.raise_for_status() # Check for HTTP errors
                 
@@ -555,9 +544,12 @@ def get_cutoff_unmet_episodes_random_page(api_url: str, api_key: str, api_timeou
     }
     url = f"{api_url}/api/v3/{endpoint}"
     
+    # Get SSL verification setting
+    verify_ssl = get_ssl_verify_setting()
+    
     try:
         # Get total record count from a minimal query
-        response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
+        response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         data = response.json()
         total_records = data.get('totalRecords', 0)
@@ -586,7 +578,7 @@ def get_cutoff_unmet_episodes_random_page(api_url: str, api_key: str, api_timeou
             "monitored": monitored_only
         }
         
-        response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
+        response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         
         data = response.json()
@@ -648,15 +640,19 @@ def get_missing_episodes_random_page(api_url: str, api_key: str, api_timeout: in
         "page": 1,
         "pageSize": 1,
         "includeSeries": "true",  # Include series info for filtering
-        "monitored": monitored_only
+        "monitored": str(monitored_only).lower()  # Sonarr API expects "true"/"false"
     }
-    url = f"{api_url}/api/v3/{endpoint}"
+    base_url = api_url.rstrip('/')
+    url = f"{base_url}/api/v3/{endpoint.lstrip('/')}"
+    
+    # Get SSL verification setting
+    verify_ssl = get_ssl_verify_setting()
     
     for attempt in range(retries + 1):
         try:
             # Get total record count from a minimal query
             sonarr_logger.debug(f"Getting missing episodes count (attempt {attempt+1}/{retries+1})")
-            response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
+            response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout, verify=verify_ssl)
             response.raise_for_status()
             
             if not response.content:
@@ -668,14 +664,17 @@ def get_missing_episodes_random_page(api_url: str, api_key: str, api_timeout: in
                 
             try:
                 data = response.json()
-                total_records = data.get('totalRecords', 0)
+                # Support both totalRecords (Sonarr v3) and total (some versions)
+                total_records = data.get('totalRecords', data.get('total', 0))
+                if isinstance(total_records, dict):
+                    total_records = 0
                 
                 if total_records == 0:
                     sonarr_logger.info("No missing episodes found in Sonarr.")
                     return []
                     
                 # Calculate total pages with our desired page size
-                total_pages = (total_records + page_size - 1) // page_size
+                total_pages = max(1, (total_records + page_size - 1) // page_size)
                 sonarr_logger.info(f"Found {total_records} total missing episodes across {total_pages} pages")
                 
                 if total_pages == 0:
@@ -691,13 +690,13 @@ def get_missing_episodes_random_page(api_url: str, api_key: str, api_timeout: in
                     "page": random_page,
                     "pageSize": page_size,
                     "includeSeries": "true",
-                    "monitored": monitored_only
+                    "monitored": str(monitored_only).lower()  # Sonarr API expects "true"/"false"
                 }
                 
                 if series_id is not None:
                     params["seriesId"] = series_id
                 
-                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
+                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout, verify=verify_ssl)
                 response.raise_for_status()
                 
                 if not response.content:
@@ -760,8 +759,9 @@ def get_missing_episodes_random_page(api_url: str, api_key: str, api_timeout: in
     sonarr_logger.error("All attempts to get missing episodes failed")
     return []
 
-def search_episode(api_url: str, api_key: str, api_timeout: int, episode_ids: List[int]) -> Optional[Union[int, str]]:
-    """Trigger a search for specific episodes in Sonarr."""
+def search_episode(api_url: str, api_key: str, api_timeout: int, episode_ids: List[int],
+                   instance_name: Optional[str] = None) -> Optional[Union[int, str]]:
+    """Trigger a search for specific episodes in Sonarr. instance_name used for per-instance API cap."""
     if not episode_ids:
         sonarr_logger.warning("No episode IDs provided for search.")
         return None
@@ -782,19 +782,15 @@ def search_episode(api_url: str, api_key: str, api_timeout: int, episode_ids: Li
             "name": "EpisodeSearch",
             "episodeIds": episode_ids
         }
-        response = requests.post(endpoint, headers={"X-Api-Key": api_key}, json=payload, timeout=api_timeout)
+        
+        # Get SSL verification setting
+        verify_ssl = get_ssl_verify_setting()
+        
+        response = requests.post(endpoint, headers={"X-Api-Key": api_key}, json=payload, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         command_id = response.json().get('id')
         sonarr_logger.info(f"Triggered Sonarr search for episode IDs: {episode_ids}. Command ID: {command_id}")
-        
-        # Increment API counter after successful request
-        try:
-            from src.primary.stats_manager import increment_hourly_cap
-            increment_hourly_cap("sonarr", 1)
-            sonarr_logger.debug(f"Incremented Sonarr hourly API cap for episode search ({len(episode_ids)} episodes)")
-        except Exception as cap_error:
-            sonarr_logger.error(f"Failed to increment hourly API cap for episode search: {cap_error}")
-        
+        # API bar is incremented by increment_stat / increment_stat_only when hunt/upgrade is recorded (one count per trigger)
         return command_id
     except requests.exceptions.RequestException as e:
         sonarr_logger.error(f"Error triggering Sonarr search for episode IDs {episode_ids}: {e}")
@@ -807,7 +803,11 @@ def get_command_status(api_url: str, api_key: str, api_timeout: int, command_id:
     """Get the status of a Sonarr command."""
     try:
         endpoint = f"{api_url}/api/v3/command/{command_id}"
-        response = requests.get(endpoint, headers={"X-Api-Key": api_key}, timeout=api_timeout)
+        
+        # Get SSL verification setting
+        verify_ssl = get_ssl_verify_setting()
+        
+        response = requests.get(endpoint, headers={"X-Api-Key": api_key}, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         status = response.json()
         sonarr_logger.debug(f"Checked Sonarr command status for ID {command_id}: {status.get('status')}")
@@ -827,7 +827,11 @@ def get_download_queue_size(api_url: str, api_key: str, api_timeout: int) -> int
     for attempt in range(retries + 1):
         try:
             endpoint = f"{api_url}/api/v3/queue?page=1&pageSize=1" # Just get total count, don't need records
-            response = requests.get(endpoint, headers={"X-Api-Key": api_key}, params={"includeSeries": "false"}, timeout=api_timeout)
+            
+            # Get SSL verification setting
+            verify_ssl = get_ssl_verify_setting()
+            
+            response = requests.get(endpoint, headers={"X-Api-Key": api_key}, params={"includeSeries": "false"}, timeout=api_timeout, verify=verify_ssl)
             response.raise_for_status()
             
             if not response.content:
@@ -867,19 +871,15 @@ def get_download_queue_size(api_url: str, api_key: str, api_timeout: int) -> int
     sonarr_logger.error(f"All {retries+1} attempts to get download queue size failed")
     return -1
 
-def refresh_series(api_url: str, api_key: str, api_timeout: int, series_id: int) -> Optional[Union[int, str]]:
-    """Refresh functionality has been removed as it was a performance bottleneck.
-    This function now returns a placeholder success value without making any API calls.
-    """
-    sonarr_logger.debug(f"Refresh functionality disabled for series ID: {series_id}")
-    # Return a placeholder command ID (123) to simulate success without actually refreshing
-    return 123
-
 def get_series_by_id(api_url: str, api_key: str, api_timeout: int, series_id: int) -> Optional[Dict[str, Any]]:
     """Get series details by ID from Sonarr."""
     try:
         endpoint = f"{api_url}/api/v3/series/{series_id}"
-        response = requests.get(endpoint, headers={"X-Api-Key": api_key}, timeout=api_timeout)
+        
+        # Get SSL verification setting
+        verify_ssl = get_ssl_verify_setting()
+        
+        response = requests.get(endpoint, headers={"X-Api-Key": api_key}, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         series_data = response.json()
         sonarr_logger.debug(f"Fetched details for Sonarr series ID: {series_id}")
@@ -891,8 +891,9 @@ def get_series_by_id(api_url: str, api_key: str, api_timeout: int, series_id: in
         sonarr_logger.error(f"An unexpected error occurred while getting Sonarr series details: {e}")
         return None
 
-def search_season(api_url: str, api_key: str, api_timeout: int, series_id: int, season_number: int) -> Optional[Union[int, str]]:
-    """Trigger a search for a specific season in Sonarr."""
+def search_season(api_url: str, api_key: str, api_timeout: int, series_id: int, season_number: int,
+                  instance_name: Optional[str] = None) -> Optional[Union[int, str]]:
+    """Trigger a search for a specific season in Sonarr. instance_name used for per-instance API cap."""
     
     # Check API limit before making request
     try:
@@ -911,20 +912,15 @@ def search_season(api_url: str, api_key: str, api_timeout: int, series_id: int,
             "seriesId": series_id,
             "seasonNumber": season_number
         }
-        response = requests.post(endpoint, headers={"X-Api-Key": api_key}, json=payload, timeout=api_timeout)
+        
+        # Get SSL verification setting
+        verify_ssl = get_ssl_verify_setting()
+        
+        response = requests.post(endpoint, headers={"X-Api-Key": api_key}, json=payload, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         command_id = response.json().get('id')
         sonarr_logger.info(f"Triggered Sonarr season search for series ID: {series_id}, season: {season_number}. Command ID: {command_id}")
-        
-        # CRITICAL FIX: Track the API call in hourly cap counter
-        # This was missing and causing API counter to be inaccurate for season packs
-        try:
-            from src.primary.stats_manager import increment_hourly_cap
-            increment_hourly_cap("sonarr", 1)
-            sonarr_logger.debug(f"Incremented Sonarr hourly API cap for season search (series: {series_id}, season: {season_number})")
-        except Exception as cap_error:
-            sonarr_logger.error(f"Failed to increment hourly API cap for season search: {cap_error}")
-        
+        # API bar is incremented by increment_stat / increment_stat_only when hunt/upgrade is recorded (one count per trigger)
         return command_id
     except requests.exceptions.RequestException as e:
         sonarr_logger.error(f"Error triggering Sonarr season search for series ID {series_id}, season {season_number}: {e}")
@@ -973,11 +969,15 @@ def get_cutoff_unmet_episodes_for_series(api_url: str, api_key: str, api_timeout
                 "seriesId": series_id,  # Filter by series ID - this limits results to only this series
                 "monitored": monitored_only
             }
+            
+            # Get SSL verification setting
+            verify_ssl = get_ssl_verify_setting()
+            
             url = f"{api_url}/api/v3/{endpoint}"
             sonarr_logger.debug(f"Requesting cutoff unmet page {page} for series {series_id} (attempt {retry_count+1}/{retries_per_page+1})")
             
             try:
-                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout)
+                response = requests.get(url, headers={"X-Api-Key": api_key}, params=params, timeout=api_timeout, verify=verify_ssl)
                 sonarr_logger.debug(f"Sonarr API response status code for cutoff unmet page {page}: {response.status_code}")
                 response.raise_for_status() # Check for HTTP errors
                 
@@ -1125,7 +1125,11 @@ def get_series_with_missing_episodes(api_url: str, api_key: str, api_timeout: in
         # Get all episodes for this series
         try:
             endpoint = f"{api_url}/api/v3/episode?seriesId={series_id}"
-            response = requests.get(endpoint, headers={"X-Api-Key": api_key}, timeout=api_timeout)
+            
+            # Get SSL verification setting
+            verify_ssl = get_ssl_verify_setting()
+            
+            response = requests.get(endpoint, headers={"X-Api-Key": api_key}, timeout=api_timeout, verify=verify_ssl)
             response.raise_for_status()
             
             if not response.content:
@@ -1178,6 +1182,88 @@ def get_series_with_missing_episodes(api_url: str, api_key: str, api_timeout: in
     sonarr_logger.info(f"Examined {examined_count} series ({selection_mode} mode) and found {len(series_with_missing)} with missing episodes")
     return series_with_missing
 
+def get_tag_id_by_label(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
+    """Get tag ID by label (does not create). Used for tag-based upgrade selection."""
+    try:
+        response = arr_request(api_url, api_key, api_timeout, "tag", count_api=False)
+        if response:
+            for tag in response:
+                if tag.get('label') == tag_label:
+                    return tag.get('id')
+        return None
+    except Exception as e:
+        sonarr_logger.error(f"Error getting tag '{tag_label}': {e}")
+        return None
+
+
+def get_exempt_tag_ids(api_url: str, api_key: str, api_timeout: int, exempt_tag_labels: list) -> dict:
+    """Resolve exempt tag labels to tag IDs. Returns dict tag_id -> label. Exact match. Issue #676.
+    Only counts tags that are actually entered (non-empty); empty list or all-whitespace = no exclusions.
+    """
+    if exempt_tag_labels is None:
+        return {}
+    # Normalize: accept list or single string; only consider non-empty stripped labels
+    if isinstance(exempt_tag_labels, str):
+        exempt_tag_labels = [exempt_tag_labels]
+    labels = [str(l).strip() for l in (exempt_tag_labels or []) if l is not None and str(l).strip()]
+    if not labels:
+        return {}
+    result = {}
+    for label in labels:
+        tid = get_tag_id_by_label(api_url, api_key, api_timeout, label)
+        if tid is not None:
+            result[tid] = label
+    return result
+
+
+def get_series_without_tag(
+    api_url: str, api_key: str, api_timeout: int, tag_label: str, monitored_only: bool
+) -> Optional[List[Dict[str, Any]]]:
+    """
+    Get series that DON'T have the given tag (Upgradinatorr-style: tag tracks processed).
+    After searching, the tag will be added to mark them as processed.
+    """
+    tag_id = get_or_create_tag(api_url, api_key, api_timeout, tag_label)
+    if tag_id is None:
+        sonarr_logger.error(f"Failed to get or create tag '{tag_label}' in Sonarr.")
+        return None
+    all_series = get_series(api_url, api_key, api_timeout)
+    if all_series is None:
+        sonarr_logger.error("Failed to retrieve series from Sonarr API.")
+        return None
+    if not isinstance(all_series, list):
+        all_series = [all_series]
+    out = [
+        s for s in all_series
+        if tag_id not in s.get('tags', [])
+        and (not monitored_only or s.get('monitored', False))
+    ]
+    sonarr_logger.debug(f"Found {len(out)} series WITHOUT tag '{tag_label}' (monitored_only={monitored_only}).")
+    return out
+
+
+def series_search(api_url: str, api_key: str, api_timeout: int, series_id: int) -> Optional[int]:
+    """Trigger SeriesSearch for a series (all episodes)."""
+    try:
+        endpoint = f"{api_url.rstrip('/')}/api/v3/command"
+        payload = {"name": "SeriesSearch", "seriesId": series_id}
+        
+        # Get SSL verification setting
+        verify_ssl = get_ssl_verify_setting()
+        
+        response = requests.post(endpoint, headers={"X-Api-Key": api_key}, json=payload, timeout=api_timeout, verify=verify_ssl)
+        response.raise_for_status()
+        command_id = response.json().get('id')
+        sonarr_logger.info(f"Triggered Sonarr SeriesSearch for series ID: {series_id}. Command ID: {command_id}")
+        return command_id
+    except requests.exceptions.RequestException as e:
+        sonarr_logger.error(f"Error triggering Sonarr SeriesSearch for series ID {series_id}: {e}")
+        return None
+    except Exception as e:
+        sonarr_logger.error(f"Unexpected error triggering Sonarr SeriesSearch: {e}")
+        return None
+
+
 def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
     """
     Get existing tag ID or create a new tag in Sonarr.
