@@ -14,6 +14,7 @@ from src.primary.settings_manager import load_settings, get_advanced_setting
 from src.primary.utils.history_utils import log_processed_media
 from src.primary.stats_manager import increment_stat, increment_stat_only, check_hourly_cap_exceeded
 from src.primary.stateful_manager import is_processed, add_processed_id
+from src.primary.apps._common.tagging import try_tag_item, extract_tag_settings
 from src.primary.apps.sonarr import api as sonarr_api
 
 def should_delay_episode_search(air_date_str: str, delay_days: int) -> bool:
@@ -51,11 +52,38 @@ def should_delay_episode_search(air_date_str: str, delay_days: int) -> bool:
 # Get logger for the Sonarr app
 sonarr_logger = get_logger("sonarr")
 
+
+def _normalize_exempt_tags(exempt_tags: list) -> list:
+    """Only count tags that are entered (non-empty). Empty/whitespace = no exclusions. Issue #805."""
+    if exempt_tags is None:
+        return []
+    return [t for t in (exempt_tags if isinstance(exempt_tags, list) else [exempt_tags])
+            if isinstance(t, str) and (t or "").strip()]
+
+
+def _get_exempt_series_ids(api_url: str, api_key: str, api_timeout: int, exempt_tags: list) -> set:
+    """Return set of series IDs that have any exempt tag. Empty exempt list = no series excluded."""
+    exempt_series_ids = set()
+    exempt_tags = _normalize_exempt_tags(exempt_tags)
+    if not exempt_tags:
+        return exempt_series_ids
+    exempt_id_to_label = sonarr_api.get_exempt_tag_ids(api_url, api_key, api_timeout, exempt_tags)
+    if not exempt_id_to_label:
+        return exempt_series_ids
+    all_series = sonarr_api.get_series(api_url, api_key, api_timeout)
+    for s in (all_series or []):
+        for tid in s.get("tags", []):
+            if tid in exempt_id_to_label:
+                exempt_series_ids.add(s.get("id"))
+                break
+    return exempt_series_ids
+
+
 def process_missing_episodes(
     api_url: str,
     api_key: str,
     instance_name: str,
-    api_timeout: int = get_advanced_setting("api_timeout", 120),
+    api_timeout: int = 120,
     monitored_only: bool = True,
     skip_future_episodes: bool = True,
 
@@ -64,7 +92,13 @@ def process_missing_episodes(
     air_date_delay_days: int = 0,
     command_wait_delay: int = get_advanced_setting("command_wait_delay", 1),
     command_wait_attempts: int = get_advanced_setting("command_wait_attempts", 600),
-    stop_check: Callable[[], bool] = lambda: False
+    stop_check: Callable[[], bool] = lambda: False,
+    tag_processed_items: bool = False,
+    tag_enable_missing: bool = False,
+    tag_enable_shows_missing: bool = False,
+    custom_tags: dict = None,
+    exempt_tags: list = None,
+    instance_display_name: Optional[str] = None,
 ) -> bool:
     """
     Process missing episodes for Sonarr.
@@ -77,14 +111,28 @@ def process_missing_episodes(
         
     sonarr_logger.info(f"Checking for {hunt_missing_items} missing episodes in {hunt_missing_mode} mode for instance '{instance_name}'...")
 
+    # Use custom tags if provided, otherwise use defaults
+    if custom_tags is None:
+        custom_tags = {
+            "missing": "huntarr-missing",
+            "upgrade": "huntarr-upgrade",
+            "shows_missing": "huntarr-shows-missing"
+        }
+
+    # Only count tags that are entered; empty = process every item (Issue #805)
+    exempt_tags = _normalize_exempt_tags(exempt_tags or [])
+
     # Handle different modes
+    display_name = instance_display_name or instance_name
     if hunt_missing_mode == "seasons_packs":
         # Handle season pack searches (using SeasonSearch command)
         sonarr_logger.info("Season [Packs] mode selected - searching for complete season packs")
         return process_missing_seasons_packs_mode(
             api_url, api_key, instance_name, api_timeout, monitored_only, 
             skip_future_episodes, hunt_missing_items, air_date_delay_days,
-            command_wait_delay, command_wait_attempts, stop_check
+            command_wait_delay, command_wait_attempts, stop_check,
+            tag_processed_items, tag_enable_missing, tag_enable_shows_missing, custom_tags, exempt_tags=exempt_tags,
+            hunt_missing_mode=hunt_missing_mode, instance_display_name=display_name
         )
     elif hunt_missing_mode == "shows":
         # Handle show-based missing items (all episodes from a show)
@@ -92,7 +140,9 @@ def process_missing_episodes(
         return process_missing_shows_mode(
             api_url, api_key, instance_name, api_timeout, monitored_only, 
             skip_future_episodes, hunt_missing_items, air_date_delay_days,
-            command_wait_delay, command_wait_attempts, stop_check
+            command_wait_delay, command_wait_attempts, stop_check,
+            tag_processed_items, tag_enable_missing, tag_enable_shows_missing, custom_tags, exempt_tags=exempt_tags,
+            instance_display_name=display_name
         )
     elif hunt_missing_mode == "episodes":
         # Handle individual episode processing (reinstated with warnings)
@@ -100,7 +150,9 @@ def process_missing_episodes(
         return process_missing_episodes_mode(
             api_url, api_key, instance_name, api_timeout, monitored_only, 
             skip_future_episodes, hunt_missing_items, air_date_delay_days,
-            command_wait_delay, command_wait_attempts, stop_check
+            command_wait_delay, command_wait_attempts, stop_check,
+            tag_processed_items, tag_enable_missing, tag_enable_shows_missing, custom_tags, exempt_tags=exempt_tags,
+            instance_display_name=display_name
         )
     else:
         sonarr_logger.error(f"Invalid hunt_missing_mode: {hunt_missing_mode}. Valid options are 'seasons_packs', 'shows', or 'episodes'.")
@@ -117,7 +169,14 @@ def process_missing_seasons_packs_mode(
     air_date_delay_days: int,
     command_wait_delay: int,
     command_wait_attempts: int,
-    stop_check: Callable[[], bool]
+    stop_check: Callable[[], bool],
+    tag_processed_items: bool = False,
+    tag_enable_missing: bool = False,
+    tag_enable_shows_missing: bool = False,
+    custom_tags: dict = None,
+    exempt_tags: list = None,
+    hunt_missing_mode: str = "seasons_packs",
+    instance_display_name: Optional[str] = None,
 ) -> bool:
     """
     Process missing seasons using the SeasonSearch command
@@ -125,10 +184,25 @@ def process_missing_seasons_packs_mode(
     Uses a direct episode lookup approach which is much more efficient
     """
     processed_any = False
-    
-    # Load settings to check if tagging is enabled
-    sonarr_settings = load_settings("sonarr")
-    tag_processed_items = sonarr_settings.get("tag_processed_items", True)
+    exempt_tags = _normalize_exempt_tags(exempt_tags or [])
+
+    # Use custom tags if provided, otherwise use defaults
+    if custom_tags is None:
+        custom_tags = {
+            "missing": "huntarr-missing",
+            "upgrade": "huntarr-upgrade",
+            "shows_missing": "huntarr-shows-missing"
+        }
+
+    # Build unified tag settings
+    _tag_settings = {
+        "enabled": tag_processed_items,
+        "tag_enable_missing": tag_enable_missing,
+        "tag_enable_upgrade": False,
+        "tag_enable_upgraded": False,
+        "tag_enable_shows_missing": tag_enable_shows_missing,
+        "custom_tags": custom_tags or {},
+    }
     
     # Get all missing episodes using efficient random page selection instead of fetching all
     missing_episodes = sonarr_api.get_missing_episodes_random_page(
@@ -216,6 +290,19 @@ def process_missing_seasons_packs_mode(
     # Convert to list and sort by episode count (most missing episodes first)
     seasons_list = list(missing_seasons.values())
     seasons_list.sort(key=lambda x: x['episode_count'], reverse=True)
+
+    # Filter out series with exempt tags
+    if exempt_tags:
+        exempt_series_ids = _get_exempt_series_ids(api_url, api_key, api_timeout, exempt_tags)
+        if exempt_series_ids:
+            series_id_to_title = {s['series_id']: s['series_title'] for s in seasons_list}
+            for sid in exempt_series_ids:
+                if sid in series_id_to_title:
+                    sonarr_logger.info(
+                        f"Skipping series \"{series_id_to_title[sid]}\" (ID: {sid}) - has exempt tag"
+                    )
+            seasons_list = [s for s in seasons_list if s['series_id'] not in exempt_series_ids]
+            sonarr_logger.info(f"Exempt tags filter: {len(seasons_list)} seasons remaining after excluding series with exempt tags.")
     
     # Filter out already processed seasons
     unprocessed_seasons = []
@@ -272,8 +359,8 @@ def process_missing_seasons_packs_mode(
         
         sonarr_logger.info(f"Searching for season pack: {series_title} - Season {season_number} (contains {episode_count} missing episodes)")
         
-        # Trigger an API call to search for the entire season
-        command_id = sonarr_api.search_season(api_url, api_key, api_timeout, series_id, season_number)
+        # Trigger an API call to search for the entire season (pass instance_name for per-instance API cap)
+        command_id = sonarr_api.search_season(api_url, api_key, api_timeout, series_id, season_number, instance_name=instance_name)
         
         if command_id:
             processed_any = True
@@ -284,32 +371,27 @@ def process_missing_seasons_packs_mode(
             success = add_processed_id("sonarr", instance_name, season_id)
             sonarr_logger.debug(f"Added season ID {season_id} to processed list for {instance_name}, success: {success}")
             
-                    # Tag the series if enabled
-        if tag_processed_items:
-            from src.primary.settings_manager import get_custom_tag
-            custom_tag = get_custom_tag("sonarr", "missing", "huntarr-missing")
-            try:
-                sonarr_api.tag_processed_series(api_url, api_key, api_timeout, series_id, custom_tag)
-                sonarr_logger.debug(f"Tagged series {series_id} with '{custom_tag}'")
-            except Exception as e:
-                sonarr_logger.warning(f"Failed to tag series {series_id} with '{custom_tag}': {e}")
+            try_tag_item(_tag_settings, "missing", sonarr_api.tag_processed_series,
+                         api_url, api_key, api_timeout, series_id,
+                         sonarr_logger, f"series {series_id}")
             
             # Log to history system
             media_name = f"{series_title} - Season {season_number} (contains {episode_count} missing episodes)"
-            log_processed_media("sonarr", media_name, season_id, instance_name, "missing")
+            log_processed_media("sonarr", media_name, season_id, instance_name, "missing", display_name_for_log=instance_display_name or instance_name)
             sonarr_logger.debug(f"Logged history entry for season pack: {media_name}")
             
             # CRITICAL FIX: Use increment_stat_only to avoid double-counting API calls
             # The API call is already tracked in search_season(), so we only increment stats here
             for i in range(episode_count):
-                increment_stat_only("sonarr", "hunted")
+                increment_stat_only("sonarr", "hunted", 1, instance_name)
             sonarr_logger.debug(f"Incremented sonarr hunted statistics for {episode_count} episodes in season pack (API call already tracked separately)")
             
             # Wait for command to complete if configured
             if command_wait_delay > 0 and command_wait_attempts > 0:
                 if wait_for_command(
                     api_url, api_key, api_timeout, command_id, 
-                    command_wait_delay, command_wait_attempts, "Season Search", stop_check
+                    command_wait_delay, command_wait_attempts, "Season Search", stop_check,
+                    instance_name=instance_name
                 ):
                     pass
         else:
@@ -329,14 +411,35 @@ def process_missing_shows_mode(
     air_date_delay_days: int,
     command_wait_delay: int,
     command_wait_attempts: int,
-    stop_check: Callable[[], bool]
+    stop_check: Callable[[], bool],
+    tag_processed_items: bool = False,
+    tag_enable_missing: bool = False,
+    tag_enable_shows_missing: bool = False,
+    custom_tags: dict = None,
+    exempt_tags: list = None,
+    instance_display_name: Optional[str] = None,
 ) -> bool:
     """Process missing episodes in show mode - gets all missing episodes for entire shows."""
     processed_any = False
-    
-    # Load settings to check if tagging is enabled
-    sonarr_settings = load_settings("sonarr")
-    tag_processed_items = sonarr_settings.get("tag_processed_items", True)
+    exempt_tags = _normalize_exempt_tags(exempt_tags or [])
+
+    # Use custom tags if provided, otherwise use defaults
+    if custom_tags is None:
+        custom_tags = {
+            "missing": "huntarr-missing",
+            "upgrade": "huntarr-upgrade",
+            "shows_missing": "huntarr-shows-missing"
+        }
+
+    # Build unified tag settings
+    _tag_settings = {
+        "enabled": tag_processed_items,
+        "tag_enable_missing": tag_enable_missing,
+        "tag_enable_upgrade": False,
+        "tag_enable_upgraded": False,
+        "tag_enable_shows_missing": tag_enable_shows_missing,
+        "custom_tags": custom_tags or {},
+    }
     
     # Get series with missing episodes
     sonarr_logger.info("Retrieving series with missing episodes...")
@@ -346,6 +449,18 @@ def process_missing_shows_mode(
     if not series_with_missing:
         sonarr_logger.info("No series with missing episodes found.")
         return False
+
+    # Filter out series with exempt tags
+    if exempt_tags:
+        exempt_series_ids = _get_exempt_series_ids(api_url, api_key, api_timeout, exempt_tags)
+        if exempt_series_ids:
+            for show in series_with_missing:
+                if show.get("series_id") in exempt_series_ids:
+                    sonarr_logger.info(
+                        f"Skipping series \"{show.get('series_title', 'Unknown')}\" (ID: {show.get('series_id')}) - has exempt tag"
+                    )
+            series_with_missing = [s for s in series_with_missing if s.get("series_id") not in exempt_series_ids]
+            sonarr_logger.info(f"Exempt tags filter: {len(series_with_missing)} series remaining after excluding series with exempt tags.")
     
     # Filter out shows that have been processed
     unprocessed_series = []
@@ -458,21 +573,15 @@ def process_missing_shows_mode(
         
         # Search for all episodes in the show
         sonarr_logger.info(f"Searching for {len(episode_ids)} missing episodes for {show_title}...")
-        search_successful = sonarr_api.search_episode(api_url, api_key, api_timeout, episode_ids)
+        search_successful = sonarr_api.search_episode(api_url, api_key, api_timeout, episode_ids, instance_name=instance_name)
         
         if search_successful:
             processed_any = True
             sonarr_logger.info(f"Successfully processed {len(episode_ids)} missing episodes in {show_title}")
             
-                    # Tag the series if enabled
-        if tag_processed_items:
-            from src.primary.settings_manager import get_custom_tag
-            custom_tag = get_custom_tag("sonarr", "shows_missing", "huntarr-shows-missing")
-            try:
-                sonarr_api.tag_processed_series(api_url, api_key, api_timeout, show_id, custom_tag)
-                sonarr_logger.debug(f"Tagged series {show_id} with '{custom_tag}'")
-            except Exception as e:
-                sonarr_logger.warning(f"Failed to tag series {show_id} with '{custom_tag}': {e}")
+            try_tag_item(_tag_settings, "shows_missing", sonarr_api.tag_processed_series,
+                         api_url, api_key, api_timeout, show_id,
+                         sonarr_logger, f"series {show_id}")
             
             # Add episode IDs to stateful manager IMMEDIATELY after processing each batch
             for episode_id in episode_ids:
@@ -494,7 +603,7 @@ def process_missing_shows_mode(
                             season_episode = f"S{season}E{ep_num}"
                             
                         media_name = f"{show_title} - {season_episode} - {title}"
-                        log_processed_media("sonarr", media_name, str(episode_id), instance_name, "missing")
+                        log_processed_media("sonarr", media_name, str(episode_id), instance_name, "missing", display_name_for_log=instance_display_name or instance_name)
                         sonarr_logger.debug(f"Logged history entry for episode: {media_name}")
                         break
             
@@ -504,11 +613,11 @@ def process_missing_shows_mode(
             
             # Also log the entire show to history
             media_name = f"{show_title} - Complete Series ({len(episode_ids)} episodes)"
-            log_processed_media("sonarr", media_name, str(show_id), instance_name, "missing")
+            log_processed_media("sonarr", media_name, str(show_id), instance_name, "missing", display_name_for_log=instance_display_name or instance_name)
             sonarr_logger.debug(f"Logged history entry for complete series: {media_name}")
             
             # Increment the hunted statistics
-            increment_stat("sonarr", "hunted", len(episode_ids))
+            increment_stat("sonarr", "hunted", len(episode_ids), instance_name)
             sonarr_logger.debug(f"Incremented sonarr hunted statistics by {len(episode_ids)}")
         else:
             sonarr_logger.error(f"Failed to trigger search for {show_title}.")
@@ -527,7 +636,13 @@ def process_missing_episodes_mode(
     air_date_delay_days: int,
     command_wait_delay: int,
     command_wait_attempts: int,
-    stop_check: Callable[[], bool]
+    stop_check: Callable[[], bool],
+    tag_processed_items: bool = False,
+    tag_enable_missing: bool = False,
+    tag_enable_shows_missing: bool = False,
+    custom_tags: dict = None,
+    exempt_tags: list = None,
+    instance_display_name: Optional[str] = None,
 ) -> bool:
     """
     Process missing episodes in individual episode mode.
@@ -539,6 +654,15 @@ def process_missing_episodes_mode(
     which can be useful for targeting specific episodes but is not recommended for most users.
     """
     processed_any = False
+    exempt_tags = _normalize_exempt_tags(exempt_tags or [])
+
+    # Use custom tags if provided, otherwise use defaults
+    if custom_tags is None:
+        custom_tags = {
+            "missing": "huntarr-missing",
+            "upgrade": "huntarr-upgrade",
+            "shows_missing": "huntarr-shows-missing"
+        }
     
     sonarr_logger.warning("Using Episodes mode - This will make more API calls and does not support tagging")
     
@@ -550,6 +674,18 @@ def process_missing_episodes_mode(
     if not missing_episodes:
         sonarr_logger.info("No missing episodes found for individual processing.")
         return False
+
+    # Filter out episodes from series with exempt tags
+    if exempt_tags:
+        exempt_series_ids = _get_exempt_series_ids(api_url, api_key, api_timeout, exempt_tags)
+        if exempt_series_ids:
+            original_count = len(missing_episodes)
+            missing_episodes = [e for e in missing_episodes if e.get("seriesId") not in exempt_series_ids]
+            if original_count != len(missing_episodes):
+                sonarr_logger.info(f"Exempt tags filter: {len(missing_episodes)} episodes remaining after excluding series with exempt tags.")
+    if not missing_episodes:
+        sonarr_logger.info("No missing episodes left after exempt tags filter.")
+        return False
     
     # Filter out future episodes if configured
     if skip_future_episodes:
@@ -653,7 +789,7 @@ def process_missing_episodes_mode(
         sonarr_logger.info(f"Processing episode: {series_title} - {season_episode} - {episode_title}")
         
         # Search for this specific episode
-        search_successful = sonarr_api.search_episode(api_url, api_key, api_timeout, [episode_id])
+        search_successful = sonarr_api.search_episode(api_url, api_key, api_timeout, [episode_id], instance_name=instance_name)
         
         if search_successful:
             processed_any = True
@@ -665,11 +801,11 @@ def process_missing_episodes_mode(
             
             # Log to history system
             media_name = f"{series_title} - {season_episode} - {episode_title}"
-            log_processed_media("sonarr", media_name, str(episode_id), instance_name, "missing")
+            log_processed_media("sonarr", media_name, str(episode_id), instance_name, "missing", display_name_for_log=instance_display_name or instance_name)
             sonarr_logger.debug(f"Logged history entry for episode: {media_name}")
             
             # Increment statistics
-            increment_stat("sonarr", "hunted")
+            increment_stat("sonarr", "hunted", 1, instance_name)
             sonarr_logger.debug(f"Incremented sonarr hunted statistics for episode {episode_id}")
             
             # Note: No tagging is performed in episodes mode as it would be inefficient
@@ -690,7 +826,8 @@ def wait_for_command(
     wait_delay: int,
     max_attempts: int,
     command_name: str = "Command",
-    stop_check: Callable[[], bool] = lambda: False
+    stop_check: Callable[[], bool] = lambda: False,
+    instance_name: Optional[str] = None
 ) -> bool:
     """
     Wait for a Sonarr command to complete or timeout.
@@ -704,42 +841,58 @@ def wait_for_command(
         max_attempts: Maximum number of status check attempts
         command_name: Name of the command (for logging)
         stop_check: Optional function to check if operation should be aborted
+        instance_name: Optional instance name for UI cycle activity (e.g. "Season Search (360/600)")
         
     Returns:
         True if command completed successfully, False otherwise
     """
-    if wait_delay <= 0 or max_attempts <= 0:
-        sonarr_logger.debug(f"Not waiting for command to complete (wait_delay={wait_delay}, max_attempts={max_attempts})")
-        return True  # Return as if successful since we're not checking
-    
-    sonarr_logger.debug(f"Waiting for {command_name} to complete (command ID: {command_id}). Checking every {wait_delay}s for up to {max_attempts} attempts")
-    
-    # Wait for command completion
-    attempts = 0
-    while attempts < max_attempts:
-        if stop_check():
-            sonarr_logger.info(f"Stopping wait for {command_name} due to stop request")
-            return False
-            
-        command_status = sonarr_api.get_command_status(api_url, api_key, api_timeout, command_id)
-        if not command_status:
-            sonarr_logger.warning(f"Failed to get status for {command_name} (ID: {command_id}), attempt {attempts+1}")
+    try:
+        if wait_delay <= 0 or max_attempts <= 0:
+            sonarr_logger.debug(f"Not waiting for command to complete (wait_delay={wait_delay}, max_attempts={max_attempts})")
+            return True  # Return as if successful since we're not checking
+        
+        sonarr_logger.debug(f"Waiting for {command_name} to complete (command ID: {command_id}). Adaptive backoff, max {max_attempts} attempts")
+        
+        # Wait for command completion with adaptive backoff: poll less often over time to reduce API load when Sonarr is slow/queued
+        attempts = 0
+        while attempts < max_attempts:
+            if stop_check():
+                sonarr_logger.info(f"Stopping wait for {command_name} due to stop request")
+                return False
+            # Update UI every 5 attempts to avoid flicker
+            if instance_name and attempts % 5 == 0:
+                try:
+                    from src.primary.cycle_tracker import set_cycle_activity
+                    set_cycle_activity("sonarr", instance_name, f"Search {attempts+1}/{max_attempts}")
+                except Exception:
+                    pass
+            command_status = sonarr_api.get_command_status(api_url, api_key, api_timeout, command_id)
+            if not command_status:
+                sonarr_logger.warning(f"Failed to get status for {command_name} (ID: {command_id}), attempt {attempts+1}")
+                attempts += 1
+                time.sleep(wait_delay)
+                continue
+            status = command_status.get('status')
+            if status == 'completed':
+                sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) completed successfully")
+                return True
+            elif status in ['failed', 'aborted']:
+                sonarr_logger.warning(f"Sonarr {command_name} (ID: {command_id}) {status}")
+                return False
+            # Log status only every 15th attempt to reduce log spam when command stays queued
+            if attempts % 15 == 0:
+                sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) status: {status}, attempt {attempts+1}/{max_attempts}")
+            # Adaptive backoff: 1s for first 10 checks, then 2s, 3s, ... up to 15s max (reduces API calls when Sonarr is slow)
+            effective_delay = min(wait_delay + (attempts // 10), 15)
             attempts += 1
-            time.sleep(wait_delay)
-            continue
-            
-        status = command_status.get('status')
-        if status == 'completed':
-            sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) completed successfully")
-            return True
-        elif status in ['failed', 'aborted']:
-            sonarr_logger.warning(f"Sonarr {command_name} (ID: {command_id}) {status}")
-            return False
+            time.sleep(effective_delay)
         
-        sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) status: {status}, attempt {attempts+1}/{max_attempts}")
-        
-        attempts += 1
-        time.sleep(wait_delay)
-    
-    sonarr_logger.error(f"Sonarr command '{command_name}' (ID: {command_id}) timed out after {max_attempts} attempts.")
-    return False
\ No newline at end of file
+        sonarr_logger.error(f"Sonarr command '{command_name}' (ID: {command_id}) timed out after {max_attempts} attempts.")
+        return False
+    finally:
+        if instance_name:
+            try:
+                from src.primary.cycle_tracker import clear_cycle_activity
+                clear_cycle_activity("sonarr", instance_name)
+            except Exception:
+                pass
\ No newline at end of file
