@@ -5,9 +5,11 @@ Sonarr cutoff upgrade processing module for Huntarr
 
 import time
 import random
-from typing import List, Dict, Any, Set, Callable, Union
+from typing import List, Dict, Any, Set, Callable, Union, Optional
 from src.primary.utils.logger import get_logger
+from src.primary.apps._common.tagging import try_tag_item
 from src.primary.apps.sonarr import api as sonarr_api
+from src.primary.apps.sonarr.missing import _normalize_exempt_tags
 from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
 from src.primary.stateful_manager import is_processed, add_processed_id
 from src.primary.utils.history_utils import log_processed_media
@@ -20,45 +22,153 @@ def process_cutoff_upgrades(
     api_url: str,
     api_key: str,
     instance_name: str,
-    api_timeout: int = get_advanced_setting("api_timeout", 120),
+    api_timeout: int = 120,
     monitored_only: bool = True,
     # series_type: str = "standard",  # TODO: Add series type filtering (standard, daily, anime)
     hunt_upgrade_items: int = 5,
     upgrade_mode: str = "seasons_packs",
     command_wait_delay: int = get_advanced_setting("command_wait_delay", 1),
     command_wait_attempts: int = get_advanced_setting("command_wait_attempts", 600),
-    stop_check: Callable[[], bool] = lambda: False
+    stop_check: Callable[[], bool] = lambda: False,
+    tag_processed_items: bool = False,
+    tag_enable_upgrade: bool = False,
+    tag_enable_upgraded: bool = False,
+    custom_tags: dict = None,
+    upgrade_selection_method: str = "cutoff",
+    upgrade_tag: str = "",
+    exempt_tags: list = None,
+    instance_display_name: Optional[str] = None,
 ) -> bool:
     """
     Process quality cutoff upgrades for Sonarr.
     Supports seasons_packs and episodes modes. Episodes mode has been reinstated in 7.5.1+ as a non-default option with limitations.
+    upgrade_selection_method: "cutoff" (default) or "tags" for tag-based selection.
     """
     if hunt_upgrade_items <= 0:
         sonarr_logger.info("'hunt_upgrade_items' setting is 0 or less. Skipping upgrade processing.")
         return False
+
+    method = (upgrade_selection_method or "cutoff").strip().lower()
+    if method not in ("cutoff", "tags"):
+        method = "cutoff"
+    tag_label = (upgrade_tag or "").strip()
         
     sonarr_logger.info(f"Checking for {hunt_upgrade_items} quality upgrades for instance '{instance_name}'...")
     
+    # Use custom tags if provided, otherwise use defaults
+    if custom_tags is None:
+        custom_tags = {
+            "missing": "huntarr-missing",
+            "upgrade": "huntarr-upgrade",
+            "shows_missing": "huntarr-shows-missing"
+        }
+
+    # Build unified tag settings
+    _tag_settings = {
+        "enabled": tag_processed_items,
+        "tag_enable_missing": False,
+        "tag_enable_upgrade": tag_enable_upgrade,
+        "tag_enable_upgraded": tag_enable_upgraded,
+        "tag_enable_shows_missing": False,
+        "custom_tags": custom_tags or {},
+    }
+    
+    # Tag-based upgrade path (Upgradinatorr-style): series WITHOUT tag -> SeriesSearch -> add tag
+    if method == "tags":
+        if not tag_label:
+            sonarr_logger.warning("Upgrade selection method is 'Tags' but no upgrade tag is configured. Skipping.")
+            return False
+        sonarr_logger.info(f"Retrieving series WITHOUT tag \"{tag_label}\" (Upgradinatorr-style: tag tracks processed)...")
+        series_list = sonarr_api.get_series_without_tag(api_url, api_key, api_timeout, tag_label, monitored_only)
+        if series_list is None:
+            return False
+        if not series_list:
+            sonarr_logger.info(f"No series found without the tag \"{tag_label}\" (all have been processed).")
+            return False
+        sonarr_logger.info(f"Found {len(series_list)} series without tag \"{tag_label}\".")
+        # Filter out series with exempt tags (issue #676)
+        if exempt_tags:
+            exempt_id_to_label = sonarr_api.get_exempt_tag_ids(api_url, api_key, api_timeout, exempt_tags)
+            if exempt_id_to_label:
+                filtered = []
+                for s in series_list:
+                    skip = False
+                    for tid in s.get("tags", []):
+                        if tid in exempt_id_to_label:
+                            sonarr_logger.info(
+                                f"Skipping series \"{s.get('title', 'Unknown')}\" (ID: {s.get('id')}) - has exempt tag \"{exempt_id_to_label[tid]}\""
+                            )
+                            skip = True
+                            break
+                    if not skip:
+                        filtered.append(s)
+                series_list = filtered
+                sonarr_logger.info(f"Exempt tags filter: {len(series_list)} series remaining for upgrades after excluding series with exempt tags.")
+        unprocessed = [
+            s for s in series_list
+            if not is_processed("sonarr", instance_name, f"series_{s.get('id')}")
+        ]
+        if not unprocessed:
+            sonarr_logger.info("All series with the upgrade tag have been processed recently. Skipping.")
+            return False
+        to_process = random.sample(unprocessed, min(hunt_upgrade_items, len(unprocessed)))
+        processed_count = 0
+        for s in to_process:
+            if stop_check():
+                break
+            series_id = s.get("id")
+            title = s.get("title", f"Series {series_id}")
+            sonarr_logger.info(f"Processing tag-based upgrade for series: \"{title}\" (ID: {series_id})")
+            if sonarr_api.series_search(api_url, api_key, api_timeout, series_id):
+                add_processed_id("sonarr", instance_name, f"series_{series_id}")
+                from src.primary.stats_manager import increment_stat_only
+                increment_stat_only("sonarr", "upgraded", 1, instance_name)
+                log_processed_media("sonarr", title, str(series_id), instance_name, "upgrade", display_name_for_log=instance_display_name or instance_name)
+                
+                # Add the upgrade tag to mark as processed (Upgradinatorr-style)
+                tag_id = sonarr_api.get_or_create_tag(api_url, api_key, api_timeout, tag_label)
+                if tag_id:
+                    try:
+                        ok = sonarr_api.add_tag_to_series(api_url, api_key, api_timeout, series_id, tag_id)
+                        if ok:
+                            sonarr_logger.debug(f"Added upgrade tag '{tag_label}' to series {series_id} ('{title}') to mark as processed")
+                        else:
+                            sonarr_logger.warning(f"Failed to add upgrade tag '{tag_label}' to series {series_id} ('{title}') - API returned failure")
+                    except Exception as e:
+                        sonarr_logger.warning(f"Failed to add upgrade tag '{tag_label}' to series {series_id} ('{title}'): {e}")
+                
+                try_tag_item(_tag_settings, "upgraded", sonarr_api.tag_processed_series,
+                             api_url, api_key, api_timeout, series_id,
+                             sonarr_logger, f"series {series_id}")
+                processed_count += 1
+        sonarr_logger.info(f"Finished tag-based upgrade cycle: processed {processed_count} series.")
+        return processed_count > 0
+    
     sonarr_logger.info(f"Using {upgrade_mode.upper()} mode for quality upgrades")
 
-    # Use seasons_packs mode or episodes mode
+    display_name = instance_display_name or instance_name
+    # Use seasons_packs mode or episodes mode (exempt_tags already normalized above)
     if upgrade_mode == "seasons_packs":
         return process_upgrade_seasons_mode(
             api_url, api_key, instance_name, api_timeout, monitored_only, 
-            hunt_upgrade_items, command_wait_delay, command_wait_attempts, stop_check
+            hunt_upgrade_items, command_wait_delay, command_wait_attempts, stop_check,
+            tag_processed_items, tag_enable_upgrade, tag_enable_upgraded, custom_tags, exempt_tags=exempt_tags,
+            instance_display_name=display_name
         )
     elif upgrade_mode == "episodes":
         # Handle individual episode upgrades (reinstated with warnings)
         sonarr_logger.warning("Episodes mode selected for upgrades - WARNING: This mode makes excessive API calls and does not support tagging. Consider using Season Packs mode instead.")
         return process_upgrade_episodes_mode(
             api_url, api_key, instance_name, api_timeout, monitored_only, 
-            hunt_upgrade_items, command_wait_delay, command_wait_attempts, stop_check
+            hunt_upgrade_items, command_wait_delay, command_wait_attempts, stop_check,
+            tag_processed_items, tag_enable_upgrade, tag_enable_upgraded, custom_tags, exempt_tags=exempt_tags,
+            instance_display_name=display_name
         )
     else:
         sonarr_logger.error(f"Invalid upgrade_mode: {upgrade_mode}. Valid options are 'seasons_packs' or 'episodes'.")
         return False
 
-def log_season_pack_upgrade(api_url: str, api_key: str, api_timeout: int, series_id: int, season_number: int, instance_name: str):
+def log_season_pack_upgrade(api_url: str, api_key: str, api_timeout: int, series_id: int, season_number: int, instance_name: str, instance_display_name: Optional[str] = None):
     """Log a season pack upgrade to the history."""
     try:
         # Get series details for better history logging
@@ -80,7 +190,7 @@ def log_season_pack_upgrade(api_url: str, api_key: str, api_timeout: int, series
             media_name = f"{series_title} - {season_id}"
             
             # Log the season pack upgrade to history with normal 'upgrade' operation type
-            log_processed_media("sonarr", media_name, season_id_num, instance_name, "upgrade")
+            log_processed_media("sonarr", media_name, season_id_num, instance_name, "upgrade", display_name_for_log=instance_display_name or instance_name)
             sonarr_logger.debug(f"Logged season pack upgrade to history for {series_title} Season {season_number}")
     except Exception as e:
         sonarr_logger.error(f"Failed to log season pack upgrade to history: {str(e)}")
@@ -94,14 +204,35 @@ def process_upgrade_seasons_mode(
     hunt_upgrade_items: int,
     command_wait_delay: int,
     command_wait_attempts: int,
-    stop_check: Callable[[], bool]
+    stop_check: Callable[[], bool],
+    tag_processed_items: bool = False,
+    tag_enable_upgrade: bool = False,
+    tag_enable_upgraded: bool = False,
+    custom_tags: dict = None,
+    exempt_tags: list = None,
+    instance_display_name: Optional[str] = None,
 ) -> bool:
     """Process upgrades in season mode - groups episodes by season."""
     processed_any = False
-    
-    # Load settings to check if tagging is enabled
-    sonarr_settings = load_settings("sonarr")
-    tag_processed_items = sonarr_settings.get("tag_processed_items", True)
+    exempt_tags = exempt_tags or []
+
+    # Use custom tags if provided, otherwise use defaults
+    if custom_tags is None:
+        custom_tags = {
+            "missing": "huntarr-missing",
+            "upgrade": "huntarr-upgrade",
+            "shows_missing": "huntarr-shows-missing"
+        }
+
+    # Build unified tag settings
+    _tag_settings = {
+        "enabled": tag_processed_items,
+        "tag_enable_missing": False,
+        "tag_enable_upgrade": tag_enable_upgrade,
+        "tag_enable_upgraded": tag_enable_upgraded,
+        "tag_enable_shows_missing": False,
+        "custom_tags": custom_tags or {},
+    }
     
     # Flag to skip individual episode history logging since we log the whole season pack
     skip_episode_history = True
@@ -112,6 +243,26 @@ def process_upgrade_seasons_mode(
     sample_size = hunt_upgrade_items * 10
     cutoff_unmet_episodes = sonarr_api.get_cutoff_unmet_episodes_random_page(
         api_url, api_key, api_timeout, monitored_only, sample_size)
+
+    # Filter out episodes whose series has an exempt tag (issue #676)
+    if exempt_tags and cutoff_unmet_episodes:
+        exempt_id_to_label = sonarr_api.get_exempt_tag_ids(api_url, api_key, api_timeout, exempt_tags)
+        if exempt_id_to_label:
+            all_series = sonarr_api.get_series(api_url, api_key, api_timeout)
+            exempt_series_ids = set()
+            if all_series:
+                if not isinstance(all_series, list):
+                    all_series = [all_series]
+                for s in all_series:
+                    for tid in s.get("tags", []):
+                        if tid in exempt_id_to_label:
+                            exempt_series_ids.add(s.get("id"))
+                            sonarr_logger.info(
+                                f"Skipping series \"{s.get('title', 'Unknown')}\" (ID: {s.get('id')}) - has exempt tag \"{exempt_id_to_label[tid]}\""
+                            )
+                            break
+            cutoff_unmet_episodes = [ep for ep in cutoff_unmet_episodes if ep.get("seriesId") not in exempt_series_ids]
+            sonarr_logger.info(f"Exempt tags filter: {len(cutoff_unmet_episodes)} episodes remaining for upgrades after excluding series with exempt tags.")
     
     sonarr_logger.info(f"Received {len(cutoff_unmet_episodes)} cutoff unmet episodes from random page (before filtering).")
     
@@ -216,30 +367,25 @@ def process_upgrade_seasons_mode(
             
         # Trigger search for the entire season instead of individual episodes
         sonarr_logger.debug(f"Attempting to search for entire Season {season_number} of {series_title} for upgrades")
-        search_command_id = sonarr_api.search_season(api_url, api_key, api_timeout, series_id, season_number)
+        search_command_id = sonarr_api.search_season(api_url, api_key, api_timeout, series_id, season_number, instance_name=instance_name)
         
         if search_command_id:
             # Wait for search command to complete
             if wait_for_command(
                 api_url, api_key, api_timeout, search_command_id,
-                command_wait_delay, command_wait_attempts, "Episode Upgrade Search", stop_check
+                command_wait_delay, command_wait_attempts, "Episode Upgrade Search", stop_check,
+                instance_name=instance_name
             ):
                 # Mark as processed if search command completed successfully
                 processed_any = True
                 sonarr_logger.info(f"Successfully triggered season pack search for {series_title} Season {season_number} with {len(episode_ids)} cutoff unmet episodes")
                 
-                # Tag the series if enabled
-                if tag_processed_items:
-                    from src.primary.settings_manager import get_custom_tag
-                    custom_tag = get_custom_tag("sonarr", "upgrade", "huntarr-upgraded")
-                    try:
-                        sonarr_api.tag_processed_series(api_url, api_key, api_timeout, series_id, custom_tag)
-                        sonarr_logger.debug(f"Tagged series {series_id} with '{custom_tag}'")
-                    except Exception as e:
-                        sonarr_logger.warning(f"Failed to tag series {series_id} with '{custom_tag}': {e}")
+                try_tag_item(_tag_settings, "upgrade", sonarr_api.tag_processed_series,
+                             api_url, api_key, api_timeout, series_id,
+                             sonarr_logger, f"series {series_id}")
                 
                 # Log this as a season pack upgrade in the history
-                log_season_pack_upgrade(api_url, api_key, api_timeout, series_id, season_number, instance_name)
+                log_season_pack_upgrade(api_url, api_key, api_timeout, series_id, season_number, instance_name, instance_display_name)
                 
                 # CRITICAL FIX: Mark the season as processed at the season level to prevent reprocessing
                 season_id = f"{series_id}_{season_number}"
@@ -258,7 +404,7 @@ def process_upgrade_seasons_mode(
                     # CRITICAL FIX: Use increment_stat_only to avoid double-counting API calls
                     # The API call is already tracked in search_season(), so we only increment stats here
                     from src.primary.stats_manager import increment_stat_only
-                    increment_stat_only("sonarr", "upgraded")
+                    increment_stat_only("sonarr", "upgraded", 1, instance_name)
                     sonarr_logger.debug(f"Incremented sonarr upgraded statistic for episode {episode_id} (API call already tracked separately)")
                     
                     # Find the episode information for history logging
@@ -280,7 +426,7 @@ def process_upgrade_seasons_mode(
                             media_name = f"{series_title} - {season_episode} - {episode_title}"
                             # Skip logging individual episodes since we log the season pack
                             if not skip_episode_history:
-                                log_processed_media("sonarr", media_name, episode_id, instance_name, "upgrade")
+                                log_processed_media("sonarr", media_name, episode_id, instance_name, "upgrade", display_name_for_log=instance_display_name or instance_name)
                             sonarr_logger.debug(f"Logged quality upgrade to history for episode ID {episode_id}")
                     except Exception as e:
                         sonarr_logger.error(f"Failed to log history for episode ID {episode_id}: {str(e)}")
@@ -301,14 +447,26 @@ def process_upgrade_shows_mode(
     hunt_upgrade_items: int,
     command_wait_delay: int,
     command_wait_attempts: int,
-    stop_check: Callable[[], bool]
+    stop_check: Callable[[], bool],
+    tag_processed_items: bool = False,
+    tag_enable_upgrade: bool = False,
+    tag_enable_upgraded: bool = False,
+    custom_tags: dict = None,
 ) -> bool:
     """Process upgrades in show mode - gets all cutoff unmet episodes for entire shows."""
     processed_any = False
-    
-    # Load settings to check if tagging is enabled
-    sonarr_settings = load_settings("sonarr")
-    tag_processed_items = sonarr_settings.get("tag_processed_items", True)
+    if custom_tags is None:
+        custom_tags = {"missing": "huntarr-missing", "upgrade": "huntarr-upgrade", "shows_missing": "huntarr-shows-missing"}
+
+    # Build unified tag settings
+    _tag_settings = {
+        "enabled": tag_processed_items,
+        "tag_enable_missing": False,
+        "tag_enable_upgrade": tag_enable_upgrade,
+        "tag_enable_upgraded": tag_enable_upgraded,
+        "tag_enable_shows_missing": False,
+        "custom_tags": custom_tags or {},
+    }
     
     # For shows mode, we want individual episode history entries
     skip_episode_history = False
@@ -409,27 +567,22 @@ def process_upgrade_shows_mode(
             
         # Trigger search for all cutoff unmet episodes in this series
         sonarr_logger.debug(f"Attempting to search for {len(episode_ids)} episodes in {series_title} for upgrades")
-        search_command_id = sonarr_api.search_episode(api_url, api_key, api_timeout, episode_ids)
+        search_command_id = sonarr_api.search_episode(api_url, api_key, api_timeout, episode_ids, instance_name=instance_name)
         
         if search_command_id:
             # Wait for search command to complete
             if wait_for_command(
                 api_url, api_key, api_timeout, search_command_id,
-                command_wait_delay, command_wait_attempts, "Episode Upgrade Search", stop_check
+                command_wait_delay, command_wait_attempts, "Episode Upgrade Search", stop_check,
+                instance_name=instance_name
             ):
                 # Mark as processed if search command completed successfully
                 processed_any = True
                 sonarr_logger.info(f"Successfully processed {len(episode_ids)} cutoff unmet episodes in {series_title}")
                 
-                # Tag the series if enabled
-                if tag_processed_items:
-                    from src.primary.settings_manager import get_custom_tag
-                    custom_tag = get_custom_tag("sonarr", "upgrade", "huntarr-upgraded")
-                    try:
-                        sonarr_api.tag_processed_series(api_url, api_key, api_timeout, series_id, custom_tag)
-                        sonarr_logger.debug(f"Tagged series {series_id} with '{custom_tag}'")
-                    except Exception as e:
-                        sonarr_logger.warning(f"Failed to tag series {series_id} with '{custom_tag}': {e}")
+                try_tag_item(_tag_settings, "upgrade", sonarr_api.tag_processed_series,
+                             api_url, api_key, api_timeout, series_id,
+                             sonarr_logger, f"series {series_id}")
                 
                 # We'll increment stats individually for each episode instead of in batch
                 # increment_stat("sonarr", "upgraded", len(episode_ids))
@@ -441,7 +594,7 @@ def process_upgrade_shows_mode(
                     sonarr_logger.debug(f"Marked episode ID {episode_id} as processed for upgrades")
                     
                     # Increment stats for this episode (consistent with Radarr's approach)
-                    increment_stat("sonarr", "upgraded")
+                    increment_stat("sonarr", "upgraded", 1, instance_name)
                     sonarr_logger.debug(f"Incremented sonarr upgraded statistic for episode {episode_id}")
                     
                     # Find the episode information for history logging
@@ -463,7 +616,7 @@ def process_upgrade_shows_mode(
                             media_name = f"{series_title} - {season_episode} - {episode_title}"
                             # Skip logging individual episodes since we log the season pack
                             if not skip_episode_history:
-                                log_processed_media("sonarr", media_name, episode_id, instance_name, "upgrade")
+                                log_processed_media("sonarr", media_name, episode_id, instance_name, "upgrade", display_name_for_log=instance_display_name or instance_name)
                             sonarr_logger.debug(f"Logged quality upgrade to history for episode ID {episode_id}")
                     except Exception as e:
                         sonarr_logger.error(f"Failed to log history for episode ID {episode_id}: {str(e)}")
@@ -484,7 +637,13 @@ def process_upgrade_episodes_mode(
     hunt_upgrade_items: int,
     command_wait_delay: int,
     command_wait_attempts: int,
-    stop_check: Callable[[], bool]
+    stop_check: Callable[[], bool],
+    tag_processed_items: bool = False,
+    tag_enable_upgrade: bool = False,
+    tag_enable_upgraded: bool = False,
+    custom_tags: dict = None,
+    exempt_tags: list = None,
+    instance_display_name: Optional[str] = None,
 ) -> bool:
     """
     Process upgrades in individual episode mode.
@@ -496,6 +655,15 @@ def process_upgrade_episodes_mode(
     which can be useful for targeting specific episodes but is not recommended for most users.
     """
     processed_any = False
+    exempt_tags = exempt_tags or []
+
+    # Use custom tags if provided, otherwise use defaults
+    if custom_tags is None:
+        custom_tags = {
+            "missing": "huntarr-missing",
+            "upgrade": "huntarr-upgrade",
+            "shows_missing": "huntarr-shows-missing"
+        }
     
     sonarr_logger.warning("Using Episodes mode for upgrades - This will make more API calls and does not support tagging")
     
@@ -503,6 +671,26 @@ def process_upgrade_episodes_mode(
     sonarr_logger.debug(f"Using random page selection for cutoff unmet episodes in episodes mode")
     cutoff_unmet_episodes = sonarr_api.get_cutoff_unmet_episodes_random_page(
         api_url, api_key, api_timeout, monitored_only, hunt_upgrade_items * 2)
+
+    # Filter out episodes whose series has an exempt tag (issue #676)
+    if exempt_tags and cutoff_unmet_episodes:
+        exempt_id_to_label = sonarr_api.get_exempt_tag_ids(api_url, api_key, api_timeout, exempt_tags)
+        if exempt_id_to_label:
+            all_series = sonarr_api.get_series(api_url, api_key, api_timeout)
+            exempt_series_ids = set()
+            if all_series:
+                if not isinstance(all_series, list):
+                    all_series = [all_series]
+                for s in all_series:
+                    for tid in s.get("tags", []):
+                        if tid in exempt_id_to_label:
+                            exempt_series_ids.add(s.get("id"))
+                            sonarr_logger.info(
+                                f"Skipping series \"{s.get('title', 'Unknown')}\" (ID: {s.get('id')}) - has exempt tag \"{exempt_id_to_label[tid]}\""
+                            )
+                            break
+            cutoff_unmet_episodes = [ep for ep in cutoff_unmet_episodes if ep.get("seriesId") not in exempt_series_ids]
+            sonarr_logger.info(f"Exempt tags filter: {len(cutoff_unmet_episodes)} episodes remaining for upgrades (episodes mode) after excluding series with exempt tags.")
     
     sonarr_logger.info(f"Received {len(cutoff_unmet_episodes)} cutoff unmet episodes from random page (before filtering).")
     
@@ -578,14 +766,15 @@ def process_upgrade_episodes_mode(
         sonarr_logger.info(f"Processing upgrade for episode: {series_title} - {season_episode} - {episode_title}")
         
         # Search for this specific episode upgrade
-        search_successful = sonarr_api.search_episode(api_url, api_key, api_timeout, [episode_id])
+        search_successful = sonarr_api.search_episode(api_url, api_key, api_timeout, [episode_id], instance_name=instance_name)
         
         if search_successful:
             # Wait for command to complete if configured
             if command_wait_delay > 0 and command_wait_attempts > 0:
                 if wait_for_command(
                     api_url, api_key, api_timeout, search_successful,
-                    command_wait_delay, command_wait_attempts, "Episode Upgrade Search", stop_check
+                    command_wait_delay, command_wait_attempts, "Episode Upgrade Search", stop_check,
+                    instance_name=instance_name
                 ):
                     processed_any = True
                     processed_count += 1
@@ -596,11 +785,11 @@ def process_upgrade_episodes_mode(
                     
                     # Log to history system
                     media_name = f"{series_title} - {season_episode} - {episode_title}"
-                    log_processed_media("sonarr", media_name, str(episode_id), instance_name, "upgrade")
+                    log_processed_media("sonarr", media_name, str(episode_id), instance_name, "upgrade", display_name_for_log=instance_display_name or instance_name)
                     sonarr_logger.debug(f"Logged upgrade to history for episode: {media_name}")
                     
                     # Increment statistics
-                    increment_stat("sonarr", "upgraded")
+                    increment_stat("sonarr", "upgraded", 1, instance_name)
                     sonarr_logger.debug(f"Incremented sonarr upgraded statistics for episode {episode_id}")
                     
                     # Note: No tagging is performed in episodes mode as it would be inefficient
@@ -618,11 +807,11 @@ def process_upgrade_episodes_mode(
                 
                 # Log to history system
                 media_name = f"{series_title} - {season_episode} - {episode_title}"
-                log_processed_media("sonarr", media_name, str(episode_id), instance_name, "upgrade")
+                log_processed_media("sonarr", media_name, str(episode_id), instance_name, "upgrade", display_name_for_log=instance_display_name or instance_name)
                 sonarr_logger.debug(f"Logged upgrade to history for episode: {media_name}")
                 
                 # Increment statistics
-                increment_stat("sonarr", "upgraded")
+                increment_stat("sonarr", "upgraded", 1, instance_name)
                 sonarr_logger.debug(f"Incremented sonarr upgraded statistics for episode {episode_id}")
         else:
             sonarr_logger.error(f"Failed to trigger upgrade search for episode: {series_title} - {season_episode}")
@@ -639,7 +828,8 @@ def wait_for_command(
     wait_delay: int,
     max_attempts: int,
     command_name: str = "Command",
-    stop_check: Callable[[], bool] = lambda: False
+    stop_check: Callable[[], bool] = lambda: False,
+    instance_name: Optional[str] = None
 ) -> bool:
     """
     Wait for a Sonarr command to complete or timeout.
@@ -653,44 +843,56 @@ def wait_for_command(
         max_attempts: Maximum number of status check attempts
         command_name: Name of the command (for logging)
         stop_check: Optional function to check if operation should be aborted
+        instance_name: Optional instance name for UI cycle activity (e.g. "Episode Upgrade Search (360/600)")
         
     Returns:
         True if command completed successfully, False otherwise
     """
-    if wait_delay <= 0 or max_attempts <= 0:
-        sonarr_logger.debug(f"Not waiting for command to complete (wait_delay={wait_delay}, max_attempts={max_attempts})")
-        return True  # Return as if successful since we're not checking
-    
-    sonarr_logger.debug(f"Waiting for {command_name} to complete (command ID: {command_id}). Checking every {wait_delay}s for up to {max_attempts} attempts")
-    
-    # Wait for command completion
-    attempts = 0
-    while attempts < max_attempts:
-        if stop_check():
-            sonarr_logger.info(f"Stopping wait for {command_name} due to stop request")
-            return False
-            
-        command_status = sonarr_api.get_command_status(api_url, api_key, api_timeout, command_id)
+    try:
+        if wait_delay <= 0 or max_attempts <= 0:
+            sonarr_logger.debug(f"Not waiting for command to complete (wait_delay={wait_delay}, max_attempts={max_attempts})")
+            return True  # Return as if successful since we're not checking
         
-        if command_status is None: 
-            sonarr_logger.warning(
-                f"Failed to get status for {command_name} (ID: {command_id}), attempt {attempts+1}. "
-                f"Command may have already completed or the ID is no longer valid."
-            )
-            return False # Stop polling if command ID is not found or error occurs
-            
-        status = command_status.get('status')
-        if status == 'completed':
-            sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) completed successfully")
-            return True
-        elif status in ['failed', 'aborted']:
-            sonarr_logger.warning(f"Sonarr {command_name} (ID: {command_id}) {status}")
-            return False
+        sonarr_logger.debug(f"Waiting for {command_name} to complete (command ID: {command_id}). Adaptive backoff, max {max_attempts} attempts")
         
-        sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) status: {status}, attempt {attempts+1}/{max_attempts}")
+        # Wait for command completion with adaptive backoff: poll less often over time to reduce API load when Sonarr is slow/queued
+        attempts = 0
+        while attempts < max_attempts:
+            if stop_check():
+                sonarr_logger.info(f"Stopping wait for {command_name} due to stop request")
+                return False
+            if instance_name and attempts % 5 == 0:
+                try:
+                    from src.primary.cycle_tracker import set_cycle_activity
+                    set_cycle_activity("sonarr", instance_name, f"Search {attempts+1}/{max_attempts}")
+                except Exception:
+                    pass
+            command_status = sonarr_api.get_command_status(api_url, api_key, api_timeout, command_id)
+            if command_status is None:
+                sonarr_logger.warning(
+                    f"Failed to get status for {command_name} (ID: {command_id}), attempt {attempts+1}. "
+                    f"Command may have already completed or the ID is no longer valid."
+                )
+                return False
+            status = command_status.get('status')
+            if status == 'completed':
+                sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) completed successfully")
+                return True
+            elif status in ['failed', 'aborted']:
+                sonarr_logger.warning(f"Sonarr {command_name} (ID: {command_id}) {status}")
+                return False
+            if attempts % 15 == 0:
+                sonarr_logger.debug(f"Sonarr {command_name} (ID: {command_id}) status: {status}, attempt {attempts+1}/{max_attempts}")
+            effective_delay = min(wait_delay + (attempts // 10), 15)
+            attempts += 1
+            time.sleep(effective_delay)
         
-        attempts += 1
-        time.sleep(wait_delay)
-    
-    sonarr_logger.error(f"Sonarr command '{command_name}' (ID: {command_id}) timed out after {max_attempts} attempts.")
-    return False
\ No newline at end of file
+        sonarr_logger.error(f"Sonarr command '{command_name}' (ID: {command_id}) timed out after {max_attempts} attempts.")
+        return False
+    finally:
+        if instance_name:
+            try:
+                from src.primary.cycle_tracker import clear_cycle_activity
+                clear_cycle_activity("sonarr", instance_name)
+            except Exception:
+                pass
\ No newline at end of file
