@@ -107,13 +107,16 @@ def generate_item_hash(item):
     return hashlib.md5(hash_input.encode('utf-8')).hexdigest()
 
 def check_for_malicious_files(item, settings):
-    """Check if download contains malicious file types"""
+    """Check if download contains malicious file types.
+    Extension check uses only the actual file extension (part after the last dot)
+    to avoid false positives like 'today.is.an.exercise.day.mkv' matching .exe.
+    """
     if not settings.get('malicious_file_detection', False):
         return False, None
     
     # Use user-defined malicious extensions from settings
     malicious_extensions = settings.get('malicious_extensions', [
-        '.lnk', '.exe', '.bat', '.cmd', '.scr', '.pif', '.com', 
+        '.lnk', '.exe', '.bat', '.cmd', '.scr', '.pif', '.com',
         '.zipx', '.jar', '.vbs', '.js', '.jse', '.wsf', '.wsh'
     ])
     
@@ -125,16 +128,29 @@ def check_for_malicious_files(item, settings):
     
     item_name = item.get('name', '').lower()
     
-    # Check for malicious extensions in the title/name
-    for ext in malicious_extensions:
-        if ext.lower() in item_name:
-            swaparr_logger.warning(f"Malicious file detected in '{item_name}': contains {ext}")
-            return True, f"Contains malicious file type: {ext}"
+    # Extension check: only match the actual file extension (last dot), not substring.
+    # e.g. "today.is.an.exercise.day.mkv" -> extension is ".mkv", not ".exe".
+    if '.' in item_name:
+        actual_extension = '.' + item_name.rsplit('.', 1)[-1]
+    else:
+        actual_extension = ''
     
-    # Check for suspicious patterns
+    normalized_malicious = set()
+    for ext in malicious_extensions:
+        e = ext.lower().strip()
+        if e and not e.startswith('.'):
+            e = '.' + e
+        if e:
+            normalized_malicious.add(e)
+    
+    if actual_extension and actual_extension in normalized_malicious:
+        swaparr_logger.warning(f"Malicious file detected in '{item.get('name', '')}': extension {actual_extension}")
+        return True, f"Contains malicious file type: {actual_extension}"
+    
+    # Check for suspicious patterns (substring match; these are intentional names like "keygen")
     for pattern in suspicious_patterns:
         if pattern.lower() in item_name:
-            swaparr_logger.warning(f"Suspicious content detected in '{item_name}': contains {pattern}")
+            swaparr_logger.warning(f"Suspicious content detected in '{item.get('name', '')}': contains {pattern}")
             return True, f"Contains suspicious content: {pattern}"
     
     return False, None
@@ -292,7 +308,12 @@ def get_queue_items(app_name, api_url, api_key, api_timeout=120):
         
         try:
             SWAPARR_STATS['api_calls_made'] += 1
-            response = requests.get(queue_url, headers=headers, timeout=api_timeout)
+            
+            # Get SSL verification setting
+            from src.primary.settings_manager import get_ssl_verify_setting
+            verify_ssl = get_ssl_verify_setting()
+            
+            response = requests.get(queue_url, headers=headers, timeout=api_timeout, verify=verify_ssl)
             response.raise_for_status()
             queue_data = response.json()
             
@@ -374,8 +395,10 @@ def parse_queue_items(records, item_type, app_name):
             "id": record.get("id"),
             "name": name,
             "size": record.get("size", 0),
+            "sizeleft": record.get("sizeleft", 0),
             "status": record.get("status", "unknown").lower(),
             "eta": eta_seconds,
+            "protocol": record.get("protocol", "unknown").lower(),
             "error_message": record.get("errorMessage", "")
         })
     
@@ -469,7 +492,12 @@ def trigger_search_for_item(app_name, api_url, api_key, item, api_timeout=120):
         
         # Execute the search command
         SWAPARR_STATS['api_calls_made'] += 1
-        response = requests.post(search_url, headers=headers, json=payload, timeout=api_timeout)
+        
+        # Get SSL verification setting
+        from src.primary.settings_manager import get_ssl_verify_setting
+        verify_ssl = get_ssl_verify_setting()
+        
+        response = requests.post(search_url, headers=headers, json=payload, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         
         swaparr_logger.info(f"Successfully triggered search for {item.get('name', 'unknown')} in {app_name}")
@@ -497,7 +525,12 @@ def delete_download(app_name, api_url, api_key, download_id, remove_from_client=
     
     try:
         SWAPARR_STATS['api_calls_made'] += 1
-        response = requests.delete(delete_url, headers=headers, timeout=api_timeout)
+        
+        # Get SSL verification setting
+        from src.primary.settings_manager import get_ssl_verify_setting
+        verify_ssl = get_ssl_verify_setting()
+        
+        response = requests.delete(delete_url, headers=headers, timeout=api_timeout, verify=verify_ssl)
         response.raise_for_status()
         swaparr_logger.info(f"Successfully removed download {download_id} from {app_name}")
         SWAPARR_STATS['downloads_removed'] += 1
@@ -520,12 +553,12 @@ def delete_download(app_name, api_url, api_key, download_id, remove_from_client=
 
 def process_stalled_downloads(app_name, instance_name, instance_data, settings):
     """Process stalled downloads for a specific app instance."""
-    swaparr_logger.info(f"Checking download queue for {app_name} instance: {instance_name}")
+    swaparr_logger.debug(f"Checking download queue for {app_name} instance: {instance_name}")
     
     try:
         # Check if instance has Swaparr enabled
         if not instance_data.get("swaparr_enabled", False):
-            swaparr_logger.info(f"Swaparr not enabled for {app_name} instance: {instance_name}, skipping")
+            swaparr_logger.debug(f"Swaparr not enabled for {app_name} instance: {instance_name}, skipping")
             return 0  # Return 0 processed
         
         # Check for disabled setting during processing (every 10 items to avoid excessive I/O)
@@ -538,10 +571,10 @@ def process_stalled_downloads(app_name, instance_name, instance_data, settings):
         queue_response = get_queue_items(app_name, instance_data["api_url"], instance_data["api_key"])
         queue_items = queue_response
         
-        swaparr_logger.info(f"Found {len(queue_items)} downloads in queue for {app_name} instance: {instance_name}")
+        swaparr_logger.debug(f"Found {len(queue_items)} downloads in queue for {app_name} instance: {instance_name}")
         
         if len(queue_items) == 0:
-            swaparr_logger.info(f"No downloads to process for {app_name} instance: {instance_name}")
+            swaparr_logger.debug(f"No downloads to process for {app_name} instance: {instance_name}")
             return 0
         
         # Load strike data and removed items for this app
@@ -778,14 +811,60 @@ def process_stalled_downloads(app_name, instance_name, instance_data, settings):
             should_strike = False
             strike_reason = ""
             
-            # Strike if metadata issue, eta too long, or no progress (eta = 0 and not queued)
+            # Calculate progress percentage for usenet downloads
+            progress_percent = 0
+            if item.get("protocol") == "usenet" and item.get("size", 0) > 0:
+                sizeleft = item.get("sizeleft", 0)
+                total_size = item.get("size", 0)
+                progress_percent = ((total_size - sizeleft) / total_size) * 100 if total_size > 0 else 0
+            
+            # For usenet downloads with 0% progress and long ETA, ignore them if the setting is enabled
+            # This solves issue #622: SABnzbd queues items sequentially, so queued items show compound ETAs
+            if settings.get("ignore_usenet_queued", True) and item.get("protocol") == "usenet" and progress_percent == 0 and item["eta"] > 0:
+                swaparr_logger.debug(f"Ignoring queued usenet download (0% progress): {item['name']} (ETA: {item['eta']}s)")
+                item_state = "Ignored (Usenet Queued)"
+                SWAPARR_STATS['items_ignored'] += 1
+                if not settings.get("dry_run", False):
+                    increment_swaparr_stat("ignored", 1)
+                continue
+            
+            # Strike if metadata issue, eta too long, or no progress (eta = 0 and not queued).
+            # Per docs/issue #687: "Has been downloading longer than your Max Download Time" must be true
+            # for ETA-too-long and No-progress strikes (wall-clock time, not ETA).
+            # Per issue #706: Do not strike/remove downloads that are 100% complete (sizeleft=0) unless
+            # remove_completed_stalled is True - they are often waiting for manual import (e.g. name/year mismatch).
+            raw_sizeleft = item.get("sizeleft")
+            try:
+                sizeleft = int(raw_sizeleft) if raw_sizeleft is not None else 0
+            except (TypeError, ValueError):
+                sizeleft = 0
+            is_completed = sizeleft == 0
+            remove_completed_stalled = settings.get("remove_completed_stalled", True)
+            if is_completed and not remove_completed_stalled:
+                swaparr_logger.debug(f"Ignoring completed download (100% - waiting for import): {item['name']}")
+                item_state = "Ignored (Completed - waiting for import)"
+                SWAPARR_STATS['items_ignored'] += 1
+                if not settings.get("dry_run", False):
+                    increment_swaparr_stat("ignored", 1)
+                continue
+
+            max_dl_seconds = parse_time_string_to_seconds(settings.get("max_download_time", "2h"))
+            first_seen_str = strike_data[item_id].get("first_strike_time")
+            exceeded_max_download_time = False
+            if first_seen_str:
+                try:
+                    first_seen = datetime.fromisoformat(first_seen_str.replace("Z", "+00:00"))
+                    time_downloading_seconds = (datetime.utcnow() - first_seen.replace(tzinfo=None)).total_seconds()
+                    exceeded_max_download_time = time_downloading_seconds >= max_dl_seconds
+                except (ValueError, TypeError):
+                    exceeded_max_download_time = False
             if metadata_issue:
                 should_strike = True
                 strike_reason = "Metadata"
-            elif item["eta"] >= parse_time_string_to_seconds(settings.get("max_download_time", "2h")):
+            elif item["eta"] >= max_dl_seconds and exceeded_max_download_time:
                 should_strike = True
                 strike_reason = "ETA too long"
-            elif item["eta"] == 0 and item["status"] not in ["queued", "delay"]:
+            elif item["eta"] == 0 and item["status"] not in ["queued", "delay"] and exceeded_max_download_time:
                 should_strike = True
                 strike_reason = "No progress"
             
@@ -844,8 +923,8 @@ def process_stalled_downloads(app_name, instance_name, instance_data, settings):
         # Update last run time
         SWAPARR_STATS['last_run_time'] = datetime.utcnow().isoformat()
         
-        swaparr_logger.info(f"Finished processing {items_processed_this_run} downloads for {app_name} instance: {instance_name}")
-        swaparr_logger.info(f"Session stats - Strikes: {SWAPARR_STATS['strikes_added']}, Removed: {SWAPARR_STATS['downloads_removed']}, Ignored: {SWAPARR_STATS['items_ignored']}, API calls: {SWAPARR_STATS['api_calls_made']}")
+        swaparr_logger.debug(f"Finished processing {items_processed_this_run} downloads for {app_name} instance: {instance_name}")
+        swaparr_logger.debug(f"Session stats - Strikes: {SWAPARR_STATS['strikes_added']}, Removed: {SWAPARR_STATS['downloads_removed']}, Ignored: {SWAPARR_STATS['items_ignored']}, API calls: {SWAPARR_STATS['api_calls_made']}")
         
         return items_processed_this_run
     except Exception as e:
@@ -863,9 +942,9 @@ def run_swaparr():
         # Swaparr is disabled - no need to log this repeatedly
         return
     
-    swaparr_logger.info("Starting Swaparr stalled download detection cycle")
+    # swaparr_logger.info("Starting Swaparr stalled download detection cycle")
     
-    instances = get_configured_instances()
+    instances = get_configured_instances(quiet=True)
     total_instances = sum(len(app_instances) for app_instances in instances.values())
     
     # Count only Swaparr-enabled instances
@@ -898,7 +977,7 @@ def run_swaparr():
                 continue
             
             swaparr_enabled_instances += 1
-            swaparr_logger.info(f"Processing {app_name} instance '{instance_name}' - Swaparr enabled")
+            swaparr_logger.debug(f"Processing {app_name} instance '{instance_name}' - Swaparr enabled")
             
             # Check if Swaparr has been disabled during processing
             current_settings = load_settings("swaparr")
@@ -916,8 +995,9 @@ def run_swaparr():
                 processed_instances += 1
     
     stats = get_session_stats()
-    swaparr_logger.info(f"=== SWAPARR cycle completed. Processed {processed_instances} Swaparr-enabled app instances. ===")
+    # swaparr_logger.info(f"=== SWAPARR cycle completed. Processed {processed_instances} Swaparr-enabled app instances. ===")
     
     # Log summary stats if there was activity
     if stats['total_processed'] > 0:
+        swaparr_logger.info(f"=== SWAPARR cycle completed. Processed {processed_instances} Swaparr-enabled app instances. ===")
         swaparr_logger.info(f"Swaparr activity summary: {stats['strikes_added']} strikes added, {stats['downloads_removed']} downloads removed, {stats['items_ignored']} items ignored") 
\ No newline at end of file
