@@ -16,7 +16,7 @@ whisparr_logger = get_logger("whisparr")
 
 
 
-def get_configured_instances():
+def get_configured_instances(quiet=False):
     """Get all configured and enabled Whisparr instances"""
     settings = load_settings("whisparr")
     instances = []
@@ -24,7 +24,8 @@ def get_configured_instances():
 
 
     if not settings:
-        whisparr_logger.debug("No settings found for Whisparr")
+        if not quiet:
+            whisparr_logger.debug("No settings found for Whisparr")
         return instances
 
     # Always use Whisparr V2 API
@@ -43,16 +44,31 @@ def get_configured_instances():
 
             # Enhanced URL validation - ensure URL has proper scheme
             if api_url and not (api_url.startswith('http://') or api_url.startswith('https://')):
-                whisparr_logger.debug(f"Instance '{instance.get('name', 'Unnamed')}' has URL without http(s) scheme: {api_url}")
+                if not quiet:
+                    whisparr_logger.debug(f"Instance '{instance.get('name', 'Unnamed')}' has URL without http(s) scheme: {api_url}")
                 api_url = f"http://{api_url}"
-                whisparr_logger.debug(f"Auto-correcting URL to: {api_url}")
+                if not quiet:
+                    whisparr_logger.debug(f"Auto-correcting URL to: {api_url}")
 
             is_enabled = instance.get("enabled", True)
 
             # Only include properly configured instances
             if is_enabled and api_url and api_key:
-                instance_name = instance.get("name", "Default")
-                
+                raw = instance.get("name", "Default") or "Default"
+                instance_name = (raw.strip() if isinstance(raw, str) else "Default") or "Default"
+                # Ensure stable instance_id so renaming does not break tracking
+                instance_id = instance.get("instance_id")
+                if not instance_id:
+                    from src.primary.utils.instance_id import generate_instance_id
+                    from src.primary.settings_manager import save_settings
+                    from src.primary.utils.database import get_database
+                    existing_ids = {inst.get("instance_id") for inst in settings["instances"] if isinstance(inst, dict) and inst.get("instance_id")}
+                    instance_id = generate_instance_id("whisparr", existing_ids)
+                    settings["instances"][idx]["instance_id"] = instance_id
+                    save_settings("whisparr", settings)
+                    get_database().migrate_instance_identifier("whisparr", instance_name, instance_id)
+                    instance_id = settings["instances"][idx].get("instance_id")
+
                 # Create a settings object for this instance by combining global settings with instance-specific ones
                 instance_settings = settings.copy()
                 
@@ -64,6 +80,7 @@ def get_configured_instances():
                 instance_settings["api_url"] = api_url
                 instance_settings["api_key"] = api_key
                 instance_settings["instance_name"] = instance_name
+                instance_settings["instance_id"] = instance_id
                 instance_settings["swaparr_enabled"] = instance.get("swaparr_enabled", False)
                 
                 # Add timeout setting with default if not present
@@ -71,22 +88,44 @@ def get_configured_instances():
                     instance_settings["api_timeout"] = 30
                 
                 # Use debug level to prevent log spam
-                whisparr_logger.debug(f"Adding configured Whisparr instance: {instance_name}")
+                if not quiet:
+                    whisparr_logger.debug(f"Adding configured Whisparr instance: {instance_name}")
 
                 # Return only essential instance details including per-instance hunt values
                 instance_data = {
+                    "instance_id": instance_id,
                     "instance_name": instance_name,
                     "api_url": api_url,
                     "api_key": api_key,
                     "swaparr_enabled": instance.get("swaparr_enabled", False),
                     "hunt_missing_items": instance.get("hunt_missing_items", 1),  # Per-instance missing hunt value
                     "hunt_upgrade_items": instance.get("hunt_upgrade_items", 0),  # Per-instance upgrade hunt value
+                    "upgrade_selection_method": (instance.get("upgrade_selection_method") or "cutoff").strip().lower(),
+                    "upgrade_tag": (instance.get("upgrade_tag") or "").strip(),
+                    "sleep_duration": instance.get("sleep_duration", settings.get("sleep_duration", 900)),
+                    "hourly_cap": instance.get("hourly_cap", settings.get("hourly_cap", 20)),
+                    "exempt_tags": instance.get("exempt_tags") or [],
+                    "state_management_hours": instance.get("state_management_hours", 72),  # CRITICAL for Issue #717 fix
+                    "state_management_mode": instance.get("state_management_mode", "custom"),  # CRITICAL for Issue #717 fix
+                    "api_timeout": instance.get("api_timeout", 120),
+                    "command_wait_delay": instance.get("command_wait_delay", 1),
+                    "command_wait_attempts": instance.get("command_wait_attempts", 600),
+                    "max_download_queue_size": instance.get("max_download_queue_size", -1),
+                    "max_seed_queue_size": instance.get("max_seed_queue_size", -1),
+                    "seed_check_torrent_client": instance.get("seed_check_torrent_client"),
+                    # Tag settings (CRITICAL: must be passed through or they default to True)
+                    "tag_processed_items": instance.get("tag_processed_items", False),
+                    "tag_enable_missing": instance.get("tag_enable_missing", False),
+                    "tag_enable_upgrade": instance.get("tag_enable_upgrade", False),
+                    "tag_enable_upgraded": instance.get("tag_enable_upgraded", False),
+                    "custom_tags": instance.get("custom_tags", {}),
                 }
                 instances.append(instance_data)
             else:
                 name = instance.get("name", "Unnamed")
                 if not is_enabled:
-                    whisparr_logger.debug(f"Skipping disabled instance: {name}")
+                    if not quiet:
+                        whisparr_logger.debug(f"Skipping disabled instance: {name}")
                 else:
                     # For brand new installations, don't spam logs with warnings about default instances
                     if name == 'Default':
@@ -94,9 +133,11 @@ def get_configured_instances():
                         pass
                     else:
                         # Still log warnings for non-default instances
-                        whisparr_logger.warning(f"Skipping instance {name} due to missing API URL or API Key")
+                        if not quiet:
+                            whisparr_logger.warning(f"Skipping instance {name} due to missing API URL or API Key")
     else:
-        whisparr_logger.debug("No instances array found in settings or it's empty")
+        if not quiet:
+            whisparr_logger.debug("No instances array found in settings or it's empty")
     
     # Use debug level to avoid spamming logs, especially with 0 instances
     return instances
