@@ -85,13 +85,13 @@ def arr_request(api_url: str, api_key: str, api_timeout: int, endpoint: str, met
                 whisparr_logger.debug(f"Standard path returned 404, trying with V3 path: {v3_url}")
                 
                 if method == "GET":
-                    response = session.get(v3_url, headers=headers, timeout=api_timeout)
+                    response = session.get(v3_url, headers=headers, timeout=api_timeout, verify=verify_ssl)
                 elif method == "POST":
-                    response = session.post(v3_url, headers=headers, json=data, timeout=api_timeout)
+                    response = session.post(v3_url, headers=headers, json=data, timeout=api_timeout, verify=verify_ssl)
                 elif method == "PUT":
-                    response = session.put(v3_url, headers=headers, json=data, timeout=api_timeout)
+                    response = session.put(v3_url, headers=headers, json=data, timeout=api_timeout, verify=verify_ssl)
                 elif method == "DELETE":
-                    response = session.delete(v3_url, headers=headers, timeout=api_timeout)
+                    response = session.delete(v3_url, headers=headers, timeout=api_timeout, verify=verify_ssl)
                 
                 whisparr_logger.debug(f"V3 path request returned status code: {response.status_code}")
             
@@ -239,24 +239,6 @@ def get_cutoff_unmet_items(api_url: str, api_key: str, api_timeout: int, monitor
         whisparr_logger.error(f"Error retrieving cutoff unmet items: {str(e)}")
         return None
 
-def refresh_item(api_url: str, api_key: str, api_timeout: int, item_id: int) -> int:
-    """
-    Refresh functionality has been removed as it was a performance bottleneck.
-    This function now returns a placeholder command ID without making any API calls.
-    
-    Args:
-        api_url: The base URL of the Whisparr API
-        api_key: The API key for authentication
-        api_timeout: Timeout for the API request
-        item_id: The ID of the item to refresh
-        
-    Returns:
-        A placeholder command ID (123) to simulate success
-    """
-    whisparr_logger.debug(f"Refresh functionality disabled for item ID: {item_id}")
-    # Return a placeholder command ID to simulate success without actually refreshing
-    return 123
-
 def item_search(api_url: str, api_key: str, api_timeout: int, item_ids: List[int]) -> int:
     """
     Trigger a search for one or more items.
@@ -315,15 +297,7 @@ def item_search(api_url: str, api_key: str, api_timeout: int, item_ids: List[int
             if result and "id" in result:
                 command_id = result["id"]
                 whisparr_logger.debug(f"Search command triggered with ID {command_id}")
-                
-                # Increment API counter after successful request
-                try:
-                    from src.primary.stats_manager import increment_hourly_cap
-                    increment_hourly_cap("whisparr", 1)
-                    whisparr_logger.debug(f"Incremented Whisparr hourly API cap for item search ({len(item_ids)} items)")
-                except Exception as cap_error:
-                    whisparr_logger.error(f"Failed to increment hourly API cap for item search: {cap_error}")
-                
+                # API bar incremented by caller via increment_stat (one count per hunt/upgrade)
                 return command_id
             else:
                 whisparr_logger.error("Failed to trigger search command - no command ID returned")
@@ -448,16 +422,61 @@ def check_connection(api_url: str, api_key: str, api_timeout: int) -> bool:
         whisparr_logger.error(f"Error checking connection to Whisparr V2 API: {str(e)}")
         return False
 
+def get_tag_id_by_label(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
+    """Get tag ID by label (lookup only). Returns None if tag does not exist. Issue #676."""
+    try:
+        response = arr_request(api_url, api_key, api_timeout, "tag", count_api=False)
+        if response:
+            for tag in response:
+                if tag.get('label') == tag_label:
+                    return tag.get('id')
+        return None
+    except Exception as e:
+        whisparr_logger.error(f"Error getting tag '{tag_label}': {e}")
+        return None
+
+
+def get_exempt_tag_ids(api_url: str, api_key: str, api_timeout: int, exempt_tag_labels: list) -> dict:
+    """Resolve exempt tag labels to tag IDs. Returns dict tag_id -> label. Exact match. Issue #676.
+    Only counts tags that are actually entered (non-empty); empty list or all-whitespace = no exclusions.
+    """
+    if exempt_tag_labels is None:
+        return {}
+    if isinstance(exempt_tag_labels, str):
+        exempt_tag_labels = [exempt_tag_labels]
+    labels = [str(l).strip() for l in (exempt_tag_labels or []) if l is not None and str(l).strip()]
+    if not labels:
+        return {}
+    result = {}
+    for label in labels:
+        tid = get_tag_id_by_label(api_url, api_key, api_timeout, label)
+        if tid is not None:
+            result[tid] = label
+    return result
+
+
+def get_series(api_url: str, api_key: str, api_timeout: int, series_id: Optional[int] = None) -> Union[List, Dict, None]:
+    """
+    Get series information from Whisparr.
+    Returns list of all series, a single series, or None if request failed.
+    """
+    if series_id is not None:
+        endpoint = f"series/{series_id}"
+    else:
+        endpoint = "series"
+    return arr_request(api_url, api_key, api_timeout, endpoint, count_api=False)
+
+
 def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: str) -> Optional[int]:
     """
     Get existing tag ID or create a new tag in Whisparr.
-    
+
     Args:
         api_url: The base URL of the Whisparr API
         api_key: The API key for authentication
         api_timeout: Timeout for the API request
         tag_label: The label/name of the tag to create or find
-        
+
     Returns:
         The tag ID if successful, None otherwise
     """
@@ -470,7 +489,7 @@ def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: s
                     tag_id = tag.get('id')
                     whisparr_logger.debug(f"Found existing tag '{tag_label}' with ID: {tag_id}")
                     return tag_id
-        
+
         # Tag doesn't exist, create it
         tag_data = {"label": tag_label}
         response = arr_request(api_url, api_key, api_timeout, "tag", method="POST", data=tag_data, count_api=False)
@@ -481,7 +500,7 @@ def get_or_create_tag(api_url: str, api_key: str, api_timeout: int, tag_label: s
         else:
             whisparr_logger.error(f"Failed to create tag '{tag_label}'. Response: {response}")
             return None
-            
+
     except Exception as e:
         whisparr_logger.error(f"Error managing tag '{tag_label}': {e}")
         return None
