@@ -13,10 +13,14 @@ from typing import List, Dict, Any, Set, Callable
 from src.primary.utils.logger import get_logger
 from src.primary.apps.whisparr import api as whisparr_api
 from src.primary.settings_manager import load_settings, get_advanced_setting
-from src.primary.stateful_manager import is_processed, add_processed_id
-from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
+from src.primary.stateful_manager import add_processed_id
+from src.primary.stats_manager import increment_stat
 from src.primary.utils.history_utils import log_processed_media
 from src.primary.state import check_state_reset
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item
 
 # Get logger for the app
 whisparr_logger = get_logger("whisparr")
@@ -41,36 +45,26 @@ def process_missing_items(
     # Reset state files if enough time has passed
     check_state_reset("whisparr")
     
-    # Load settings to check if tagging is enabled
-    whisparr_settings = load_settings("whisparr")
-    tag_processed_items = whisparr_settings.get("tag_processed_items", True)
+    # Extract common settings using shared utility
+    # Whisparr uses hunt_missing_items with fallback to hunt_missing_scenes
+    hunt_key_value = app_settings.get("hunt_missing_items", app_settings.get("hunt_missing_scenes", 0))
+    app_settings["hunt_missing_items"] = hunt_key_value  # Normalize for extract
+    s = extract_app_settings(app_settings, "whisparr", "hunt_missing_items", "Whisparr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_missing_items = s['hunt_count']
+    tag_settings = s['tag_settings']
     
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    instance_name = app_settings.get("instance_name", "Whisparr Default")
-    
-    # Use the centralized advanced setting for stateful management hours
-    stateful_management_hours = get_advanced_setting("stateful_management_hours", 168)
-    
-    monitored_only = app_settings.get("monitored_only", True)
+    # App-specific settings
     skip_future_releases = app_settings.get("skip_future_releases", True)
-    # skip_item_refresh setting removed as it was a performance bottleneck
     
-    # Use the new hunt_missing_items parameter name, falling back to hunt_missing_scenes for backwards compatibility
-    hunt_missing_items = app_settings.get("hunt_missing_items", app_settings.get("hunt_missing_scenes", 0))
-    
-    # Use advanced settings from database for command operations
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-    
-    # Log that we're using Whisparr V2 API
     whisparr_logger.debug(f"Using Whisparr V2 API for instance: {instance_name}")
 
-    # Skip if hunt_missing_items is set to 0
-    if hunt_missing_items <= 0:
-        whisparr_logger.info("'hunt_missing_items' setting is 0 or less. Skipping missing item processing.")
+    if not validate_settings(api_url, api_key, hunt_missing_items, "whisparr", whisparr_logger):
         return False
 
     # Check for stop signal
@@ -116,16 +110,33 @@ def process_missing_items(
     if not missing_items:
         whisparr_logger.info("No missing items left to process after filtering future releases.")
         return False
+
+    # Filter out items whose series has an exempt tag (issue #676)
+    exempt_tags = app_settings.get("exempt_tags") or []
+    if exempt_tags:
+        exempt_id_to_label = whisparr_api.get_exempt_tag_ids(api_url, api_key, api_timeout, exempt_tags)
+        if exempt_id_to_label:
+            all_series = whisparr_api.get_series(api_url, api_key, api_timeout)
+            exempt_series_ids = set()
+            if all_series:
+                if not isinstance(all_series, list):
+                    all_series = [all_series]
+                for s in all_series:
+                    for tid in (s.get("tags") or []):
+                        if tid in exempt_id_to_label:
+                            exempt_series_ids.add(s.get("id"))
+                            whisparr_logger.info(
+                                f"Skipping series \"{s.get('title', 'Unknown')}\" (ID: {s.get('id')}) - has exempt tag \"{exempt_id_to_label[tid]}\""
+                            )
+                            break
+            missing_items = [item for item in missing_items if item.get("seriesId") not in exempt_series_ids]
+            whisparr_logger.info(f"Exempt tags filter: {len(missing_items)} items remaining after excluding series with exempt tags.")
         
-    # Filter out already processed items using stateful management
-    unprocessed_items = []
-    for item in missing_items:
-        item_id = str(item.get("id"))
-        if not is_processed("whisparr", instance_name, item_id):
-            unprocessed_items.append(item)
-        else:
-            whisparr_logger.debug(f"Skipping already processed item ID: {item_id}")
-    
+    # Filter out already processed items using shared utility
+    unprocessed_items = filter_unprocessed(
+        missing_items, "whisparr", instance_key,
+        get_id_fn=lambda item: item.get("id"), logger=whisparr_logger
+    )
     whisparr_logger.info(f"Found {len(unprocessed_items)} unprocessed items out of {len(missing_items)} total items with missing files.")
     
     if not unprocessed_items:
@@ -143,20 +154,9 @@ def process_missing_items(
 
     # Process selected items
     for item in items_to_search:
-        # Check for stop signal before each item
-        if stop_check():
-            whisparr_logger.info("Stop requested during item processing. Aborting...")
+        if not should_continue_processing("whisparr", stop_check, whisparr_logger):
             break
         
-        # Check API limit before processing each item
-        try:
-            if check_hourly_cap_exceeded("whisparr"):
-                whisparr_logger.warning(f"🛑 Whisparr API hourly limit reached - stopping missing items processing after {items_processed} items")
-                break
-        except Exception as e:
-            whisparr_logger.error(f"Error checking hourly API cap: {e}")
-            # Continue processing if cap check fails - safer than stopping
-        
         # Re-check limit in case it changed
         current_limit = app_settings.get("hunt_missing_items", app_settings.get("hunt_missing_scenes", 1))
         if items_processed >= current_limit:
@@ -172,7 +172,7 @@ def process_missing_items(
         # Refresh functionality has been removed as it was identified as a performance bottleneck
         
         # Mark the item as processed BEFORE triggering any searches
-        add_processed_id("whisparr", instance_name, str(item_id))
+        add_processed_id("whisparr", instance_key, str(item_id))
         whisparr_logger.debug(f"Added item ID {item_id} to processed list for {instance_name}")
         
         # Check for stop signal before searching
@@ -186,28 +186,23 @@ def process_missing_items(
         if search_command_id:
             whisparr_logger.info(f"Triggered search command {search_command_id}. Assuming success for now.")
             
-            # Tag the series if enabled
-            if tag_processed_items:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("whisparr", "missing", "huntarr-missing")
-                series_id = item.get('seriesId')
-                if series_id:
-                    try:
-                        whisparr_api.tag_processed_series(api_url, api_key, api_timeout, series_id, custom_tag)
-                        whisparr_logger.debug(f"Tagged series {series_id} with '{custom_tag}'")
-                    except Exception as e:
-                        whisparr_logger.warning(f"Failed to tag series {series_id} with '{custom_tag}': {e}")
+            # Tag the series if enabled (unified tagging)
+            series_id = item.get('seriesId')
+            if series_id:
+                try_tag_item(tag_settings, "missing", whisparr_api.tag_processed_series,
+                             api_url, api_key, api_timeout, series_id,
+                             whisparr_logger, f"series {series_id}")
             
             # Log to history system
             media_name = f"{title} - {season_episode}"
-            log_processed_media("whisparr", media_name, item_id, instance_name, "missing")
+            log_processed_media("whisparr", media_name, item_id, instance_key, "missing", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             whisparr_logger.debug(f"Logged history entry for item: {media_name}")
             
             items_processed += 1
             processing_done = True
             
             # Increment the hunted statistics for Whisparr
-            increment_stat("whisparr", "hunted", 1)
+            increment_stat("whisparr", "hunted", 1, instance_key)
             whisparr_logger.debug(f"Incremented whisparr hunted statistics by 1")
 
             # Log progress
