@@ -13,10 +13,14 @@ from datetime import datetime, timedelta
 from src.primary.utils.logger import get_logger
 from src.primary.apps.whisparr import api as whisparr_api
 from src.primary.settings_manager import load_settings, get_advanced_setting
-from src.primary.stateful_manager import is_processed, add_processed_id
-from src.primary.stats_manager import increment_stat, check_hourly_cap_exceeded
+from src.primary.stateful_manager import add_processed_id
+from src.primary.stats_manager import increment_stat
 from src.primary.utils.history_utils import log_processed_media
 from src.primary.state import check_state_reset
+from src.primary.apps._common.settings import extract_app_settings, validate_settings
+from src.primary.apps._common.filtering import filter_unprocessed
+from src.primary.apps._common.processing import should_continue_processing
+from src.primary.apps._common.tagging import try_tag_item, extract_tag_settings
 
 # Get logger for the app
 whisparr_logger = get_logger("whisparr")
@@ -41,34 +45,23 @@ def process_cutoff_upgrades(
     # Reset state files if enough time has passed
     check_state_reset("whisparr")
     
-    # Load settings to check if tagging is enabled
-    whisparr_settings = load_settings("whisparr")
-    tag_processed_items = whisparr_settings.get("tag_processed_items", True)
+    # Extract common settings using shared utility
+    # Whisparr uses hunt_upgrade_items with fallback to hunt_upgrade_scenes
+    hunt_key_value = app_settings.get("hunt_upgrade_items", app_settings.get("hunt_upgrade_scenes", 0))
+    app_settings["hunt_upgrade_items"] = hunt_key_value  # Normalize for extract
+    s = extract_app_settings(app_settings, "whisparr", "hunt_upgrade_items", "Whisparr Default")
+    instance_name = s['instance_name']
+    instance_key = s['instance_key']
+    api_url = s['api_url']
+    api_key = s['api_key']
+    api_timeout = s['api_timeout']
+    monitored_only = s['monitored_only']
+    hunt_upgrade_items = s['hunt_count']
+    tag_settings = extract_tag_settings(app_settings)
     
-    # Extract necessary settings
-    api_url = app_settings.get("api_url", "").strip()
-    api_key = app_settings.get("api_key", "").strip()
-    api_timeout = get_advanced_setting("api_timeout", 120)  # Use database value
-    instance_name = app_settings.get("instance_name", "Whisparr Default")
-    
-    # Use advanced settings from database for command operations
-    command_wait_delay = get_advanced_setting("command_wait_delay", 1)
-    command_wait_attempts = get_advanced_setting("command_wait_attempts", 600)
-    
-    monitored_only = app_settings.get("monitored_only", True)
-    # skip_item_refresh setting removed as it was a performance bottleneck
-    
-    # Use the new hunt_upgrade_items parameter name, falling back to hunt_upgrade_scenes for backwards compatibility
-    hunt_upgrade_items = app_settings.get("hunt_upgrade_items", app_settings.get("hunt_upgrade_scenes", 0))
-    
-    state_reset_interval_hours = get_advanced_setting("stateful_management_hours", 168)  
-    
-    # Log that we're using Whisparr V2 API
     whisparr_logger.debug(f"Using Whisparr V2 API for instance: {instance_name}")
 
-    # Skip if hunt_upgrade_items is set to 0
-    if hunt_upgrade_items <= 0:
-        whisparr_logger.info("'hunt_upgrade_items' setting is 0 or less. Skipping quality upgrade processing.")
+    if not validate_settings(api_url, api_key, hunt_upgrade_items, "whisparr", whisparr_logger):
         return False
 
     # Check for stop signal
@@ -90,16 +83,33 @@ def process_cutoff_upgrades(
         return False
         
     whisparr_logger.info(f"Found {len(upgrade_eligible_data)} items eligible for quality upgrade.")
-    
-    # Filter out already processed items using stateful management
-    unprocessed_items = []
-    for item in upgrade_eligible_data:
-        item_id = str(item.get("id"))
-        if not is_processed("whisparr", instance_name, item_id):
-            unprocessed_items.append(item)
-        else:
-            whisparr_logger.debug(f"Skipping already processed item ID: {item_id}")
-    
+
+    # Filter out items whose series has an exempt tag (issue #676)
+    exempt_tags = app_settings.get("exempt_tags") or []
+    if exempt_tags:
+        exempt_id_to_label = whisparr_api.get_exempt_tag_ids(api_url, api_key, api_timeout, exempt_tags)
+        if exempt_id_to_label:
+            all_series = whisparr_api.get_series(api_url, api_key, api_timeout)
+            exempt_series_ids = set()
+            if all_series:
+                if not isinstance(all_series, list):
+                    all_series = [all_series]
+                for s in all_series:
+                    for tid in (s.get("tags") or []):
+                        if tid in exempt_id_to_label:
+                            exempt_series_ids.add(s.get("id"))
+                            whisparr_logger.info(
+                                f"Skipping series \"{s.get('title', 'Unknown')}\" (ID: {s.get('id')}) - has exempt tag \"{exempt_id_to_label[tid]}\""
+                            )
+                            break
+            upgrade_eligible_data = [item for item in upgrade_eligible_data if item.get("seriesId") not in exempt_series_ids]
+            whisparr_logger.info(f"Exempt tags filter: {len(upgrade_eligible_data)} items remaining for upgrades after excluding series with exempt tags.")
+
+    # Filter out already processed items using shared utility
+    unprocessed_items = filter_unprocessed(
+        upgrade_eligible_data, "whisparr", instance_key,
+        get_id_fn=lambda item: item.get("id"), logger=whisparr_logger
+    )
     whisparr_logger.info(f"Found {len(unprocessed_items)} unprocessed items out of {len(upgrade_eligible_data)} total items eligible for quality upgrade.")
     
     if not unprocessed_items:
@@ -117,19 +127,8 @@ def process_cutoff_upgrades(
     
     # Process selected items
     for item in items_to_upgrade:
-        # Check for stop signal before each item
-        if stop_check():
-            whisparr_logger.info("Stop requested during item processing. Aborting...")
+        if not should_continue_processing("whisparr", stop_check, whisparr_logger):
             break
-        
-        # Check API limit before processing each item
-        try:
-            if check_hourly_cap_exceeded("whisparr"):
-                whisparr_logger.warning(f"🛑 Whisparr API hourly limit reached - stopping upgrade processing after {items_processed} items")
-                break
-        except Exception as e:
-            whisparr_logger.error(f"Error checking hourly API cap: {e}")
-            # Continue processing if cap check fails - safer than stopping
             
         # Re-check limit in case it changed
         current_limit = app_settings.get("hunt_upgrade_items", app_settings.get("hunt_upgrade_scenes", 1))
@@ -154,7 +153,7 @@ def process_cutoff_upgrades(
             break
         
         # Mark the item as processed BEFORE triggering any searches
-        add_processed_id("whisparr", instance_name, str(item_id))
+        add_processed_id("whisparr", instance_key, str(item_id))
         whisparr_logger.debug(f"Added item ID {item_id} to processed list for {instance_name}")
         
         # Search for the item
@@ -163,29 +162,24 @@ def process_cutoff_upgrades(
         if search_command_id:
             whisparr_logger.info(f"Triggered search command {search_command_id}. Assuming success for now.")
             
-            # Tag the series if enabled
-            if tag_processed_items:
-                from src.primary.settings_manager import get_custom_tag
-                custom_tag = get_custom_tag("whisparr", "upgrade", "huntarr-upgraded")
-                series_id = item.get('seriesId')
-                if series_id:
-                    try:
-                        whisparr_api.tag_processed_series(api_url, api_key, api_timeout, series_id, custom_tag)
-                        whisparr_logger.debug(f"Tagged series {series_id} with '{custom_tag}'")
-                    except Exception as e:
-                        whisparr_logger.warning(f"Failed to tag series {series_id} with '{custom_tag}': {e}")
+            # Tag the series if enabled (unified tagging)
+            series_id = item.get('seriesId')
+            if series_id:
+                try_tag_item(tag_settings, "upgraded", whisparr_api.tag_processed_series,
+                             api_url, api_key, api_timeout, series_id,
+                             whisparr_logger, f"series {series_id}")
             
             # Log to history so the upgrade appears in the history UI
             series_title = item.get("series", {}).get("title", "Unknown Series")
             media_name = f"{series_title} - {season_episode} - {title}"
-            log_processed_media("whisparr", media_name, item_id, instance_name, "upgrade")
+            log_processed_media("whisparr", media_name, item_id, instance_key, "upgrade", display_name_for_log=app_settings.get("instance_display_name") or instance_name)
             whisparr_logger.debug(f"Logged quality upgrade to history for item ID {item_id}")
             
             items_processed += 1
             processing_done = True
             
             # Increment the upgraded statistics for Whisparr
-            increment_stat("whisparr", "upgraded", 1)
+            increment_stat("whisparr", "upgraded", 1, instance_key)
             whisparr_logger.debug(f"Incremented whisparr upgraded statistics by 1")
             
             # Log progress
