@@ -10,6 +10,7 @@ import json
 import hashlib
 import secrets
 import time
+import threading
 import pathlib
 import base64
 import io
@@ -23,13 +24,36 @@ from typing import Dict, Any, Optional, Tuple, Union
 from flask import request, redirect, url_for, session
 from .utils.logger import logger # Ensure logger is imported
 
-# Database setup
 from src.primary.utils.database import get_database
+from src.primary import settings_manager
 
-# Session settings
-SESSION_EXPIRY = 60 * 60 * 24 * 7  # 1 week in seconds
+SESSION_EXPIRY = 60 * 60 * 24 * 7
 SESSION_COOKIE_NAME = "huntarr_session"
 
+# In-memory cache for auth middleware hot-path checks (avoids DB hit per request)
+_auth_cache = {
+    "user_exists": None,           # bool or None
+    "user_exists_ts": 0,           # timestamp
+    "setup_in_progress": None,     # bool or None
+    "setup_in_progress_ts": 0,     # timestamp
+    "auth_settings": None,         # dict or None
+    "auth_settings_ts": 0,         # timestamp
+}
+_AUTH_CACHE_TTL = 10  # seconds — short enough to pick up setup/login changes quickly
+_auth_cache_lock = threading.Lock()
+
+def get_base_url_path():
+    try:
+        base_url = settings_manager.get_setting('general', 'base_url', '').strip()
+        if not base_url or base_url == '/':
+            return ''
+        base_url = base_url.strip('/')
+        base_url = '/' + base_url
+        return base_url
+    except Exception as e:
+        logger.error(f"Error getting base_url from settings: {e}")
+        return ''
+
 # Plex OAuth settings
 PLEX_CLIENT_IDENTIFIER = None  # Will be generated on first use
 PLEX_PRODUCT_NAME = "Huntarr"
@@ -37,6 +61,7 @@ PLEX_VERSION = "1.0"
 
 # Store active sessions
 active_sessions = {}
+_session_cleanup_ts = 0  # Last time expired sessions were swept
 
 # Store active Plex PINs
 active_plex_pins = {}
@@ -51,8 +76,8 @@ def get_user_data(username: str = None) -> Dict[str, Any]:
         # For backward compatibility, return first user if no username specified
         # This is used in legacy code that expects single user
         try:
-            # Get the first user from the database
-            with sqlite3.connect(db.db_path) as conn:
+            # Get the first user from the database using configured connection
+            with db.get_connection() as conn:
                 conn.row_factory = sqlite3.Row
                 cursor = conn.execute('SELECT * FROM users LIMIT 1')
                 row = cursor.fetchone()
@@ -118,22 +143,54 @@ def save_user_data(user_data: Dict[str, Any]) -> bool:
         return False
 # --- End Helper functions ---
 
+
+def _password_is_hashed(stored: str) -> bool:
+    """True if stored value looks like a hash (not plaintext). Supports bcrypt and salt:hash."""
+    if not stored or len(stored) < 10:
+        return False
+    if stored.startswith("$2") and stored.count("$") >= 3:
+        return True  # bcrypt
+    if ":" in stored and len(stored) > 40:
+        return True  # salt:hash (e.g. our SHA-256 format)
+    return False
+
+
 def hash_password(password: str) -> str:
-    """Hash a password for storage"""
-    # Use SHA-256 with a salt
+    """Hash a password for storage (SHA-256 with salt). Use for all new and updated passwords."""
     salt = secrets.token_hex(16)
     pw_hash = hashlib.sha256((password + salt).encode()).hexdigest()
     return f"{salt}:{pw_hash}"
 
+
 def verify_password(stored_password: str, provided_password: str) -> bool:
-    """Verify a password against its hash"""
-    try:
-        salt, pw_hash = stored_password.split(':', 1)
-        verify_hash = hashlib.sha256((provided_password + salt).encode()).hexdigest()
-        return secrets.compare_digest(verify_hash, pw_hash)
-    except Exception as e:
-        logger.error(f"Error verifying password hash: {e}", exc_info=True)
+    """
+    Verify a password against stored value.
+    Supports: salt:hash (SHA-256), bcrypt, and legacy plaintext (for migration; rehash on next login).
+    """
+    if not stored_password or not provided_password:
         return False
+    # Bcrypt
+    if stored_password.startswith("$2") and stored_password.count("$") >= 3:
+        try:
+            import bcrypt
+            return bcrypt.checkpw(
+                provided_password.encode("utf-8"),
+                stored_password.encode("utf-8")
+            )
+        except Exception as e:
+            logger.debug("Bcrypt verify failed: %s", e)
+            return False
+    # Salt:hash (our SHA-256 format)
+    if ":" in stored_password and len(stored_password) > 40:
+        try:
+            salt, pw_hash = stored_password.split(":", 1)
+            verify_hash = hashlib.sha256((provided_password + salt).encode()).hexdigest()
+            return secrets.compare_digest(verify_hash, pw_hash)
+        except Exception as e:
+            logger.debug("Salt hash verify failed: %s", e)
+            return False
+    # Legacy plaintext (allow login then rehash on next save)
+    return secrets.compare_digest(stored_password, provided_password)
 
 def hash_username(username: str) -> str:
     """Create a normalized hash of the username"""
@@ -155,31 +212,62 @@ def validate_password_strength(password: str) -> Optional[str]:
     # If check passes
     return None
 
+def invalidate_auth_cache():
+    """Clear the auth middleware cache (call after user creation, setup changes, auth mode changes)."""
+    with _auth_cache_lock:
+        _auth_cache["user_exists"] = None
+        _auth_cache["user_exists_ts"] = 0
+        _auth_cache["setup_in_progress"] = None
+        _auth_cache["setup_in_progress_ts"] = 0
+        _auth_cache["auth_settings"] = None
+        _auth_cache["auth_settings_ts"] = 0
+
+def _auto_clear_setup_progress():
+    """Silently clear any lingering setup_progress record.
+    
+    Called when a user is already authenticated (via session or bypass).
+    If they can use the app, setup is done — the record is stale.
+    Only runs once per process to avoid hitting the DB on every request.
+    """
+    # Fast path: if cache says setup is not in progress, nothing to clear
+    if _auth_cache.get("setup_in_progress") is False:
+        return
+    try:
+        db = get_database()
+        if db.is_setup_in_progress():
+            db.clear_setup_progress()
+            with _auth_cache_lock:
+                _auth_cache["setup_in_progress"] = False
+                _auth_cache["setup_in_progress_ts"] = time.time()
+            logger.info("Auto-cleared stale setup_progress (user is already authenticated)")
+    except Exception as e:
+        logger.debug(f"_auto_clear_setup_progress: {e}")
+
 def user_exists() -> bool:
     """Check if a user has been created"""
     db = get_database()
     return db.user_exists()
 
 def create_user(username: str, password: str) -> bool:
-    """Create a new user"""
+    """Create a new user. Password is hashed before storage (security)."""
     if not username or not password:
         logger.error("Attempted to create user with empty username or password")
         return False
-        
-    # Store credentials in database (no hashing as requested)
+
     db = get_database()
     success = db.create_user(
         username=username,
-        password=password,
+        password=hash_password(password),
         two_fa_enabled=False,
         two_fa_secret=None
     )
-    
+
     if success:
         logger.info("User creation successful")
+        invalidate_auth_cache()
     else:
         logger.error("User creation failed")
-    
+
     return success
 
 def verify_user(username: str, password: str, otp_code: str = None) -> Tuple[bool, bool]:
@@ -200,39 +288,51 @@ def verify_user(username: str, password: str, otp_code: str = None) -> Tuple[boo
         if not user_data:
             logger.warning(f"Login attempt failed: User '{username}' not found.")
             return False, False
-        
-        # Verify password (no hashing as requested)
-        if user_data.get("password") == password:
-            # Check if 2FA is enabled
-            two_fa_enabled = user_data.get("two_fa_enabled", False)
-            logger.debug(f"2FA enabled for user '{username}': {two_fa_enabled}")
-            logger.debug(f"2FA secret present: {bool(user_data.get('two_fa_secret'))}")
-            logger.debug(f"OTP code provided: {bool(otp_code)}")
-            
-            if two_fa_enabled:
-                # If 2FA code was provided, verify it
-                if otp_code:
-                    totp = pyotp.TOTP(user_data.get("two_fa_secret"))
-                    valid_code = totp.verify(otp_code)
-                    logger.debug(f"OTP code validation result: {valid_code}")
-                    if valid_code:
-                        logger.debug(f"User '{username}' authenticated successfully with 2FA.")
-                        return True, False
-                    else:
-                        logger.warning(f"Login attempt failed for user '{username}': Invalid 2FA code.")
-                        return False, True
-                else:
-                    # No OTP code provided but 2FA is enabled
-                    logger.warning(f"Login attempt failed for user '{username}': 2FA code required but not provided.")
-                    logger.debug("Returning needs_2fa=True to trigger 2FA input display")
-                    return False, True
-            else:
-                # 2FA not enabled, password is correct
-                logger.debug(f"User '{username}' authenticated successfully (no 2FA).")
-                return True, False
-        else:
+
+        stored_password = user_data.get("password") or ""
+        if not verify_password(stored_password, password):
             logger.warning(f"Login attempt failed for user '{username}': Invalid password.")
             return False, False
+
+        # If password was stored in plaintext, rehash and update so it is not stored plaintext (security)
+        if not _password_is_hashed(stored_password):
+            try:
+                db.update_user_password(username, hash_password(password))
+                logger.info("Rehashed plaintext password for user '%s' (security fix).", username)
+            except Exception as e:
+                logger.warning("Could not rehash plaintext password for '%s': %s", username, e)
+
+        # Check if 2FA is enabled
+        two_fa_enabled = user_data.get("two_fa_enabled", False)
+        logger.debug(f"2FA enabled for user '{username}': {two_fa_enabled}")
+        logger.debug(f"2FA secret present: {bool(user_data.get('two_fa_secret'))}")
+        logger.debug(f"OTP code provided: {bool(otp_code)}")
+
+        if two_fa_enabled:
+            two_fa_secret = user_data.get("two_fa_secret") or ""
+            if not two_fa_secret.strip():
+                logger.warning(f"Login attempt failed for user '{username}': 2FA enabled but secret missing.")
+                return False, False
+            # If 2FA code was provided, verify it
+            if otp_code:
+                totp = pyotp.TOTP(two_fa_secret)
+                valid_code = totp.verify(otp_code)
+                logger.debug(f"OTP code validation result: {valid_code}")
+                if valid_code:
+                    logger.debug(f"User '{username}' authenticated successfully with 2FA.")
+                    return True, False
+                else:
+                    logger.warning(f"Login attempt failed for user '{username}': Invalid 2FA code.")
+                    return False, True
+            else:
+                # No OTP code provided but 2FA is enabled
+                logger.warning(f"Login attempt failed for user '{username}': 2FA code required but not provided.")
+                logger.debug("Returning needs_2fa=True to trigger 2FA input display")
+                return False, True
+        else:
+            # 2FA not enabled, password is correct
+            logger.debug(f"User '{username}' authenticated successfully (no 2FA).")
+            return True, False
     except Exception as e:
         logger.error(f"Error during user verification for '{username}': {e}", exc_info=True)
     
@@ -255,19 +355,35 @@ def create_session(username: str) -> str:
 
 def verify_session(session_id: str) -> bool:
     """Verify if a session is valid"""
+    global _session_cleanup_ts
+    now = time.time()
+
+    # Periodic sweep: remove ALL expired sessions every 5 minutes
+    if now - _session_cleanup_ts > 300:
+        _session_cleanup_ts = now
+        expired = [sid for sid, sd in active_sessions.items()
+                   if sd.get("expires_at", 0) < now]
+        for sid in expired:
+            active_sessions.pop(sid, None)
+        # Also clean expired Plex PINs
+        expired_pins = [pid for pid, pd in active_plex_pins.items()
+                        if pd.get("expires_at", 0) < now]
+        for pid in expired_pins:
+            active_plex_pins.pop(pid, None)
+
     if not session_id or session_id not in active_sessions:
         return False
         
     session_data = active_sessions[session_id]
     
     # Check if session has expired
-    if session_data.get("expires_at", 0) < time.time():
+    if session_data.get("expires_at", 0) < now:
         # Clean up expired session
         del active_sessions[session_id]
         return False
         
     # Extend session expiry
-    active_sessions[session_id]["expires_at"] = time.time() + SESSION_EXPIRY
+    active_sessions[session_id]["expires_at"] = now + SESSION_EXPIRY
     return True
 
 def get_username_from_session(session_id: str) -> Optional[str]:
@@ -307,107 +423,73 @@ def authenticate_request():
     is_polling_endpoint = any(endpoint in request.path for endpoint in [
         '/api/logs/', '/api/cycle/', '/api/hourly-caps', '/api/swaparr/status'
     ])
-    
+
     if not is_polling_endpoint:
         pass  # Path checking debug removed to reduce log spam
 
     # FIRST: Always allow setup and user page access - this handles returns from external auth like Plex
-    if request.path in ['/setup', '/user']:
+    if request.path.endswith('/setup') or request.path.endswith('/user'):
         if not is_polling_endpoint:
             logger.debug(f"Allowing setup/user page access for path: {request.path}")
         return None
 
-    # Skip authentication for static files, API setup, health check path, ping, and github sponsors
-    if request.path.startswith((static_path, api_setup_path)) or request.path in (favicon_path, health_check_path, ping_path, '/api/github_sponsors', '/api/sponsors/init'):
+    # Skip authentication for static files, API setup, health check path, ping, github sponsors, and version endpoint
+    if request.path.startswith('/static/') or request.path.startswith('/api/setup') or request.path.endswith('/favicon.ico') or request.path.startswith('/api/health') or request.path.endswith('/ping') or request.path.startswith('/api/github_sponsors') or request.path.startswith('/api/sponsors/init') or request.path.endswith('/api/version'):
         return None
-    
+
     # Skip authentication for login pages, Plex auth endpoints, recovery key endpoints, and setup-related user endpoints
-    # This must come BEFORE setup checks to allow API access during setup
-    recovery_key_path = "/auth/recovery-key"
-    api_user_2fa_path = "/api/user/2fa/"
-    api_settings_general_path = "/api/settings/general"
-    
-    # Check if request is for login/auth paths (including Plex auth) - skip authentication
-    if request.path.startswith((login_path, api_login_path, api_auth_plex_path, recovery_key_path, api_user_2fa_path)) or request.path == api_settings_general_path:
+    if request.path.endswith('/login') or request.path.startswith('/api/login') or request.path.startswith('/api/auth/plex') or request.path.startswith('/auth/recovery-key') or '/api/user/2fa/' in request.path or request.path.endswith('/api/settings/general'):
         if not is_polling_endpoint:
             # Reduced logging frequency for common paths to prevent spam
             if hash(request.path) % 20 == 0:  # Log ~5% of auth skips
                 logger.debug(f"Skipping authentication for login/plex/recovery/2fa/settings path '{request.path}'")
         return None
     
-    # If no user exists, redirect to setup
-    if not user_exists():
+    # Cached auth checks — avoids hitting the database on every single request
+    now = time.time()
+    
+    # Check if user exists (cached for 10s)
+    _user_exists = _auth_cache["user_exists"]
+    if _user_exists is None or (now - _auth_cache["user_exists_ts"]) > _AUTH_CACHE_TTL:
+        _user_exists = user_exists()
+        with _auth_cache_lock:
+            _auth_cache["user_exists"] = _user_exists
+            _auth_cache["user_exists_ts"] = now
+    
+    if not _user_exists:
         if not is_polling_endpoint:
             logger.debug(f"No user exists, redirecting to setup")
-        
-        # Get the base URL from settings to ensure proper subpath redirect
-        try:
-            from src.primary.settings_manager import get_setting
-            base_url = get_setting('general', 'base_url', '')
-            if base_url and not base_url.startswith('/'):
-                base_url = f'/{base_url}'
-            if base_url and base_url.endswith('/'):
-                base_url = base_url.rstrip('/')
-            setup_url = f"{base_url}/setup" if base_url else "/setup"
-            logger.debug(f"Redirecting to setup with base URL: {setup_url}")
-            return redirect(setup_url)
-        except Exception as e:
-            logger.warning(f"Error getting base URL for setup redirect: {e}")
-            return redirect(url_for("common.setup"))
+        # Return JSON for API calls so the frontend doesn't try to parse HTML
+        if request.path.startswith("/api/"):
+            from flask import jsonify as _jsonify
+            return _jsonify({"error": "Setup required", "setup_required": True}), 503
+        return redirect(get_base_url_path() + url_for("common.setup"))
     
-    # If user exists but setup is in progress, redirect to setup
-    try:
-        from src.primary.utils.database import get_database
-        db = get_database()
-        if db.is_setup_in_progress():
-            if not is_polling_endpoint:
-                logger.debug(f"Setup is in progress, redirecting to setup")
-            
-            # Get the base URL from settings to ensure proper subpath redirect
-            try:
-                from src.primary.settings_manager import get_setting
-                base_url = get_setting('general', 'base_url', '')
-                if base_url and not base_url.startswith('/'):
-                    base_url = f'/{base_url}'
-                if base_url and base_url.endswith('/'):
-                    base_url = base_url.rstrip('/')
-                setup_url = f"{base_url}/setup" if base_url else "/setup"
-                logger.debug(f"Redirecting to setup (in progress) with base URL: {setup_url}")
-                return redirect(setup_url)
-            except Exception as e:
-                logger.warning(f"Error getting base URL for setup redirect: {e}")
-                return redirect(url_for("common.setup"))
-    except Exception as e:
-        logger.error(f"Error checking setup progress in auth middleware: {e}")
-        # Don't block access if we can't check setup progress
-    
-    # Load general settings
+    # Load auth settings EARLY (cached for 10s) — auth bypass modes must be
+    # evaluated before is_setup_in_progress so that a user who already chose
+    # No Login or Local Bypass during setup isn't locked out by leftover
+    # setup_progress records.
     local_access_bypass = False
     proxy_auth_bypass = False
-    try:
-        # Force reload settings from disk to ensure we have the latest
-        from src.primary.settings_manager import load_settings
-        from src.primary import settings_manager
-        
-        # Ensure we're getting fresh settings by clearing any cache
-        if hasattr(settings_manager, 'settings_cache'):
-            settings_manager.settings_cache = {}
-            
-        settings = load_settings("general")  # Specify 'general' as the app_type
-        general_settings = settings
-        local_access_bypass = general_settings.get("local_access_bypass", False)
-        proxy_auth_bypass = general_settings.get("proxy_auth_bypass", False)
-        
-        # Log settings only for non-polling endpoints to reduce spam
-        if not is_polling_endpoint:
-            pass
-
-    except Exception as e:
-        logger.error(f"Error loading authentication bypass settings: {e}", exc_info=True)
+    _cached_settings = _auth_cache["auth_settings"]
+    if _cached_settings is None or (now - _auth_cache["auth_settings_ts"]) > _AUTH_CACHE_TTL:
+        try:
+            from src.primary.settings_manager import load_settings
+            _cached_settings = load_settings("general")
+        except Exception as e:
+            logger.error(f"Error loading authentication bypass settings: {e}")
+            _cached_settings = {}
+        with _auth_cache_lock:
+            _auth_cache["auth_settings"] = _cached_settings
+            _auth_cache["auth_settings_ts"] = now
+    
+    local_access_bypass = _cached_settings.get("local_access_bypass", False)
+    proxy_auth_bypass = _cached_settings.get("proxy_auth_bypass", False)
     
     # Check if proxy auth bypass is enabled - this completely disables authentication
-    # Note: This has highest priority and is checked first (matching the "No Login Mode" in the UI)
+    # Checked before is_setup_in_progress so "No Login Mode" users aren't blocked
     if proxy_auth_bypass:
+        _auto_clear_setup_progress()
         return None
     
     remote_addr = request.remote_addr
@@ -463,18 +545,42 @@ def authenticate_request():
         if is_local:
             if not is_polling_endpoint:
                 logger.debug(f"Local network access from {remote_addr} - Authentication bypassed! (Local Bypass Mode)")
+            _auto_clear_setup_progress()
             return None
         else:
             if not is_polling_endpoint:
                 logger.warning(f"Access from {remote_addr} is not recognized as local network - Authentication required")
     
-    # Check for valid session
+    # Check for valid session BEFORE is_setup_in_progress so that
+    # logged-in users aren't kicked back to setup by stale records.
     session_id = request.cookies.get(SESSION_COOKIE_NAME)
     if session_id and verify_session(session_id):
         if not is_polling_endpoint:
             pass  # Session valid - debug spam removed
+        _auto_clear_setup_progress()
         return None
     
+    # No bypass, no session — check if setup is still in progress.
+    _setup_in_progress = _auth_cache["setup_in_progress"]
+    if _setup_in_progress is None or (now - _auth_cache["setup_in_progress_ts"]) > _AUTH_CACHE_TTL:
+        try:
+            db = get_database()
+            _setup_in_progress = db.is_setup_in_progress()
+        except Exception as e:
+            logger.error(f"Error checking setup progress in auth middleware: {e}")
+            _setup_in_progress = False
+        with _auth_cache_lock:
+            _auth_cache["setup_in_progress"] = _setup_in_progress
+            _auth_cache["setup_in_progress_ts"] = now
+    
+    if _setup_in_progress:
+        if not is_polling_endpoint:
+            logger.debug(f"Setup is in progress, redirecting to setup")
+        if request.path.startswith("/api/"):
+            from flask import jsonify as _jsonify
+            return _jsonify({"error": "Setup in progress", "setup_required": True}), 503
+        return redirect(get_base_url_path() + url_for("common.setup"))
+    
     # Use less verbose logging for polling endpoints
     if is_polling_endpoint:
         # Only log occasionally for polling endpoints to reduce spam
@@ -484,7 +590,7 @@ def authenticate_request():
     else:
         logger.debug(f"No valid session for path '{request.path}', session_id: {session_id}")
     
-    # For API calls, return 401 Unauthorized
+    # For API calls, return 401 Unauthorized as proper JSON
     if request.path.startswith("/api/"):
         # Return 401 with less verbose logging for polling endpoints
         if is_polling_endpoint:
@@ -492,26 +598,13 @@ def authenticate_request():
             pass
         else:
             logger.debug(f"Returning 401 for API path '{request.path}'")
-        return {"error": "Unauthorized"}, 401
+        from flask import jsonify as _jsonify
+        return _jsonify({"error": "Unauthorized"}), 401
     
     # No valid session, redirect to login
     if not is_polling_endpoint:
         logger.debug(f"Redirecting to login for path '{request.path}'")
-    
-    # Get the base URL from settings to ensure proper subpath redirect
-    try:
-        from src.primary.settings_manager import get_setting
-        base_url = get_setting('general', 'base_url', '')
-        if base_url and not base_url.startswith('/'):
-            base_url = f'/{base_url}'
-        if base_url and base_url.endswith('/'):
-            base_url = base_url.rstrip('/')
-        login_url = f"{base_url}/login" if base_url else "/login"
-        logger.debug(f"Redirecting to login with base URL: {login_url}")
-        return redirect(login_url)
-    except Exception as e:
-        logger.warning(f"Error getting base URL for login redirect: {e}")
-        return redirect(url_for("common.login_route"))
+    return redirect(get_base_url_path() + url_for("common.login_route"))
 
 def logout(session_id: str):
     """Log out the current user by invalidating their session"""
@@ -669,9 +762,9 @@ def disable_2fa_with_password_and_otp(username: str, password: str, otp_code: st
             logger.warning(f"Failed to disable 2FA for '{username}': User not found.")
             return False
         
-        # 1. Verify Password (stored in plain text according to memories)
+        # 1. Verify Password using proper hash verification
         stored_password = user_data.get("password", "")
-        if stored_password != password:
+        if not verify_password(stored_password, password):
             logger.warning(f"Failed to disable 2FA for '{username}': Invalid password provided.")
             return False
             
@@ -712,8 +805,9 @@ def change_username(current_username: str, new_username: str, password: str) ->
         logger.warning(f"Username change failed: User '{current_username}' not found in database.")
         return False
     
-    # Verify current password (direct comparison as used in verify_user)
-    if user_data.get("password") != password:
+    # Verify current password using the proper verify_password function
+    stored_password = user_data.get("password") or ""
+    if not verify_password(stored_password, password):
         logger.warning(f"Username change failed for '{current_username}': Invalid password provided.")
         return False
     
@@ -744,12 +838,13 @@ def change_password(current_password: str, new_password: str) -> bool:
         logger.warning(f"Password change failed: User '{username}' not found in database.")
         return False
     
-    # Verify current password (direct comparison as used in verify_user)
-    if user_data.get("password") != current_password:
+    # Verify current password using the proper verify_password function
+    stored_password = user_data.get("password") or ""
+    if not verify_password(stored_password, current_password):
         logger.warning(f"Password change failed for '{username}': Invalid current password provided.")
         return False
     
-    # Update password in database (store new password directly, no hashing)
+    # Update password in database (update_user_password hashes automatically)
     if db.update_user_password(username, new_password):
         logger.info(f"Password changed successfully for user '{username}'.")
         return True
@@ -823,16 +918,15 @@ def create_plex_pin(setup_mode: bool = False, user_mode: bool = False) -> Option
         }
         
         # Create auth URL with redirect URI for main window flow
-        # Determine redirect based on mode
-        base_url = request.host_url.rstrip('/') if request else 'http://localhost:9705'
-        
-        # Determine redirect based on mode
+        host_url = request.host_url.rstrip('/') if request else 'http://localhost:9705'
+        base_path = get_base_url_path()
+
         if setup_mode:
-            redirect_uri = f"{base_url}/setup"
+            redirect_uri = f"{host_url}{base_path}/setup"
         elif user_mode:
-            redirect_uri = f"{base_url}/user"
+            redirect_uri = f"{host_url}{base_path}/user"
         else:
-            redirect_uri = f"{base_url}/"
+            redirect_uri = f"{host_url}{base_path}/"
         
         logger.info(f"Created Plex PIN: {pin_id} (setup_mode: {setup_mode}, user_mode: {user_mode})")
         logger.info(f"Plex redirect_uri set to: {redirect_uri}")
@@ -969,6 +1063,7 @@ def create_user_with_plex(plex_token: str, plex_user_data: Dict[str, Any]) -> bo
     try:
         if save_user_data(user_data):
             logger.info(f"Plex user created: {plex_user_data.get('username')}")
+            invalidate_auth_cache()
             return True
         else:
             logger.error("Failed to save Plex user data")
@@ -1056,7 +1151,7 @@ def unlink_plex_from_user(username: str = None) -> bool:
             
         # Check if Plex data exists to unlink
         if not user_data.get('plex_token'):
-            logger.warning("No Plex account linked to unlink")
+            logger.debug("No Plex account linked to unlink - operation successful (no-op)")
             return True  # Not an error, just nothing to do
             
         # If auth_type is plex, we need to handle this carefully
