@@ -24,7 +24,9 @@ history_locks = {
     "readarr": threading.Lock(),
     "whisparr": threading.Lock(),
     "eros": threading.Lock(),
-    "swaparr": threading.Lock()
+    "swaparr": threading.Lock(),
+    "movie_hunt": threading.Lock(),
+    "tv_hunt": threading.Lock()
 }
 
 def add_history_entry(app_type, entry_data):
@@ -42,10 +44,11 @@ def add_history_entry(app_type, entry_data):
         logger.error(f"Invalid app type: {app_type}")
         return None
     
-    # Extract instance name from entry data
+    # Extract instance key (for DB) and optional display name (for logs)
     instance_name = entry_data.get("instance_name", "Default")
-    
-    logger.debug(f"Adding history entry for {app_type} with instance_name: '{instance_name}'")
+    instance_display = entry_data.get("instance_display_name") or instance_name
+
+    logger.debug(f"Adding history entry for {app_type} with instance: '{instance_display}'")
     
     # Thread-safe database operation
     with history_locks[app_type]:
@@ -63,7 +66,7 @@ def add_history_entry(app_type, entry_data):
             # Add additional fields for compatibility
             entry["app_type"] = app_type  # Include app_type in the entry for display in UI
             
-            logger.info(f"Added history entry for {app_type}-{instance_name}: {entry_data['name']}")
+            logger.info(f"Added history entry for {app_type}-{instance_display}: {entry_data['name']}")
             
             # Send notification about this history entry
             try:
@@ -79,7 +82,7 @@ def add_history_entry(app_type, entry_data):
             logger.error(f"Database error adding history entry for {app_type}: {e}")
             return None
 
-def get_history(app_type, search_query=None, page=1, page_size=20):
+def get_history(app_type, search_query=None, page=1, page_size=20, instance_name=None):
     """
     Get history entries for an app
     
@@ -88,6 +91,7 @@ def get_history(app_type, search_query=None, page=1, page_size=20):
     - search_query: str - Optional search query to filter results
     - page: int - Page number (1-based)
     - page_size: int - Number of entries per page
+    - instance_name: str - Optional instance name to filter (e.g. instance_id as string for movie_hunt/tv_hunt)
     
     Returns:
     - dict with entries, total_entries, and total_pages
@@ -102,12 +106,25 @@ def get_history(app_type, search_query=None, page=1, page_size=20):
             app_type=app_type,
             search_query=search_query,
             page=page,
-            page_size=page_size
+            page_size=page_size,
+            instance_name=instance_name
         )
-        
+        # Convert date_time to user timezone for display (prefer database so in-app choice wins in Docker)
+        try:
+            from src.primary.utils.timezone_utils import get_user_timezone
+            import pytz
+            user_tz = get_user_timezone(prefer_database_for_display=True)
+            for entry in result.get("entries", []):
+                ts = entry.get("date_time")
+                if ts is not None:
+                    utc_dt = datetime.fromtimestamp(ts, tz=pytz.UTC)
+                    local_dt = utc_dt.astimezone(user_tz)
+                    entry["date_time_readable"] = local_dt.strftime("%Y-%m-%d %H:%M:%S")
+        except Exception as tz_err:
+            logger.debug(f"Could not convert history timestamps to user timezone: {tz_err}")
         logger.debug(f"Retrieved {len(result['entries'])} history entries for {app_type} (page {page})")
         return result
-        
+
     except Exception as e:
         logger.error(f"Database error getting history for {app_type}: {e}")
         return {"entries": [], "total_entries": 0, "total_pages": 0, "current_page": 1}
