@@ -6,10 +6,13 @@ Common routes blueprint for Huntarr web interface
 import os
 import json
 import base64
+import secrets
+import string
 import io
 import qrcode
 import pyotp
 import logging
+import requests
 # Add render_template, send_from_directory, session
 from flask import Blueprint, request, jsonify, make_response, redirect, url_for, current_app, render_template, send_from_directory, session, send_file
 from ..auth import (
@@ -18,13 +21,26 @@ from ..auth import (
     update_session_username,
     validate_password_strength, logout, verify_session, disable_2fa_with_password_and_otp,
     user_exists, create_user, generate_2fa_secret, verify_2fa_code, is_2fa_enabled, # Add missing auth imports
-    hash_password # Add hash_password import for recovery key reset
+    hash_password, # Add hash_password import for recovery key reset
+    get_base_url_path, # For cookie path matching base URL
 )
-from ..utils.logger import logger # Ensure logger is imported
+from ..utils.logger import logger
 from .. import settings_manager # Import settings_manager
+from ..utils.tmdb_cache import tmdb_cache # Import TMDB cache
 from datetime import datetime
 
 
+def _cookie_path():
+    """Return the cookie path that matches the configured base URL.
+    
+    When a base URL like '/huntarr' is set, cookies must use that path
+    so the browser sends them for requests under that prefix.
+    Falls back to '/' when no base URL is configured.
+    """
+    base = get_base_url_path()
+    return base if base else '/'
+
+
 common_bp = Blueprint('common', __name__)
 
 # --- Static File Serving --- #
@@ -42,6 +58,49 @@ def logo_files(filename):
     logo_dir = os.path.join(current_app.static_folder, 'logo')
     return send_from_directory(logo_dir, filename)
 
+@common_bp.route('/api/tmdb/image', methods=['GET'])
+def tmdb_image_proxy():
+    """
+    Proxy/cache TMDB images server-side
+    Query params: url (TMDB image URL), cache_days (optional)
+    """
+    try:
+        image_url = request.args.get('url')
+        if not image_url:
+            return jsonify({'error': 'Missing url parameter'}), 400
+        
+        # Security: only allow TMDB image URLs to prevent open redirect / SSRF
+        ALLOWED_PREFIXES = ('https://image.tmdb.org/', 'http://image.tmdb.org/')
+        if not image_url.startswith(ALLOWED_PREFIXES):
+            logger.warning(f"[TMDBCache] Blocked non-TMDB URL: {image_url}")
+            return jsonify({'error': 'Only TMDB image URLs are allowed'}), 400
+        
+        # Get cache settings
+        general_settings = settings_manager.load_settings('general')
+        cache_days = int(general_settings.get('tmdb_image_cache_days', 7))
+        
+        # If caching is disabled, just redirect to TMDB
+        if cache_days == 0:
+            return redirect(image_url)
+        
+        # Check if image is cached and valid
+        if tmdb_cache.is_cached(image_url, max_age_days=cache_days):
+            cached_path = tmdb_cache.get_cached_path(image_url)
+            if cached_path and os.path.exists(cached_path):
+                return send_file(cached_path, mimetype='image/jpeg')
+        
+        # Cache the image
+        cached_path = tmdb_cache.cache_image(image_url)
+        if cached_path and os.path.exists(cached_path):
+            return send_file(cached_path, mimetype='image/jpeg')
+        
+        # Fallback to redirect if caching fails
+        return redirect(image_url)
+    
+    except Exception as e:
+        logger.error(f"Error in TMDB image proxy: {e}")
+        return jsonify({'error': 'Image proxy error'}), 500
+
 # --- API Routes --- #
 
 @common_bp.route('/health', methods=['GET'])
@@ -55,7 +114,7 @@ def health_check():
         try:
             import main
             is_shutting_down = main.is_shutting_down()
-        except:
+        except Exception:
             is_shutting_down = stop_event.is_set()
         
         if is_shutting_down:
@@ -77,7 +136,7 @@ def health_check():
             "status": "healthy",
             "message": "Application is running normally",
             "ready": True,
-            "timestamp": datetime.utcnow().isoformat()
+            "timestamp": datetime.now().isoformat()
         }), 200
         
     except Exception as e:
@@ -99,7 +158,7 @@ def readiness_check():
         try:
             import main
             is_shutting_down = main.is_shutting_down()
-        except:
+        except Exception:
             is_shutting_down = stop_event.is_set()
         
         if is_shutting_down:
@@ -138,6 +197,7 @@ def readiness_check():
             "message": f"Readiness check failed: {str(e)}"
         }), 503
 
+
 @common_bp.route('/api/sleep.json', methods=['GET'])
 def api_get_sleep_json():
     """API endpoint to serve sleep/cycle data from the database for frontend access"""
@@ -168,12 +228,57 @@ def api_get_sleep_json():
         response.headers.add('Access-Control-Allow-Origin', '*')
         return response, 200
 
+# --- Login Rate Limiting --- #
+import threading as _login_threading
+import time as _login_time
+
+_login_attempts = {}  # {ip: [timestamp, timestamp, ...]}
+_login_attempts_lock = _login_threading.Lock()
+_LOGIN_MAX_ATTEMPTS = 5       # max failed attempts before lockout
+_LOGIN_LOCKOUT_SECONDS = 300  # 5-minute lockout window
+_LOGIN_WINDOW_SECONDS = 300   # track attempts within this window
+
+def _check_login_rate_limit(ip):
+    """Check if an IP is rate-limited. Returns (allowed, retry_after_seconds)."""
+    now = _login_time.time()
+    with _login_attempts_lock:
+        if ip in _login_attempts:
+            # Prune old attempts outside the window
+            _login_attempts[ip] = [t for t in _login_attempts[ip] if now - t < _LOGIN_WINDOW_SECONDS]
+            if len(_login_attempts[ip]) >= _LOGIN_MAX_ATTEMPTS:
+                oldest = _login_attempts[ip][0]
+                retry_after = int(_LOGIN_LOCKOUT_SECONDS - (now - oldest))
+                return False, max(retry_after, 1)
+    return True, 0
+
+def _record_failed_login(ip):
+    """Record a failed login attempt from this IP."""
+    now = _login_time.time()
+    with _login_attempts_lock:
+        if ip not in _login_attempts:
+            _login_attempts[ip] = []
+        _login_attempts[ip].append(now)
+
+def _clear_login_attempts(ip):
+    """Clear failed attempts after a successful login."""
+    with _login_attempts_lock:
+        _login_attempts.pop(ip, None)
+
 # --- Authentication Routes --- #
 
 @common_bp.route('/login', methods=['GET', 'POST'])
 def login_route():
     if request.method == 'POST':
         try: # Wrap the POST logic in a try block for better error handling
+            # Rate limit check
+            client_ip = request.headers.get('X-Forwarded-For', request.remote_addr or '127.0.0.1').split(',')[0].strip()
+            allowed, retry_after = _check_login_rate_limit(client_ip)
+            if not allowed:
+                logger.warning(f"Login rate-limited for IP {client_ip}. Retry after {retry_after}s.")
+                response = jsonify({"success": False, "error": f"Too many login attempts. Try again in {retry_after} seconds."})
+                response.headers['Retry-After'] = str(retry_after)
+                return response, 429
+
             data = request.json
             username = data.get('username')
             password = data.get('password')
@@ -190,10 +295,13 @@ def login_route():
 
             if auth_success:
                 # User is authenticated (password correct, and 2FA if needed was correct)
+                _clear_login_attempts(client_ip)
                 session_token = create_session(username)
                 session[SESSION_COOKIE_NAME] = session_token # Store token in Flask session immediately
-                response = jsonify({"success": True, "redirect": "./"}) # Add redirect URL
-                response.set_cookie(SESSION_COOKIE_NAME, session_token, httponly=True, samesite='Lax', path='/') # Add path
+                redirect_url = (_cookie_path().rstrip('/') or '') + '/'
+                response = jsonify({"success": True, "redirect": redirect_url})
+                is_https = request.headers.get('X-Forwarded-Proto') == 'https' or request.is_secure
+                response.set_cookie(SESSION_COOKIE_NAME, session_token, httponly=True, samesite='Lax', path=_cookie_path(), secure=is_https)
                 logger.debug(f"User '{username}' logged in successfully.")
                 return response
             elif needs_2fa:
@@ -214,6 +322,7 @@ def login_route():
             else:
                 # Authentication failed for other reasons (e.g., wrong password, user not found)
                 # Specific reason logged in verify_user
+                _record_failed_login(client_ip)
                 logger.warning(f"Login failed for '{username}': Invalid credentials or other error.")
                 return jsonify({"success": False, "error": "Invalid username or password"}), 401 # Use 401
 
@@ -222,49 +331,30 @@ def login_route():
             return jsonify({"success": False, "error": "An internal server error occurred during login."}), 500
     else:
         # GET request - show login page
-        # If user doesn't exist or setup is in progress, redirect to setup
         if not user_exists():
              logger.info("No user exists, redirecting to setup.")
-             
-             # Get the base URL from settings to ensure proper subpath redirect
-             try:
-                 from src.primary.settings_manager import get_setting
-                 base_url = get_setting('general', 'base_url', '')
-                 if base_url and not base_url.startswith('/'):
-                     base_url = f'/{base_url}'
-                 if base_url and base_url.endswith('/'):
-                     base_url = base_url.rstrip('/')
-                 setup_url = f"{base_url}/setup" if base_url else "/setup"
-                 logger.debug(f"Redirecting to setup with base URL: {setup_url}")
-                 return redirect(setup_url)
-             except Exception as e:
-                 logger.warning(f"Error getting base URL for setup redirect: {e}")
-                 return redirect(url_for('common.setup'))
+             from src.primary import settings_manager
+             base_url = settings_manager.get_setting('general', 'base_url', '').strip()
+             if base_url and base_url != '/':
+                 base_url = '/' + base_url.strip('/')
+             else:
+                 base_url = ''
+             return redirect(base_url + url_for('common.setup'))
         
-        # Check if setup is in progress even if user exists
         try:
             from src.primary.utils.database import get_database
             db = get_database()
             if db.is_setup_in_progress():
                 logger.info("Setup is in progress, redirecting to setup.")
-                
-                # Get the base URL from settings to ensure proper subpath redirect
-                try:
-                    from src.primary.settings_manager import get_setting
-                    base_url = get_setting('general', 'base_url', '')
-                    if base_url and not base_url.startswith('/'):
-                        base_url = f'/{base_url}'
-                    if base_url and base_url.endswith('/'):
-                        base_url = base_url.rstrip('/')
-                    setup_url = f"{base_url}/setup" if base_url else "/setup"
-                    logger.debug(f"Redirecting to setup (in progress) with base URL: {setup_url}")
-                    return redirect(setup_url)
-                except Exception as e:
-                    logger.warning(f"Error getting base URL for setup redirect: {e}")
-                    return redirect(url_for('common.setup'))
+                from src.primary import settings_manager
+                base_url = settings_manager.get_setting('general', 'base_url', '').strip()
+                if base_url and base_url != '/':
+                    base_url = '/' + base_url.strip('/')
+                else:
+                    base_url = ''
+                return redirect(base_url + url_for('common.setup'))
         except Exception as e:
             logger.error(f"Error checking setup progress in login route: {e}")
-            # Continue to show login page if we can't check setup progress
         
         # Check if any users have Plex authentication configured
         try:
@@ -290,7 +380,7 @@ def logout_route():
 
         response = jsonify({"success": True})
         # Ensure cookie deletion happens even if logout function had issues
-        response.delete_cookie(SESSION_COOKIE_NAME, path='/', samesite='Lax') # Specify path and samesite
+        response.delete_cookie(SESSION_COOKIE_NAME, path=_cookie_path(), samesite='Lax') # Match base URL path
         logger.info("Logout successful, cookie deleted.")
         return response
     except Exception as e:
@@ -317,7 +407,13 @@ def setup():
             # If user exists but setup is in progress, allow continuation
             if user_exists() and not db.is_setup_in_progress():
                 logger.info("User exists and setup is complete, redirecting to login")
-                return redirect(url_for('common.login_route'))
+                from src.primary import settings_manager
+                base_url = settings_manager.get_setting('general', 'base_url', '').strip()
+                if base_url and base_url != '/':
+                    base_url = '/' + base_url.strip('/')
+                else:
+                    base_url = ''
+                return redirect(base_url + url_for('common.login_route'))
             
             # Render setup page with progress data
             return render_template('setup.html', setup_progress=setup_progress)
@@ -405,7 +501,8 @@ def setup():
                 session[SESSION_COOKIE_NAME] = session_token # Store token in session
                 response = jsonify({"success": True})
                 # Set cookie in the response
-                response.set_cookie(SESSION_COOKIE_NAME, session_token, httponly=True, samesite='Lax', path='/') # Add path
+                is_https = request.headers.get('X-Forwarded-Proto') == 'https' or request.is_secure
+                response.set_cookie(SESSION_COOKIE_NAME, session_token, httponly=True, samesite='Lax', path=_cookie_path(), secure=is_https)
                 return response
             else:
                 # create_user itself failed, but didn't raise an exception
@@ -461,7 +558,6 @@ def get_user_info_route():
 
     # Pass username to is_2fa_enabled
     two_fa_status = is_2fa_enabled(username) # This function should now be defined via import
-    logger.debug(f"Retrieved user info for '{username}'. 2FA enabled: {two_fa_status}")
     return jsonify({"username": username, "is_2fa_enabled": two_fa_status})
 
 @common_bp.route('/api/user/change-username', methods=['POST'])
@@ -691,7 +787,8 @@ def generate_recovery_key():
                 return jsonify({"success": False, "error": "Current password is required"}), 400
 
             # Verify current password
-            if not verify_user(username, current_password):
+            auth_success, _ = verify_user(username, current_password)
+            if not auth_success:
                 logger.warning(f"Recovery key generation for '{username}' failed: Invalid password.")
                 return jsonify({"success": False, "error": "Invalid current password"}), 400
 
@@ -949,7 +1046,22 @@ def get_stats_api():
         return jsonify({"success": True, "stats": stats})
     except Exception as e:
         logger.error(f"Error retrieving stats: {e}", exc_info=True)
-        return jsonify({"success": False, "error": str(e)}), 500
+        return jsonify({"success": False, "error": "Failed to retrieve stats"}), 500
+
+def _allow_stats_reset():
+    """Return True if request is allowed to reset stats (session, bypass, or No Login Mode)."""
+    username = get_user_for_request()
+    if username:
+        return True
+    try:
+        from src.primary.settings_manager import load_settings
+        settings = load_settings("general")
+        if settings.get("proxy_auth_bypass") or settings.get("local_access_bypass"):
+            return True
+    except Exception as e:
+        logger.error(f"Error checking bypass for stats reset: {e}")
+    return False
+
 
 @common_bp.route('/api/stats/reset', methods=['POST'])
 def reset_stats_api():
@@ -958,17 +1070,15 @@ def reset_stats_api():
         # Import here to avoid circular imports
         from ..stats_manager import reset_stats
         
-        # Check if authenticated
-        session_token = request.cookies.get(SESSION_COOKIE_NAME)
-        if not verify_session(session_token):
-            logger.warning("Stats reset attempt failed: Not authenticated.")
-            return jsonify({"error": "Unauthorized"}), 401
+        if not _allow_stats_reset():
+            logger.warning("Stats reset attempt failed: Not authenticated and not in bypass mode.")
+            return jsonify({"success": False, "error": "Unauthorized"}), 401
             
         # Get app type from request if provided
         data = request.json or {}
         app_type = data.get('app_type')  # None will reset all
         
-        if app_type is not None and app_type not in ["sonarr", "radarr", "lidarr", "readarr", "whisparr"]:
+        if app_type is not None and app_type not in ["sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros"]:
             logger.warning(f"Invalid app_type for stats reset: {app_type}")
             return jsonify({"success": False, "error": "Invalid app_type"}), 400
             
@@ -983,7 +1093,7 @@ def reset_stats_api():
             return jsonify({"success": False, "error": error_msg}), 500
     except Exception as e:
         logger.error(f"Error resetting stats: {e}", exc_info=True)
-        return jsonify({"success": False, "error": str(e)}), 500
+        return jsonify({"success": False, "error": "Failed to reset stats"}), 500
 
 # Ensure all routes previously in this file that interact with settings
 # are either moved to web_server.py or updated here using the new settings_manager functions.
@@ -1182,10 +1292,15 @@ def clear_setup_progress():
     """Clear setup progress (called when setup is complete)"""
     try:
         from src.primary.utils.database import get_database
+        from src.primary.auth import invalidate_auth_cache
         db = get_database()
         
         success = db.clear_setup_progress()
         
+        # Bust the auth middleware cache so it immediately picks up
+        # that setup is no longer in progress (prevents double-login)
+        invalidate_auth_cache()
+        
         return jsonify({
             'success': success,
             'message': 'Setup progress cleared' if success else 'Failed to clear setup progress'
@@ -1218,5 +1333,87 @@ def setup_status():
         return jsonify({"success": False, "error": "Failed to check setup status"}), 500
 
 
+# ── Setup backdrop images (TMDB trending) ─────────────────────────
+_backdrop_cache = {'images': [], 'fetched_at': 0}
+
+@common_bp.route('/api/setup/backdrops', methods=['GET'])
+def setup_backdrops():
+    """Return a list of TMDB trending backdrop image URLs for the setup page."""
+    import time
+    try:
+        now = time.time()
+        # Cache for 1 hour
+        if _backdrop_cache['images'] and (now - _backdrop_cache['fetched_at']) < 3600:
+            return jsonify({'success': True, 'images': _backdrop_cache['images']})
+
+        TMDB_KEY = "9265b0bd0cd1962f7f3225989fcd7192"
+        TMDB_BASE = "https://api.themoviedb.org/3"
+        IMAGE_BASE = "https://image.tmdb.org/t/p/w1280"
+        backdrops = []
+
+        # Fetch trending movies (page 1 & 2 = 40 items)
+        for page in (1, 2):
+            try:
+                r = requests.get(f"{TMDB_BASE}/trending/movie/week", params={
+                    'api_key': TMDB_KEY, 'language': 'en-US', 'page': page
+                }, timeout=8)
+                if r.ok:
+                    for item in r.json().get('results', []):
+                        bp = item.get('backdrop_path')
+                        if bp:
+                            backdrops.append(IMAGE_BASE + bp)
+            except Exception:
+                pass
+
+        # Fetch trending TV (page 1 & 2 = 40 items)
+        for page in (1, 2):
+            try:
+                r = requests.get(f"{TMDB_BASE}/trending/tv/week", params={
+                    'api_key': TMDB_KEY, 'language': 'en-US', 'page': page
+                }, timeout=8)
+                if r.ok:
+                    for item in r.json().get('results', []):
+                        bp = item.get('backdrop_path')
+                        if bp:
+                            backdrops.append(IMAGE_BASE + bp)
+            except Exception:
+                pass
+
+        # Also fetch top-rated from the US for variety
+        try:
+            r = requests.get(f"{TMDB_BASE}/movie/popular", params={
+                'api_key': TMDB_KEY, 'language': 'en-US', 'region': 'US', 'page': 1
+            }, timeout=8)
+            if r.ok:
+                for item in r.json().get('results', []):
+                    bp = item.get('backdrop_path')
+                    if bp:
+                        url = IMAGE_BASE + bp
+                        if url not in backdrops:
+                            backdrops.append(url)
+        except Exception:
+            pass
+
+        # Deduplicate and cap
+        seen = set()
+        unique = []
+        for url in backdrops:
+            if url not in seen:
+                seen.add(url)
+                unique.append(url)
+        backdrops = unique[:80]
+
+        _backdrop_cache['images'] = backdrops
+        _backdrop_cache['fetched_at'] = now
+
+        return jsonify({'success': True, 'images': backdrops})
+    except Exception as e:
+        logger.error(f"Setup backdrops API error: {e}")
+        return jsonify({'success': False, 'images': [], 'error': str(e)})
+
+
+# Register unified calendar routes (Radarr, Sonarr)
+from .calendar_routes import register_calendar_routes
+register_calendar_routes(common_bp)
 
 
