@@ -11,8 +11,14 @@ def get_app_history(app_type):
     """Get history entries for a specific app or all apps"""
     try:
         search_query = request.args.get('search', '')
-        page = int(request.args.get('page', 1))
-        page_size = int(request.args.get('page_size', 20))
+        try:
+            page = max(1, int(request.args.get('page', 1)))
+        except (TypeError, ValueError):
+            page = 1
+        try:
+            page_size = max(1, min(100, int(request.args.get('page_size', 20))))
+        except (TypeError, ValueError):
+            page_size = 20
         
         # Validate page_size to be one of the allowed values
         allowed_page_sizes = [10, 20, 30, 50, 100, 250, 1000]
@@ -20,10 +26,10 @@ def get_app_history(app_type):
             page_size = 20
         
         # Validate app_type
-        valid_app_types = ["all", "sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros"]
+        valid_app_types = ["all", "sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros", "movie_hunt", "tv_hunt"]
         if app_type not in valid_app_types:
             return jsonify({"error": f"Invalid app type: {app_type}"}), 400
-        
+
         result = get_history(app_type, search_query, page, page_size)
         return jsonify(result), 200
     
@@ -36,10 +42,10 @@ def clear_app_history(app_type):
     """Clear history for a specific app or all apps"""
     try:
         # Validate app_type
-        valid_app_types = ["all", "sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros"]
+        valid_app_types = ["all", "sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros", "movie_hunt", "tv_hunt"]
         if app_type not in valid_app_types:
             return jsonify({"error": f"Invalid app type: {app_type}"}), 400
-        
+
         success = clear_history(app_type)
         if success:
             return jsonify({"message": f"History cleared for {app_type}"}), 200
