@@ -9,7 +9,8 @@ from src.primary.auth import (
     create_plex_pin, check_plex_pin, verify_plex_token, create_user_with_plex,
     link_plex_account, verify_plex_user, create_session, user_exists,
     SESSION_COOKIE_NAME, verify_user, unlink_plex_from_user, get_client_identifier,
-    verify_session, get_username_from_session, link_plex_account_session_auth
+    verify_session, get_username_from_session, link_plex_account_session_auth,
+    get_base_url_path,
 )
 from src.primary.utils.logger import logger
 import time
@@ -93,7 +94,7 @@ def check_pin(pin_id):
 def handle_oauth():
     """Handle Plex OAuth callback with authorization code"""
     try:
-        data = request.get_json()
+        data = request.get_json() or {}
         auth_code = data.get('code')
         state = data.get('state')  # This should contain our PIN ID
         
@@ -151,7 +152,7 @@ def handle_oauth():
 def handle_oauth_callback():
     """Handle Plex OAuth callback for login flow"""
     try:
-        data = request.get_json()
+        data = request.get_json() or {}
         auth_code = data.get('code')
         state = data.get('state')
         
@@ -244,15 +245,19 @@ def plex_login():
                 # Create session
                 session_id = create_session(plex_user_data.get('username'))
                 
+                cookie_path = get_base_url_path() or '/'
+                is_https = request.headers.get('X-Forwarded-Proto') == 'https' or request.is_secure
+                redirect_url = (get_base_url_path() or '') + '/'
                 response = jsonify({
                     'success': True,
                     'message': 'Plex user created and logged in successfully',
                     'auth_type': 'plex',
-                    'redirect': './'
+                    'redirect': redirect_url
                 })
                 session[SESSION_COOKIE_NAME] = session_id  # Store in Flask session
                 response.set_cookie(SESSION_COOKIE_NAME, session_id, 
-                                  max_age=60*60*24*7, httponly=True, secure=False)
+                                  max_age=60*60*24*7, httponly=True, samesite='Lax',
+                                  path=cookie_path, secure=is_https)
                 return response
             else:
                 return jsonify({
@@ -276,15 +281,19 @@ def plex_login():
                     username = user_data['username']
                     session_id = create_session(username)
                     
+                    cookie_path = get_base_url_path() or '/'
+                    is_https = request.headers.get('X-Forwarded-Proto') == 'https' or request.is_secure
+                    redirect_url = (get_base_url_path() or '') + '/'
                     response = jsonify({
                         'success': True,
                         'message': 'Logged in with Plex successfully',
                         'auth_type': 'plex',
-                        'redirect': './'
+                        'redirect': redirect_url
                     })
                     session[SESSION_COOKIE_NAME] = session_id  # Store in Flask session
                     response.set_cookie(SESSION_COOKIE_NAME, session_id, 
-                                      max_age=60*60*24*7, httponly=True, secure=False)
+                                      max_age=60*60*24*7, httponly=True, samesite='Lax',
+                                      path=cookie_path, secure=is_https)
                     return response
                 else:
                     return jsonify({
@@ -395,7 +404,8 @@ def link_account():
                     'success': True,
                     'message': 'Plex account linked successfully'
                 })
-                response.set_cookie(SESSION_COOKIE_NAME, session_id, httponly=True, samesite='Lax', path='/')
+                cookie_path = get_base_url_path() or '/'
+                response.set_cookie(SESSION_COOKIE_NAME, session_id, httponly=True, samesite='Lax', path=cookie_path)
                 return response
             else:
                 return jsonify({
@@ -422,7 +432,13 @@ def unlink_plex_account():
         # Check if user is authenticated via session
         session_id = request.cookies.get(SESSION_COOKIE_NAME)
         if not session_id or not verify_session(session_id):
-            return jsonify({'success': False, 'error': 'User not authenticated'}), 401
+            # Return a helpful message explaining what to do
+            logger.warning("Unlink attempt failed: User session expired or invalid")
+            return jsonify({
+                'success': False, 
+                'error': 'Session expired. Please log out and log back in with your username and password, then try unlinking again.',
+                'session_expired': True
+            }), 401
         
         # Since user is authenticated, we can directly unlink without username validation
         if unlink_plex_from_user():
@@ -491,6 +507,5 @@ def plex_status():
 @plex_auth_bp.route('/auth/plex/callback')
 def plex_callback():
     """Handle Plex authentication callback (redirect back to app)"""
-    # Redirect to main page with user hash to avoid index.html redirect conflicts
-    # This ensures proper navigation without triggering localStorage redirects
-    return redirect('/#user')
+    from src.primary.web_server import get_base_url
+    return redirect(get_base_url() + url_for('home') + '#user')
