@@ -7,11 +7,15 @@ Supports default configurations for different Arr applications
 
 import os
 import json
-import pathlib
 import logging
+import hashlib
 import time
+import threading
 from typing import Dict, Any, Optional, List
 
+# SHA-256 hash of the valid Huntarr dev key (constant only; plaintext never stored)
+_DEV_KEY_HASH = "81fc4fcfa6ec4a7a19c9aafe60eea0022ef2c5d05f78484b77cf6578d983f6d3"
+
 # Create a simple logger for settings_manager
 logging.basicConfig(level=logging.INFO)
 settings_logger = logging.getLogger("settings_manager")
@@ -19,43 +23,36 @@ settings_logger = logging.getLogger("settings_manager")
 # Database integration
 from src.primary.utils.database import get_database
 
-# Default configs location
-DEFAULT_CONFIGS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), 'default_configs'))
-
 # Known app types
-KNOWN_APP_TYPES = ["sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros", "swaparr", "prowlarr", "general"]
+KNOWN_APP_TYPES = ["sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros", "swaparr", "prowlarr", "movie_hunt", "tv_hunt", "general"]
 
-# Add a settings cache with timestamps to avoid excessive database reads
-settings_cache = {}  # Format: {app_name: {'timestamp': timestamp, 'data': settings_dict}}
-CACHE_TTL = 5  # Cache time-to-live in seconds
+# Thread-safe settings cache with timestamps to avoid excessive database reads
+_settings_cache = {}  # Format: {app_name: {'timestamp': timestamp, 'data': settings_dict}}
+_cache_lock = threading.Lock()
+CACHE_TTL = 30  # Cache time-to-live in seconds (settings rarely change mid-cycle)
+
+# Legacy alias for any code referencing the old name directly
+settings_cache = _settings_cache
 
 def clear_cache(app_name=None):
     """Clear the settings cache for a specific app or all apps."""
-    global settings_cache
-    if app_name:
-        if app_name in settings_cache:
+    global _settings_cache, settings_cache
+    with _cache_lock:
+        if app_name:
+            _settings_cache.pop(app_name, None)
             settings_logger.debug(f"Clearing cache for {app_name}")
-            settings_cache.pop(app_name, None)
-    else:
-        settings_logger.debug("Clearing entire settings cache")
-        settings_cache = {}
-
-def get_default_config_path(app_name: str) -> pathlib.Path:
-    """Get the path to the default config file for a specific app."""
-    return pathlib.Path(DEFAULT_CONFIGS_DIR) / f"{app_name}.json"
+        else:
+            _settings_cache.clear()
+            settings_logger.debug("Clearing entire settings cache")
+        settings_cache = _settings_cache
 
 def load_default_app_settings(app_name: str) -> Dict[str, Any]:
-    """Load default settings for a specific app from its JSON file."""
-    default_file = get_default_config_path(app_name)
-    if default_file.exists():
-        try:
-            with open(default_file, 'r') as f:
-                return json.load(f)
-        except Exception as e:
-            settings_logger.error(f"Error loading default settings for {app_name} from {default_file}: {e}")
-            return {}
-    else:
-        settings_logger.warning(f"Default settings file not found for {app_name}: {default_file}")
+    """Load default settings for a specific app from default_settings module."""
+    try:
+        from src.primary.default_settings import get_default_config
+        return get_default_config(app_name)
+    except Exception as e:
+        settings_logger.error(f"Failed to load default settings for {app_name}: {e}")
         return {}
 
 def _ensure_config_exists(app_name: str) -> None:
@@ -64,6 +61,14 @@ def _ensure_config_exists(app_name: str) -> None:
         db = get_database()
         
         if app_name == 'general':
+            # Preserve config on upgrade: migrate from app_configs if general_settings empty (issue #802, #815)
+            try:
+                migrate_func = getattr(db, '_migrate_general_settings_from_app_configs_if_needed', None)
+                if migrate_func and callable(migrate_func):
+                    migrate_func()
+            except Exception as e:
+                settings_logger.warning(f"Failed to migrate general settings (will use defaults): {e}")
+            
             # Check if general settings exist
             existing_settings = db.get_general_settings()
             if not existing_settings:
@@ -102,22 +107,18 @@ def load_settings(app_type, use_cache=True):
     Returns:
         Dict containing the app settings
     """
-    global settings_cache
-    
     # Only log unexpected app types that are not 'general'
     if app_type not in KNOWN_APP_TYPES and app_type != "general":
         settings_logger.warning(f"load_settings called with unexpected app_type: {app_type}")
     
-    # Check if we have a valid cache entry
-    if use_cache and app_type in settings_cache:
-        cache_entry = settings_cache[app_type]
-        cache_age = time.time() - cache_entry.get('timestamp', 0)
-        
-        if cache_age < CACHE_TTL:
-            settings_logger.debug(f"Using cached settings for {app_type} (age: {cache_age:.1f}s)")
-            return cache_entry['data']
-        else:
-            settings_logger.debug(f"Cache expired for {app_type} (age: {cache_age:.1f}s)")
+    # Check if we have a valid cache entry (thread-safe read)
+    if use_cache:
+        with _cache_lock:
+            cache_entry = _settings_cache.get(app_type)
+            if cache_entry:
+                cache_age = time.time() - cache_entry.get('timestamp', 0)
+                if cache_age < CACHE_TTL:
+                    return cache_entry['data']
     
     # No valid cache entry, load from database
     current_settings = {}
@@ -166,12 +167,17 @@ def load_settings(app_type, use_cache=True):
         settings_logger.info(f"Added missing default keys to {app_type} settings")
         save_settings(app_type, current_settings)
     
-    # Update cache
-    settings_cache[app_type] = {
-        'timestamp': time.time(),
-        'data': current_settings
-    }
-        
+    # Update cache (thread-safe write)
+    with _cache_lock:
+        _settings_cache[app_type] = {
+            'timestamp': time.time(),
+            'data': current_settings
+        }
+
+    # Update checking is always enabled; remove the toggle but enforce the behavior
+    if app_type == 'general':
+        current_settings['check_for_updates'] = True
+
     return current_settings
 
 def save_settings(app_name: str, settings_data: Dict[str, Any]) -> bool:
@@ -198,12 +204,18 @@ def save_settings(app_name: str, settings_data: Dict[str, Any]) -> bool:
         'hunt_missing_movies', 'hunt_upgrade_movies', 'hunt_missing_books', 'hunt_upgrade_books'
     ]
     
-    # Special validation for sleep_duration (minimum 600 seconds = 10 minutes)
+    # Sleep duration: min 600s (10 min) normally; min 60s (1 min) when dev mode (allows 1-min, does not force it)
+    # When saving general, use incoming data to avoid load_settings -> save_settings -> is_dev_mode -> load_settings recursion
+    if app_name == 'general':
+        _dev = _is_dev_mode_from_general(settings_data)
+    else:
+        _dev = is_dev_mode()
+    _sleep_min = 60 if _dev else 600
     if 'sleep_duration' in settings_data:
         original_value = settings_data['sleep_duration']
-        if isinstance(original_value, (int, float)) and original_value < 600:
-            settings_data['sleep_duration'] = 600
-            settings_logger.warning(f"Sleep duration for {app_name} was {original_value} seconds, automatically set to minimum allowed value of 600 seconds (10 minutes)")
+        if isinstance(original_value, (int, float)) and original_value < _sleep_min:
+            settings_data['sleep_duration'] = _sleep_min
+            settings_logger.warning(f"Sleep duration for {app_name} was {original_value}s, set to minimum {_sleep_min}s" + (" (dev mode)" if _dev else " (10 minutes)"))
     
     for field in numeric_fields:
         if field in settings_data:
@@ -216,12 +228,17 @@ def save_settings(app_name: str, settings_data: Dict[str, Any]) -> bool:
     if 'instances' in settings_data and isinstance(settings_data['instances'], list):
         for i, instance in enumerate(settings_data['instances']):
             if isinstance(instance, dict):
-                # Special validation for sleep_duration in instances
                 if 'sleep_duration' in instance:
                     original_value = instance['sleep_duration']
-                    if isinstance(original_value, (int, float)) and original_value < 600:
-                        instance['sleep_duration'] = 600
-                        settings_logger.warning(f"Sleep duration for {app_name} instance {i+1} was {original_value} seconds, automatically set to minimum allowed value of 600 seconds (10 minutes)")
+                    if isinstance(original_value, (int, float)) and original_value < _sleep_min:
+                        instance['sleep_duration'] = _sleep_min
+                        settings_logger.warning(f"Sleep duration for {app_name} instance {i+1} was {original_value}s, set to minimum {_sleep_min}s" + (" (dev mode)" if _dev else " (10 minutes)"))
+                # Enforce hourly_cap max 400 per instance
+                if 'hourly_cap' in instance:
+                    original_cap = instance['hourly_cap']
+                    if isinstance(original_cap, (int, float)) and original_cap > 400:
+                        instance['hourly_cap'] = 400
+                        settings_logger.warning(f"Hourly cap for {app_name} instance {i+1} was {original_cap}, automatically reduced to maximum 400")
                 
                 for field in numeric_fields:
                     if field in instance:
@@ -230,6 +247,38 @@ def save_settings(app_name: str, settings_data: Dict[str, Any]) -> bool:
                             instance[field] = 0
                             settings_logger.warning(f"{field} for {app_name} instance {i+1} was {original_value}, automatically set to minimum allowed value of 0")
     
+    # Ensure every instance has an explicit 'enabled' field so .get("enabled", True) never silently re-enables
+    _apps_with_instances = ("sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros")
+    if app_name in _apps_with_instances and "instances" in settings_data and isinstance(settings_data["instances"], list):
+        for inst in settings_data["instances"]:
+            if isinstance(inst, dict) and "enabled" not in inst:
+                inst["enabled"] = False
+                settings_logger.info(f"Instance '{inst.get('name', 'Unnamed')}' in {app_name} missing 'enabled' field, defaulting to disabled")
+
+    # Ensure every instance has a stable instance_id (so new instances get one on save, not only when they have API key)
+    if app_name in _apps_with_instances and "instances" in settings_data and isinstance(settings_data["instances"], list):
+        try:
+            from src.primary.utils.instance_id import generate_instance_id
+        except ImportError:
+            pass
+        else:
+            existing_ids = set()
+            for inst in settings_data["instances"]:
+                if isinstance(inst, dict) and inst.get("instance_id"):
+                    existing_ids.add((inst.get("instance_id") or "").strip())
+            for inst in settings_data["instances"]:
+                if not isinstance(inst, dict):
+                    continue
+                current_id = (inst.get("instance_id") or "").strip()
+                if not current_id:
+                    new_id = generate_instance_id(app_name, existing_ids)
+                    inst["instance_id"] = new_id
+                    existing_ids.add(new_id)
+                    try:
+                        get_database().migrate_instance_identifier(app_name, inst.get("name") or "Default", new_id)
+                    except Exception as e:
+                        settings_logger.warning(f"Could not migrate instance identifier for {app_name}: {e}")
+    
     try:
         db = get_database()
         
@@ -277,14 +326,34 @@ def get_api_key(app_name: str) -> Optional[str]:
     """Get the API Key for a specific app."""
     return get_setting(app_name, "api_key", "")
 
+
+def _is_dev_mode_from_general(general_dict: Dict[str, Any]) -> bool:
+    """Compute dev mode from a general settings dict. Used to avoid load_settings recursion when saving general."""
+    key = (general_dict.get("dev_key") or "").strip()
+    if not key:
+        return False
+    return hashlib.sha256(key.encode()).hexdigest() == _DEV_KEY_HASH
+
+
+def is_dev_mode() -> bool:
+    """Return True if Huntarr dev mode is enabled (valid dev key saved in general settings)."""
+    general = load_settings("general", use_cache=True)
+    return _is_dev_mode_from_general(general)
+
+
 def get_all_settings() -> Dict[str, Dict[str, Any]]:
     """Load settings for all known apps."""
     all_settings = {}
     for app_name in KNOWN_APP_TYPES:
-        # Only include apps if their config exists or can be created from defaults
         settings = load_settings(app_name)
-        if settings: # Only add if settings were successfully loaded
-             all_settings[app_name] = settings
+        if not settings:
+            continue
+        if app_name == "general":
+            s = dict(settings)
+            s["dev_mode"] = is_dev_mode()
+            all_settings[app_name] = s
+        else:
+            all_settings[app_name] = settings
     return all_settings
 
 def get_configured_apps() -> List[str]:
@@ -293,7 +362,34 @@ def get_configured_apps() -> List[str]:
     for app_name in KNOWN_APP_TYPES:
         if app_name == 'general':
             continue  # Skip general settings
-            
+        if app_name == 'movie_hunt':
+            try:
+                from src.primary.utils.database import get_database
+                from src.primary.routes.media_hunt.instances import _get_movie_hunt_instance_settings
+                db = get_database()
+                instances = db.get_movie_hunt_instances()
+                for inst in instances:
+                    s = _get_movie_hunt_instance_settings(inst['id'])
+                    if s.get('enabled', True):
+                        configured.append('movie_hunt')
+                        break
+            except Exception:
+                pass
+            continue
+        if app_name == 'tv_hunt':
+            try:
+                from src.primary.utils.database import get_database
+                from src.primary.routes.media_hunt.instances import _get_tv_hunt_instance_settings
+                db = get_database()
+                instances = db.get_tv_hunt_instances()
+                for inst in instances:
+                    s = _get_tv_hunt_instance_settings(inst['id'])
+                    if s.get('enabled', True):
+                        configured.append('tv_hunt')
+                        break
+            except Exception:
+                pass
+            continue
         settings = load_settings(app_name)
         
         # First check if there are valid instances configured (multi-instance mode)
@@ -488,50 +584,29 @@ def initialize_timezone_from_env():
         settings_logger.error(f"Error initializing timezone from environment: {e}")
 
 def initialize_base_url_from_env():
-    """Initialize base_url setting from BASE_URL environment variable if not already set."""
+    """Initialize base_url setting from BASE_URL environment variable."""
     try:
-        # Get the BASE_URL environment variable
-        base_url_env = os.environ.get('BASE_URL')
-        if not base_url_env:
-            settings_logger.info("No BASE_URL environment variable found, using default (no subpath)")
+        if 'BASE_URL' not in os.environ:
             return
 
-        # Clean up the environment variable value
-        base_url_env = base_url_env.strip()
-        
-        # Ensure it starts with / if not empty
+        base_url_env = os.environ.get('BASE_URL', '').strip()
+
         if base_url_env and not base_url_env.startswith('/'):
             base_url_env = f'/{base_url_env}'
-        
-        # Remove trailing slash if present (except for root)
+
         if base_url_env and base_url_env != '/' and base_url_env.endswith('/'):
             base_url_env = base_url_env.rstrip('/')
 
-        # Load current general settings
         general_settings = load_settings("general")
-        current_base_url = general_settings.get("base_url", "").strip()
-        
-        # If base_url is not set in settings, initialize it from BASE_URL environment variable
-        if not current_base_url:
-            settings_logger.info(f"Initializing base_url from BASE_URL environment variable: {base_url_env}")
-           
-            # Update the settings with the base_url
-            general_settings["base_url"] = base_url_env
-            save_settings("general", general_settings)
-            
-            # Clear cache to ensure new settings are loaded
-            clear_cache("general")
-            
-            settings_logger.info(f"Successfully initialized base_url to {base_url_env}")
-        else:
-            settings_logger.debug(f"Base URL already configured in settings: {current_base_url}, not overriding with environment variable")
-            
+        general_settings["base_url"] = base_url_env
+        save_settings("general", general_settings)
+        clear_cache("general")
+
     except Exception as e:
         settings_logger.error(f"Error initializing base_url from environment: {e}")
 
 # Add a list of known advanced settings for clarity and documentation
 ADVANCED_SETTINGS = [
-    "api_timeout", 
     "command_wait_delay", 
     "command_wait_attempts", 
     "minimum_download_queue_size",
@@ -539,7 +614,13 @@ ADVANCED_SETTINGS = [
     "stateful_management_hours",
     "hourly_cap",
     "ssl_verify",  # Add SSL verification setting
-    "base_url"     # Add base URL setting
+    "base_url",    # Add base URL setting
+    "log_rotation_enabled",
+    "log_max_size_mb",
+    "log_backup_count",
+    "log_retention_days",
+    "log_auto_cleanup",
+    "enable_debug_logs"
 ]
 
 def get_advanced_setting(setting_name, default_value=None):
@@ -598,8 +679,7 @@ def initialize_database():
     db.ensure_database_exists()
     
     # Initialize database with default configurations
-    defaults_dir = Path(__file__).parent / "default_configs"
-    db.initialize_from_defaults(defaults_dir)
+    db.initialize_from_defaults()
     
     # Start database maintenance scheduler for integrity monitoring
     try:
@@ -640,11 +720,16 @@ def _migrate_instance_state_management_if_needed(app_name: str, new_settings_dat
             current_name = current_instance.get('name', f'Instance {i+1}')
             new_name = new_instance.get('name', f'Instance {i+1}')
             
-            # If name has changed, migrate the state management data
+            # If name has changed, migrate the state management data (only for legacy instances keyed by name)
+            # Once an instance has instance_id, per-instance DB state is keyed by instance_id, not display name,
+            # so migrating by old_name -> new_name would be a no-op and we skip to avoid unnecessary DB work.
             if current_name != new_name and current_name and new_name:
+                if new_instance.get('instance_id') or current_instance.get('instance_id'):
+                    settings_logger.debug(f"Instance {i+1} has instance_id; state is keyed by ID, skipping name-change migration")
+                    continue
                 settings_logger.info(f"Detected instance name change for {app_name} instance {i+1}: '{current_name}' -> '{new_name}'")
                 
-                # Attempt to migrate state management data
+                # Attempt to migrate state management data (legacy instance keyed by display name)
                 migration_success = db.migrate_instance_state_management(app_name, current_name, new_name)
                 
                 if migration_success:
