@@ -14,10 +14,10 @@ from typing import Dict, Any, List, Optional, Set
 stateful_logger = logging.getLogger("stateful_manager")
 
 # Constants
-DEFAULT_HOURS = 168  # Default 7 days (168 hours)
+DEFAULT_HOURS = 72  # Default 3 days (72 hours)
 
 # App types
-APP_TYPES = ["sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros"]
+APP_TYPES = ["sonarr", "radarr", "lidarr", "readarr", "whisparr", "eros", "movie_hunt", "tv_hunt"]
 
 # Import database
 from src.primary.utils.database import get_database
@@ -202,8 +202,8 @@ def add_processed_id(app_type: str, instance_name: str, media_id: str) -> bool:
     
     try:
         # First check if state management is enabled for this instance
-        instance_hours = 168  # Default
-        instance_mode = "custom"
+        # OPTIMIZATION: Only load settings if we need to check mode, not on every call
+        instance_mode = "custom"  # Default to enabled
         
         try:
             from src.primary.settings_manager import load_settings
@@ -214,7 +214,6 @@ def add_processed_id(app_type: str, instance_name: str, media_id: str) -> bool:
                 for instance in settings['instances']:
                     if instance.get('name') == instance_name:
                         instance_mode = instance.get('state_management_mode', 'custom')
-                        instance_hours = instance.get('state_management_hours', 168)
                         
                         # If state management is disabled for this instance, don't add to processed list
                         if instance_mode == 'disabled':
@@ -227,14 +226,6 @@ def add_processed_id(app_type: str, instance_name: str, media_id: str) -> bool:
         
         db = get_database()
         
-        # Initialize per-instance state management if not already done
-        db.initialize_instance_state_management(app_type, instance_name, instance_hours)
-        
-        # Check if this instance's state has expired
-        if db.check_instance_expiration(app_type, instance_name):
-            stateful_logger.info(f"State management expired for {app_type}/{instance_name}, resetting before adding new ID...")
-            db.reset_instance_state_management(app_type, instance_name, instance_hours)
-        
         # Check if already processed
         if db.is_processed(app_type, instance_name, media_id):
             stateful_logger.debug(f"[add_processed_id] ID {media_id} already in database for {app_type}/{instance_name}")
@@ -264,8 +255,8 @@ def is_processed(app_type: str, instance_name: str, media_id: str) -> bool:
     """
     try:
         # First check if state management is enabled for this instance
-        instance_hours = 168  # Default
-        instance_mode = "custom"
+        # OPTIMIZATION: Only load settings if we need to check mode, not on every call
+        instance_mode = "custom"  # Default to enabled
         
         try:
             from src.primary.settings_manager import load_settings
@@ -276,7 +267,6 @@ def is_processed(app_type: str, instance_name: str, media_id: str) -> bool:
                 for instance in settings['instances']:
                     if instance.get('name') == instance_name:
                         instance_mode = instance.get('state_management_mode', 'custom')
-                        instance_hours = instance.get('state_management_hours', 168)
                         
                         # If state management is disabled for this instance, always return False (not processed)
                         if instance_mode == 'disabled':
@@ -289,25 +279,13 @@ def is_processed(app_type: str, instance_name: str, media_id: str) -> bool:
         
         db = get_database()
         
-        # Initialize per-instance state management if not already done
-        db.initialize_instance_state_management(app_type, instance_name, instance_hours)
-        
-        # Check if this instance's state has expired
-        if db.check_instance_expiration(app_type, instance_name):
-            stateful_logger.info(f"State management expired for {app_type}/{instance_name}, resetting...")
-            db.reset_instance_state_management(app_type, instance_name, instance_hours)
-            # After reset, item is not processed
-            return False
-        
         # Converting media_id to string since some callers might pass an integer
         media_id_str = str(media_id)
         is_in_db = db.is_processed(app_type, instance_name, media_id_str)
         
-        # Get total count for logging
-        processed_ids = db.get_processed_ids(app_type, instance_name)
-        total_count = len(processed_ids)
-        
-        stateful_logger.info(f"is_processed check: {app_type}/{instance_name}, ID:{media_id_str}, Found:{is_in_db}, Total IDs:{total_count}")
+        # Only log at debug level — skip the expensive get_processed_ids() call
+        # that was previously fetching ALL IDs just to log the count
+        stateful_logger.debug(f"is_processed check: {app_type}/{instance_name}, ID:{media_id_str}, Found:{is_in_db}")
         
         return is_in_db
     except Exception as e:
