@@ -20,6 +20,20 @@ stateful_logger = get_logger("stateful")
 # Create blueprint
 stateful_api = Blueprint('stateful_api', __name__)
 
+
+def _resolve_instance_id(app_type: str, instance_name: str):
+    """Resolve display name to stable instance_id for DB operations. Returns instance_id or instance_name."""
+    try:
+        app_module = __import__(f"src.primary.apps.{app_type}", fromlist=["get_configured_instances"])
+        get_instances = getattr(app_module, "get_configured_instances", None)
+        if get_instances:
+            for inst in get_instances(quiet=True):
+                if inst.get("instance_name") == instance_name:
+                    return inst.get("instance_id") or instance_name
+    except Exception:
+        pass
+    return instance_name
+
 @stateful_api.route('/info', methods=['GET'])
 def get_info():
     """Get stateful management information."""
@@ -59,20 +73,27 @@ def reset_stateful():
             from src.primary.settings_manager import load_settings
             
             # Get instance settings for expiration hours
-            instance_hours = 168  # Default
+            instance_hours = 72  # Default
             try:
                 settings = load_settings(app_type)
                 if settings and 'instances' in settings:
                     for instance in settings['instances']:
                         if instance.get('name') == instance_name:
-                            instance_hours = instance.get('state_management_hours', 168)
+                            instance_hours = instance.get('state_management_hours', 72)
                             break
             except Exception as e:
-                stateful_logger.warning(f"Could not load instance settings for {app_type}/{instance_name}: {e}")
+                from src.primary.utils.log_deduplication import should_log_message, format_suppressed_message
+                
+                error_msg = f"Could not load instance settings for {app_type}/{instance_name}: {e}"
+                should_log, suppressed_count = should_log_message("stateful", "WARNING", error_msg)
+                if should_log:
+                    formatted_msg = format_suppressed_message(error_msg, suppressed_count)
+                    stateful_logger.warning(formatted_msg)
             
-            # Reset per-instance state management
+            # Reset per-instance state management (use instance_id for DB)
             db = get_database()
-            success = db.reset_instance_state_management(app_type, instance_name, instance_hours)
+            instance_identifier = _resolve_instance_id(app_type, instance_name)
+            success = db.reset_instance_state_management(app_type, instance_identifier, instance_hours)
             
             if success:
                 stateful_logger.info(f"Successfully reset state management for {app_type}/{instance_name}")
@@ -173,7 +194,7 @@ def get_summary():
                 for instance in settings['instances']:
                     if instance.get('name') == instance_name:
                         # Get per-instance state management hours
-                        instance_hours = instance.get('state_management_hours', 168)
+                        instance_hours = instance.get('state_management_hours', 72)
                         instance_mode = instance.get('state_management_mode', 'custom')
                         
                         # If state management is disabled for this instance, return disabled status
@@ -193,18 +214,26 @@ def get_summary():
                         
                         break
         except Exception as e:
-            stateful_logger.warning(f"Could not load instance settings for {app_type}/{instance_name}: {e}")
+            from src.primary.utils.log_deduplication import should_log_message, format_suppressed_message
+            
+            error_msg = f"Could not load instance settings for {app_type}/{instance_name}: {e}"
+            should_log, suppressed_count = should_log_message("stateful", "WARNING", error_msg)
+            if should_log:
+                formatted_msg = format_suppressed_message(error_msg, suppressed_count)
+                stateful_logger.warning(formatted_msg)
+            
             # Fall back to default hours if settings can't be loaded
-            instance_hours = 168
+            instance_hours = 72
         
-        # Get summary for the specific instance with custom hours
-        summary = get_state_management_summary(app_type, instance_name, instance_hours)
+        # Get summary for the specific instance with custom hours (use instance_id for DB lookup)
+        instance_identifier = _resolve_instance_id(app_type, instance_name)
+        summary = get_state_management_summary(app_type, instance_identifier, instance_hours)
         
         response_data = {
             "success": True,
             "processed_count": summary.get("processed_count", 0),
             "next_reset_time": summary.get("next_reset_time"),
-            "expiration_hours": summary.get("expiration_hours", instance_hours or 168),
+            "expiration_hours": summary.get("expiration_hours", instance_hours or 72),
             "has_processed_items": summary.get("has_processed_items", False),
             "state_management_enabled": True
         }
