@@ -2,13 +2,16 @@
 """
 Logging configuration for Huntarr
 Supports separate log files for each application type
+Includes log rotation and cleanup functionality
 """
 
 import logging
+import logging.handlers
 import sys
 import os
 import pathlib
 import time
+import glob
 from typing import Dict, Optional
 
 # Use the centralized path configuration
@@ -20,21 +23,55 @@ from src.primary.utils.config_paths import LOG_DIR
 # Default log file for general messages
 MAIN_LOG_FILE = LOG_DIR / "huntarr.log"
 
-# App-specific log files
+# App-specific log files (movie_hunt is independent - Activity → Logs only, no *arr)
 APP_LOG_FILES = {
-    "sonarr": LOG_DIR / "sonarr.log", # Updated filename
-    "radarr": LOG_DIR / "radarr.log", # Updated filename
-    "lidarr": LOG_DIR / "lidarr.log", # Updated filename
-    "readarr": LOG_DIR / "readarr.log", # Updated filename
-    "whisparr": LOG_DIR / "whisparr.log", # Added Whisparr
-    "eros": LOG_DIR / "eros.log",      # Added Eros for Whisparr V3
-    "swaparr": LOG_DIR / "swaparr.log",  # Added Swaparr for stalled download management
+    "sonarr": LOG_DIR / "sonarr.log",
+    "radarr": LOG_DIR / "radarr.log",
+    "lidarr": LOG_DIR / "lidarr.log",
+    "readarr": LOG_DIR / "readarr.log",
+    "whisparr": LOG_DIR / "whisparr.log",
+    "eros": LOG_DIR / "eros.log",
+    "swaparr": LOG_DIR / "swaparr.log",
+    "movie_hunt": LOG_DIR / "movie_hunt.log",
+    "tv_hunt": LOG_DIR / "tv_hunt.log",
 }
 
 # Global logger instances
 logger: Optional[logging.Logger] = None
 app_loggers: Dict[str, logging.Logger] = {}
 
+def get_rotation_settings():
+    """
+    Get log rotation settings from settings manager.
+    Imports locally to avoid circular dependencies.
+    """
+    def _safe_int(val, default):
+        try:
+            return int(val) if val is not None else default
+        except (TypeError, ValueError):
+            return default
+
+    try:
+        import src.primary.settings_manager as settings_manager
+        settings = settings_manager.load_settings("general")
+        return {
+            "enabled": settings.get("log_rotation_enabled", True),
+            "max_bytes": _safe_int(settings.get("log_max_size_mb"), 50) * 1024 * 1024,
+            "backup_count": _safe_int(settings.get("log_backup_count"), 5),
+            "retention_days": _safe_int(settings.get("log_retention_days"), 30),
+            "auto_cleanup": settings.get("log_auto_cleanup", True)
+        }
+    except Exception as e:
+        print(f"[Logger] Error loading log settings: {e}")
+        # Return defaults if settings can't be loaded
+        return {
+            "enabled": True,
+            "max_bytes": 50 * 1024 * 1024,
+            "backup_count": 5,
+            "retention_days": 30,
+            "auto_cleanup": True
+        }
+
 # Custom formatter that uses user's selected timezone instead of UTC
 class LocalTimeFormatter(logging.Formatter):
     """Custom formatter that uses user's selected timezone for log timestamps"""
@@ -85,6 +122,25 @@ class LocalTimeFormatter(logging.Formatter):
                 
             return s
 
+
+class DebugLogsFilter(logging.Filter):
+    """
+    Filter that suppresses DEBUG records when "Enable Debug Logs" is off in settings.
+    Used on console and file handlers so Docker console and log files don't flood
+    when the user disables debug in the UI (GitHub #756, #813).
+    """
+    def filter(self, record: logging.LogRecord) -> bool:
+        if record.levelno != logging.DEBUG:
+            return True
+        try:
+            from src.primary.settings_manager import get_advanced_setting
+            if get_advanced_setting("enable_debug_logs", True):
+                return True
+            return False
+        except Exception:
+            return True  # Allow DEBUG if setting can't be read (e.g. during early startup)
+
+
 def setup_main_logger():
     """Set up the main Huntarr logger."""
     global logger
@@ -107,10 +163,23 @@ def setup_main_logger():
     # Create console handler
     console_handler = logging.StreamHandler(sys.stdout)
     console_handler.setLevel(use_log_level)
+    console_handler.addFilter(DebugLogsFilter())
 
-    # Create file handler
-    file_handler = logging.FileHandler(log_file)
+    # Get rotation settings
+    rotation_settings = get_rotation_settings()
+
+    # Create file handler (Rotating or Standard)
+    if rotation_settings["enabled"]:
+        file_handler = logging.handlers.RotatingFileHandler(
+            log_file,
+            maxBytes=rotation_settings["max_bytes"],
+            backupCount=rotation_settings["backup_count"]
+        )
+    else:
+        file_handler = logging.FileHandler(log_file)
+        
     file_handler.setLevel(use_log_level)
+    file_handler.addFilter(DebugLogsFilter())
 
     # Set format for the main logger
     log_format = "%(asctime)s - huntarr - %(levelname)s - %(message)s"
@@ -171,11 +240,25 @@ def get_logger(app_type: str) -> logging.Logger:
     # Create console handler
     console_handler = logging.StreamHandler(sys.stdout)
     console_handler.setLevel(logging.DEBUG)
+    console_handler.addFilter(DebugLogsFilter())
+
+    # Get rotation settings
+    rotation_settings = get_rotation_settings()
     
     # Create file handler for the specific app log file
     log_file = APP_LOG_FILES[app_type]
-    file_handler = logging.FileHandler(log_file)
+    
+    if rotation_settings["enabled"]:
+        file_handler = logging.handlers.RotatingFileHandler(
+            log_file,
+            maxBytes=rotation_settings["max_bytes"],
+            backupCount=rotation_settings["backup_count"]
+        )
+    else:
+        file_handler = logging.FileHandler(log_file)
+        
     file_handler.setLevel(logging.DEBUG)
+    file_handler.addFilter(DebugLogsFilter())
     
     # Set a distinct format for this app log
     log_format = f"%(asctime)s - huntarr.{app_type} - %(levelname)s - %(message)s"
@@ -240,6 +323,58 @@ def refresh_timezone_formatters():
     
     print("[Logger] Timezone formatters refreshed for all loggers")
 
+def refresh_log_handlers():
+    """
+    Recreate log handlers with new rotation settings.
+    Should be called when log settings change.
+    """
+    print("[Logger] Refreshing log handlers with new settings")
+    
+    # Re-setup main logger
+    setup_main_logger()
+    
+    # Re-setup all app loggers
+    # We need to clear the cache so get_logger creates new handlers
+    global app_loggers
+    current_apps = list(app_loggers.keys())
+    app_loggers = {}
+    
+    for log_name in current_apps:
+        app_type = log_name.split('.')[-1] if '.' in log_name else log_name
+        get_logger(app_type)
+        
+    print("[Logger] Log handlers refreshed")
+
+def cleanup_old_logs():
+    """Delete log files older than retention days."""
+    settings = get_rotation_settings()
+    if not settings["auto_cleanup"] or settings["retention_days"] <= 0:
+        return
+
+    retention_days = settings["retention_days"]
+    now = time.time()
+    cutoff = now - (retention_days * 86400)
+    
+    print(f"[Logger] Cleaning up logs older than {retention_days} days...")
+    
+    count = 0
+    # Check all files in log directory
+    # Look for rotated files like .log.1, .log.2 etc.
+    for log_file in LOG_DIR.glob("*.log*"): 
+        try:
+            # Skip current active log files (no suffix or just .log)
+            if log_file.suffix == '.log':
+                continue
+                
+            if log_file.stat().st_mtime < cutoff:
+                log_file.unlink()
+                count += 1
+                print(f"[Logger] Deleted old log file: {log_file.name}")
+        except Exception as e:
+            print(f"[Logger] Error deleting {log_file.name}: {e}")
+            
+    print(f"[Logger] Cleanup complete. Removed {count} files.")
+
 def debug_log(message: str, data: object = None, app_type: Optional[str] = None) -> None:
     """
     Log debug messages with optional data.
@@ -260,7 +395,7 @@ def debug_log(message: str, data: object = None, app_type: Optional[str] = None)
                     as_json = as_json[:500] + "..."
                 # Combine message and data in single log entry to prevent fragmentation
                 current_logger.debug(f"{message} | Data: {as_json}")
-            except:
+            except Exception:
                 data_str = str(data)
                 if len(data_str) > 500:
                     data_str = data_str[:500] + "..."
@@ -270,4 +405,10 @@ def debug_log(message: str, data: object = None, app_type: Optional[str] = None)
             current_logger.debug(f"{message}")
 
 # Initialize the main logger instance when the module is imported
-logger = setup_main_logger()
\ No newline at end of file
+logger = setup_main_logger()
+
+# Run cleanup on startup if enabled
+try:
+    cleanup_old_logs()
+except Exception as e:
+    print(f"[Logger] Error running startup cleanup: {e}")
