@@ -8,25 +8,20 @@ import os
 import datetime
 import time
 from threading import Lock
-from primary.utils.logger import LOG_DIR, APP_LOG_FILES, MAIN_LOG_FILE # Import log constants
-from primary import settings_manager # Import settings_manager
-from src.primary.stateful_manager import update_lock_expiration # Import stateful update function
+from primary.utils.logger import LOG_DIR, APP_LOG_FILES, MAIN_LOG_FILE
+from primary import settings_manager
+from src.primary.stateful_manager import update_lock_expiration
 
-# import socket # No longer used
 import json
-# import signal # No longer used for reload
 import sys
 import qrcode
 import pyotp
 import logging
 import threading
-import importlib # Added import
+import importlib
 import requests
-from flask import Flask, render_template, request, jsonify, Response, send_from_directory, redirect, url_for, session, stream_with_context, Blueprint, current_app, g, make_response # Added stream_with_context and Blueprint
-# from src.primary.config import API_URL # No longer needed directly
-# Use only settings_manager
-from src.primary import settings_manager
-from src.primary.utils.logger import setup_main_logger, get_logger, LOG_DIR, update_logging_levels # Import get_logger, LOG_DIR, and update_logging_levels
+from flask import Flask, render_template, request, jsonify, Response, send_from_directory, redirect, url_for, session, stream_with_context, Blueprint, current_app, g, make_response
+from src.primary.utils.logger import setup_main_logger, get_logger, LOG_DIR, update_logging_levels, refresh_log_handlers # Import get_logger, LOG_DIR, update_logging_levels, refresh_log_handlers
 # Clean logging is now database-only
 from src.primary.auth import (
     authenticate_request, user_exists, create_user, verify_user, create_session,
@@ -37,6 +32,7 @@ from src.primary.auth import (
 )
 # Import blueprint for common routes
 from src.primary.routes.common import common_bp
+from src.primary.routes.media_hunt import movie_hunt_bp, tv_hunt_bp
 from src.primary.routes.plex_auth_routes import plex_auth_bp
 # Import blueprints for each app from the centralized blueprints module
 from src.primary.apps.blueprints import sonarr_bp, radarr_bp, lidarr_bp, readarr_bp, whisparr_bp, eros_bp, swaparr_bp, requestarr_bp, prowlarr_bp
@@ -50,11 +46,17 @@ from src.primary.routes.history_routes import history_blueprint
 # Import scheduler blueprint
 from src.primary.routes.scheduler_routes import scheduler_api
 
+# Import notification blueprint
+from src.primary.routes.notification_routes import notification_api
+
 # Import backup blueprint
 from src.routes.backup_routes import backup_bp
 
 # Import log routes blueprint
 from src.primary.routes.log_routes import log_routes_bp
+from src.primary.routes.nzb_hunt_routes import nzb_hunt_bp
+from src.primary.routes.indexer_hunt import indexer_hunt_bp
+from src.primary.routes.indexer_hunt.health import start_health_check_thread
 
 # Import background module to trigger manual cycle resets
 from src.primary import background
@@ -140,13 +142,11 @@ def get_base_url():
         str: The configured base URL (e.g., '/huntarr') or empty string if not configured
     """
     try:
-        base_url = settings_manager.get_setting('general', 'base_url', '')
-        # Ensure base_url always starts with a / if not empty
-        if base_url and not base_url.startswith('/'):
-            base_url = f'/{base_url}'
-        # Remove trailing slash if present
-        if base_url and base_url != '/' and base_url.endswith('/'):
-            base_url = base_url.rstrip('/')
+        base_url = settings_manager.get_setting('general', 'base_url', '').strip()
+        if not base_url or base_url == '/':
+            return ''
+        base_url = base_url.strip('/')
+        base_url = '/' + base_url
         return base_url
     except Exception as e:
         print(f"Error getting base_url from settings: {e}")
@@ -262,10 +262,41 @@ def debug_template_rendering():
     
 debug_template_rendering()
 
-app.secret_key = os.environ.get('SECRET_KEY', 'dev_key_for_sessions')
+def _get_or_create_secret_key():
+    """Get Flask secret key from env, config file, or generate a new one.
+    Persists to config so it survives container restarts but never uses a hardcoded fallback."""
+    # 1. Check environment variable (highest priority)
+    env_key = os.environ.get('SECRET_KEY')
+    if env_key and env_key != 'dev_key_for_sessions':
+        return env_key
+    # 2. Check persisted key in config directory
+    from primary.utils.config_paths import CONFIG_DIR
+    key_file = os.path.join(CONFIG_DIR, '.flask_secret_key')
+    try:
+        if os.path.exists(key_file):
+            with open(key_file, 'r') as f:
+                stored_key = f.read().strip()
+            if stored_key and len(stored_key) >= 32:
+                return stored_key
+    except Exception:
+        pass
+    # 3. Generate a new cryptographically secure key and persist it
+    import secrets as _secrets
+    new_key = _secrets.token_hex(32)  # 64-char hex = 256 bits
+    try:
+        with open(key_file, 'w') as f:
+            f.write(new_key)
+        os.chmod(key_file, 0o600)  # Owner read/write only
+    except Exception:
+        pass  # If we can't persist, use ephemeral key (sessions invalidated on restart)
+    return new_key
+
+app.secret_key = _get_or_create_secret_key()
 
 # Register blueprints
 app.register_blueprint(common_bp)
+app.register_blueprint(movie_hunt_bp)
+app.register_blueprint(tv_hunt_bp)
 app.register_blueprint(plex_auth_bp)
 app.register_blueprint(sonarr_bp, url_prefix='/api/sonarr')
 app.register_blueprint(radarr_bp, url_prefix='/api/radarr')
@@ -279,12 +310,75 @@ app.register_blueprint(requestarr_bp)
 app.register_blueprint(stateful_api, url_prefix='/api/stateful')
 app.register_blueprint(history_blueprint, url_prefix='/api/hunt-manager')
 app.register_blueprint(scheduler_api)
+app.register_blueprint(notification_api)
 app.register_blueprint(log_routes_bp)
+app.register_blueprint(nzb_hunt_bp)
+app.register_blueprint(indexer_hunt_bp)
 app.register_blueprint(backup_bp)
 
+# Start Indexer Hunt background health check (hourly + on startup)
+start_health_check_thread()
+
 # Register the authentication check to run before requests
 app.before_request(authenticate_request)
 
+# Security headers + cache control on all responses
+@app.after_request
+def _set_response_headers(response):
+    """Set security headers and cache control on all responses."""
+    # Security headers — protect against common web attacks
+    response.headers['X-Frame-Options'] = 'SAMEORIGIN'
+    response.headers['X-Content-Type-Options'] = 'nosniff'
+    response.headers['Referrer-Policy'] = 'strict-origin-when-cross-origin'
+    response.headers['Permissions-Policy'] = 'camera=(), microphone=(), geolocation=()'
+    # Content-Security-Policy: allow self + inline scripts/styles (needed for existing code)
+    # + TMDB images + CDN for Font Awesome and SortableJS
+    # + GitHub API for version checking, star counts, and sponsors
+    # + TMDB API for Requestarr/Movie Hunt detail fetches
+    response.headers['Content-Security-Policy'] = (
+        "default-src 'self'; "
+        "script-src 'self' 'unsafe-inline' https://cdn.jsdelivr.net; "
+        "style-src 'self' 'unsafe-inline' https://cdnjs.cloudflare.com; "
+        "font-src 'self' https://cdnjs.cloudflare.com; "
+        "img-src 'self' data: https://image.tmdb.org https://artworks.thetvdb.com https://*.plex.direct https://github.com https://avatars.githubusercontent.com; "
+        "connect-src 'self' https://api.github.com https://api.themoviedb.org; "
+        "frame-ancestors 'self';"
+    )
+    # HSTS — only when request came through HTTPS (reverse proxy sets X-Forwarded-Proto)
+    if request.headers.get('X-Forwarded-Proto') == 'https':
+        response.headers['Strict-Transport-Security'] = 'max-age=31536000; includeSubDomains'
+    
+    # Prevent aggressive browser caching of JS/CSS so deploys are picked up immediately
+    path = request.path or ''
+    if path.startswith('/static/') and (path.endswith('.js') or path.endswith('.css')):
+        response.headers['Cache-Control'] = 'no-cache, must-revalidate'
+    # Prevent API response caching — always fetch fresh when navigating between sections
+    if path.startswith('/api/'):
+        response.headers['Cache-Control'] = 'no-store, no-cache, must-revalidate'
+        response.headers['Pragma'] = 'no-cache'
+    return response
+
+# WSGI Middleware to handle BASE_URL stripping
+class BaseURLMiddleware:
+    """WSGI middleware to strip base URL prefix before Flask routing"""
+    def __init__(self, app):
+        self.app = app
+
+    def __call__(self, environ, start_response):
+        path = environ.get('PATH_INFO', '')
+        current_base = get_base_url()
+
+        if current_base:
+            if path.startswith(current_base + '/'):
+                environ['PATH_INFO'] = path[len(current_base):]
+            elif path == current_base:
+                environ['PATH_INFO'] = '/'
+
+        return self.app(environ, start_response)
+
+# Wrap app with BASE_URL stripping middleware
+app.wsgi_app = BaseURLMiddleware(app.wsgi_app)
+
 # Add base_url to template context so it can be used in templates
 @app.context_processor
 def inject_base_url():
@@ -307,7 +401,7 @@ def home():
 @app.route('/user')
 def user():
     """Redirect to main index with user section"""
-    return redirect('./#user')
+    return redirect(get_base_url() + url_for('home') + '#user')
     
 # This section previously contained code for redirecting paths to include the base URL
 # It has been removed as Flask's APPLICATION_ROOT setting provides this functionality
@@ -325,7 +419,7 @@ def user():
     app_type = request.args.get('app', 'all')  # Default to 'all' if no app specified
     web_logger = get_logger("web_server")
 
-    valid_app_types = list(KNOWN_LOG_FILES.keys()) + ['all']
+    valid_app_types = list(APP_LOG_FILES.keys()) + ['all']
     if app_type not in valid_app_types:
         web_logger.warning(f"Invalid app type '{app_type}' requested for logs. Defaulting to 'all'.")
         app_type = 'all'
@@ -385,22 +479,22 @@ def user():
             # Determine which log files to follow
             log_files_to_follow = []
             if app_type == 'all':
-                log_files_to_follow = list(KNOWN_LOG_FILES.items())
+                log_files_to_follow = list(APP_LOG_FILES.items())
             elif app_type == 'system':
                 # For system, only follow main log
-                system_log = KNOWN_LOG_FILES.get('system')
+                system_log = APP_LOG_FILES.get('system')
                 if system_log:
                     log_files_to_follow = [('system', system_log)]
                     web_logger.debug(f"Following system log: {system_log}")
             else:
                 # For specific app, follow that app's log
-                app_log = KNOWN_LOG_FILES.get(app_type)
+                app_log = APP_LOG_FILES.get(app_type)
                 if app_log:
                     log_files_to_follow = [(app_type, app_log)]
                     web_logger.debug(f"Following {app_type} log: {app_log}")
                 
                 # Also include system log for related messages
-                system_log = KNOWN_LOG_FILES.get('system')
+                system_log = APP_LOG_FILES.get('system')
                 if system_log:
                     log_files_to_follow.append(('system', system_log))
                     web_logger.debug(f"Also following system log for {app_type} messages")
@@ -563,9 +657,27 @@ def user():
 def api_settings():
     if request.method == 'GET':
         # Return all settings using the new manager function
-        all_settings = settings_manager.get_all_settings() # Corrected function name
+        all_settings = settings_manager.get_all_settings()
+        # Effective timezone (env TZ overrides settings) for logs, scheduling, Hunt Manager display
+        try:
+            from src.primary.utils.timezone_utils import get_timezone_name
+            if 'general' not in all_settings:
+                all_settings['general'] = {}
+            all_settings['general']['effective_timezone'] = get_timezone_name()
+        except Exception:
+            pass
         return jsonify(all_settings)
 
+@app.route('/api/settings/feature-flags', methods=['GET'])
+def api_feature_flags():
+    """Return lightweight feature flag settings for sidebar visibility."""
+    general = settings_manager.load_settings('general') or {}
+    return jsonify({
+        'enable_requestarr': general.get('enable_requestarr', True),
+        'enable_media_hunt': general.get('enable_media_hunt', True),
+        'enable_third_party_apps': general.get('enable_third_party_apps', True),
+    })
+
 @app.route('/api/settings/general', methods=['POST'])
 def save_general_settings():
     general_logger = get_logger("web_server")
@@ -593,6 +705,8 @@ def save_general_settings():
         elif auth_mode == 'no_login':
             data['local_access_bypass'] = False
             data['proxy_auth_bypass'] = True
+            # When switching to No Login Mode, clear any existing session to ensure UI refreshes
+            session.pop(SESSION_COOKIE_NAME, None)
         elif auth_mode == 'login':
             data['local_access_bypass'] = False
             data['proxy_auth_bypass'] = False
@@ -616,8 +730,12 @@ def save_general_settings():
             timezone_changed = True
             general_logger.info(f"Timezone changed from {current_timezone} to {new_timezone}")
     
-    # Save general settings
-    success = settings_manager.save_settings('general', data)
+    # Merge incoming data with current general settings so we never drop keys (e.g. show_trending)
+    current = settings_manager.load_settings('general')
+    merged = {**current, **data}
+    # Update checking is always enabled; ensure we persist True even if client omits the key
+    merged['check_for_updates'] = True
+    success = settings_manager.save_settings('general', merged)
     
     if success:
         # Apply timezone change if needed
@@ -655,6 +773,15 @@ def save_general_settings():
         # Update logging levels immediately when general settings are changed
         update_logging_levels()
         
+        # Check if log rotation settings changed and refresh handlers
+        log_keys = ["log_rotation_enabled", "log_max_size_mb", "log_backup_count"]
+        if any(key in data for key in log_keys):
+            try:
+                refresh_log_handlers()
+                general_logger.info("Log handlers refreshed due to rotation settings change")
+            except Exception as e:
+                general_logger.error(f"Error refreshing log handlers: {e}")
+        
         # Return all settings
         return jsonify(settings_manager.get_all_settings())
     else:
@@ -662,53 +789,34 @@ def save_general_settings():
 
 @app.route('/api/test-notification', methods=['POST'])
 def test_notification():
-    """Test notification endpoint with enhanced Windows debugging"""
+    """Legacy test notification endpoint — kept for backward compatibility."""
     import platform
     web_logger = get_logger("web_server")
     
     try:
-        from src.primary.notification_manager import send_notification, get_notification_config, apprise_import_error
+        from src.primary.notification_manager import send_notification, get_notification_config
         
-        # Enhanced debugging for Windows issues
         system_info = {
             "platform": platform.system(),
-            "platform_release": platform.release(),
             "python_version": platform.python_version(),
-            "apprise_available": apprise_import_error is None
         }
         
         web_logger.info(f"Test notification requested on {system_info}")
         
-        # Check for Apprise import issues first (common Windows problem)
-        if apprise_import_error:
-            error_msg = f"Apprise library not available: {apprise_import_error}"
-            if platform.system() == "Windows":
-                error_msg += " (Common on Windows - try: pip install apprise)"
-            web_logger.error(error_msg)
-            return jsonify({
-                "success": False, 
-                "error": error_msg,
-                "system_info": system_info
-            }), 500, {'Content-Type': 'application/json'}
-        
-        # Get the user's configured notification level
         config = get_notification_config()
         user_level = config.get('level', 'info')
         
-        # Send a test notification using the user's configured level
         success = send_notification(
-            title="🧪 Huntarr Test Notification",
-            message="This is a test notification to verify your Apprise configuration is working correctly! If you see this, your notifications are set up properly. 🎉",
+            title="Huntarr Test Notification",
+            message="This is a test notification to verify your configuration is working correctly! If you see this, your notifications are set up properly.",
             level=user_level
         )
         
         if success:
-            web_logger.info(f"Test notification sent successfully on {platform.system()}")
+            web_logger.info("Test notification sent successfully")
             return jsonify({"success": True, "message": "Test notification sent successfully!"}), 200, {'Content-Type': 'application/json'}
         else:
-            error_msg = "Failed to send test notification. Check your Apprise URLs and settings."
-            if platform.system() == "Windows":
-                error_msg += " On Windows, ensure Apprise is properly installed and all dependencies are available."
+            error_msg = "Failed to send test notification. Check your notification connections."
             web_logger.warning(f"Test notification failed: {error_msg}")
             return jsonify({
                 "success": False, 
@@ -765,8 +873,9 @@ def handle_app_settings(app_name):
         success = settings_manager.save_settings(app_name, data)
         
         if success:
-            # Auto-save enabled - no need to log every successful save
-            return jsonify({"success": True})
+            # Return updated settings so client can show server-generated fields (e.g. instance_id for new instances)
+            updated = settings_manager.load_settings(app_name)
+            return jsonify({"success": True, "settings": updated})
         else:
             web_logger.error(f"Failed to save {app_name} settings")
             return jsonify({"success": False, "error": f"Failed to save {app_name} settings"}), 500
@@ -844,7 +953,7 @@ def api_app_status(app_name):
         return jsonify({"configured": False, "connected": False, "error": "Invalid app name"}), 400
     
     try:
-        if app_name in ['sonarr', 'radarr', 'lidarr', 'readarr', 'whisparr', 'eros']:
+        if app_name in ['sonarr', 'radarr', 'lidarr', 'readarr', 'whisparr', 'eros', 'movie_hunt', 'tv_hunt']:
             # --- Multi-Instance Status Check --- # 
             connected_count = 0
             total_configured = 0
@@ -856,7 +965,7 @@ def api_app_status(app_name):
                 
                 if hasattr(instances_module, 'get_configured_instances'):
                     get_instances_func = getattr(instances_module, 'get_configured_instances')
-                    instances = get_instances_func()
+                    instances = get_instances_func(quiet=True)
                     total_configured = len(instances)
                     api_timeout = settings_manager.get_setting(app_name, "api_timeout", 10) # Get global timeout
                     
@@ -865,12 +974,12 @@ def api_app_status(app_name):
                         if hasattr(api_module, 'check_connection'):
                             check_connection_func = getattr(api_module, 'check_connection')
                             for instance in instances:
-                                inst_url = instance.get("api_url")
-                                inst_key = instance.get("api_key")
+                                inst_url = instance.get("api_url") or ""
+                                inst_key = instance.get("api_key") or ""
                                 inst_name = instance.get("instance_name", "Default")
                                 try:
-                                    # Use a short timeout per instance check
-                                    if check_connection_func(inst_url, inst_key, min(api_timeout, 5)):
+                                    # Use a short timeout per instance check (Movie Hunt has no API URL/key; check_connection returns True)
+                                    if check_connection_func(inst_url, inst_key, min(api_timeout or 10, 5)):
                                         web_logger.debug(f"{app_name.capitalize()} instance '{inst_name}' connected successfully.")
                                         connected_count += 1
                                     else:
@@ -956,29 +1065,15 @@ def apply_timezone_setting():
 
 @app.route('/api/hourly-caps', methods=['GET'])
 def api_get_hourly_caps():
-    """Get hourly API usage caps for each app"""
+    """Get hourly API usage caps for each app (per-instance when app has multiple instances)."""
     try:
-        # Import necessary functions
-        from src.primary.stats_manager import load_hourly_caps
-        from src.primary.settings_manager import load_settings
-        
-        # Get the logger
+        from src.primary.stats_manager import load_hourly_caps_for_api
         web_logger = get_logger("web_server")
-        
-        # Load the current hourly caps
-        caps = load_hourly_caps()
-        
-        # Get app-specific hourly cap limits
-        app_limits = {}
-        apps = ['sonarr', 'radarr', 'lidarr', 'readarr', 'whisparr', 'eros']
-        for app in apps:
-            app_settings = load_settings(app)
-            app_limits[app] = app_settings.get('hourly_cap', 20)  # Default to 20 if not set
-        
+        caps, limits = load_hourly_caps_for_api()
         return jsonify({
             "success": True,
             "caps": caps,
-            "limits": app_limits
+            "limits": limits
         })
     except Exception as e:
         web_logger = get_logger("web_server")
@@ -1031,6 +1126,19 @@ def version_txt():
         web_logger.error(f"Error serving version from database: {e}")
         return "N/A", 200, {'Content-Type': 'text/plain', 'Cache-Control': 'no-cache'}
 
+@app.route('/api/version')
+def api_version():
+    """Unauthenticated API endpoint to get current app version as JSON"""
+    try:
+        from src.primary.utils.database import get_database
+        db = get_database()
+        version = db.get_version()
+        return jsonify({"version": version.strip()}), 200
+    except Exception as e:
+        web_logger = get_logger("web_server")
+        web_logger.error(f"Error serving version from database: {e}")
+        return jsonify({"version": "N/A"}), 200
+
 @app.route('/api/cycle/status', methods=['GET'])
 def api_get_all_cycle_status():
     """API endpoint to get cycle status for all apps."""
@@ -1071,7 +1179,7 @@ def reset_app_cycle(app_name):
     web_logger.info(f"Manual cycle reset requested for {app_name} via API")
     
     # Check if app name is valid
-    if app_name not in ['sonarr', 'radarr', 'lidarr', 'readarr', 'whisparr', 'eros', 'swaparr']:
+    if app_name not in ['sonarr', 'radarr', 'lidarr', 'readarr', 'whisparr', 'eros', 'swaparr', 'movie_hunt', 'tv_hunt']:
         return jsonify({
             'success': False,
             'error': f"Invalid app name: {app_name}"
@@ -1097,14 +1205,34 @@ def reset_app_cycle(app_name):
             }), 400
         
     try:
-        # Trigger cycle reset using database
         from src.primary.utils.database import get_database
-        
+        # Per-instance reset for *arr apps (optional instance_name query param; resolve to instance_id for DB)
+        instance_name = request.args.get('instance_name')
+        if instance_name is None and request.is_json:
+            try:
+                j = request.get_json(silent=True)
+                instance_name = j.get('instance_name') if j else None
+            except Exception:
+                pass
+        if app_name == 'swaparr':
+            instance_name = None
+        instance_identifier = instance_name
+        # Resolve display name to stable instance_id so reset is keyed correctly
+        if instance_name and app_name != 'swaparr':
+            try:
+                app_module = __import__(f"src.primary.apps.{app_name}", fromlist=["get_configured_instances"])
+                get_instances = getattr(app_module, "get_configured_instances", None)
+                if get_instances:
+                    for inst in get_instances(quiet=True):
+                        if inst.get("instance_name") == instance_name:
+                            instance_identifier = inst.get("instance_id") or instance_name
+                            break
+            except Exception:
+                pass
         db = get_database()
-        success = db.create_reset_request(app_name)
-        
+        success = db.create_reset_request(app_name, instance_name=instance_identifier)
         if success:
-            web_logger.info(f"Created reset request for {app_name}")
+            web_logger.info(f"Created reset request for {app_name}" + (f" instance {instance_name or instance_identifier}" if instance_identifier else ""))
         else:
             web_logger.error(f"Failed to create reset request for {app_name}")
     except Exception as e:
@@ -1142,7 +1270,6 @@ def api_health_check():
     This endpoint is useful for monitoring tools and load balancers.
     """
     logger = get_logger("system")
-    logger.debug("API health check endpoint accessed")
     return jsonify({"status": "OK", "message": "Huntarr is running"})
 
 @app.route('/api/github_sponsors', methods=['GET'])
@@ -1221,7 +1348,10 @@ def start_web_server():
     web_logger.info("--- start_web_server function called ---") # Added log
     debug_mode = os.environ.get('DEBUG', 'false').lower() == 'true'
     host = '0.0.0.0'  # Listen on all interfaces
-    port = int(os.environ.get('PORT', 9705))
+    try:
+        port = int(os.environ.get('PORT', 9705))
+    except (TypeError, ValueError):
+        port = 9705
 
     # Ensure the log directory exists
     os.makedirs(LOG_DIR, exist_ok=True)
