@@ -260,7 +260,7 @@ def is_admin():
     """Check if the script is running with administrator privileges"""
     try:
         return ctypes.windll.shell32.IsUserAnAdmin() != 0
-    except:
+    except Exception:
         return False
 
 def install_service():
@@ -397,7 +397,7 @@ def install_service():
             # Clean up the batch file
             try:
                 os.unlink(batch_file)
-            except:
+            except OSError:
                 pass
                 
             # Check for errors
